/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <stdio.h>
#include <string.h>

void cli_help(const char* topic)
{

  if (strcmp (topic, "ls") == 0)
    {
      fprintf (stderr,
	       "ls - list (or print) soundsources\n\n"
	       "Synopsis:\n"
	       "\tls [-a] source_id [source_id ...]\n\n"
	       "The source_id's may be those returned by\n"
	       "subsource creation routines, or may be determined\n"
	       "by `ls' or other means.\n\n"
	       "Options:\n"
	       "\t-R   : recurse -- print the entire tree.\n"
	       "\t       usually `ls' shows just one layer: this\n"
	       "\t       makes it ascend the entire tree.\n");
      return;
    }
  if (strcmp (topic, "cache") == 0)
    {
      fprintf (stderr,
	       "cache - cache a source into a buffer (waiting)\n"
	       "\n"
	       "Synopsis:\n"
	       "\tcache buffer_name source_id\n\n"
	       "This caches source_id, leaving it at end-of-file.\n"
	       "The buffer_name is a variable you can subsequently\n"
	       "use to refer to the buffer.\n"
	       "\n"
	       "eg.\n"
	       "\tcache buf $mp3 start 0L length 44100L\n"
	       "\tbuffer_call $buf save filename \"file.wav\"\n"
	       "\tunref_buffer $buf\n");
      return;
    }

  if (strcmp (topic, "unref_buffer") == 0)
    {
      fprintf (stderr,
	       "unref_buffer - unref a cache buffer\n"
	       "\n"
	       "Detach a reference we are holding.\n"
	       "Clients may not delete eachother's references!\n");
      return;
    }

  if (strcmp (topic, "unref_source") == 0)
    {
      fprintf (stderr,
	       "unref_source - unref a source\n"
	       "\n"
	       "Detach a reference we are holding.\n"
	       "Clients may not delete eachother's references!\n");
      return;
    }

  if (strcmp (topic, "silence") == 0)
    {
      fprintf (stderr,
	       "silence - silence a mixer.\n"
	       "\n"
	       "Synopsis:\n\n"
	       "\tsilence [mixer_id]\n"
	       "Defaults to the mixer for the default device.\n");
      return;
    }

  if (strcmp (topic, "mixer") == 0)
    {
      fprintf (stderr,
	       "mixer - return the default mixer id number.\n"
	       "\n"
	       "Get the mixer id.\n"
	       "\n"
	       "This is the source id to `ls' and the default mix_id\n"
	       "for `add', but it isn't that much use...\n");
      return;
    }

  if (strcmp (topic, "device") == 0)
    {
      fprintf (stderr,
	       "device - return the default device id number.\n"
	       "\n" "Get the device id.\n");
      return;
    }

  if (strcmp (topic, "source") == 0)
    {
      fprintf (stderr,
	       "source - create a sound source.\n\n"
	       "Synopsis:\n"
	       "\tsource name GdamSourceType arg1 value1 ...\n\n"
	       "TODO: finish writing.\n");
      return;
    }
  if (strcmp (topic, "set") == 0)
    {
      fprintf (stderr,
	       "set - set a source's arguments.\n\n"
	       "Synopsis:\n"
	       "\tset [source_id|$variable_name] arg1 value1 ...\n\n"
	       "TODO: finish writing.\n");
      return;
    }
  if (strcmp (topic, "detach") == 0)
    {
      fprintf (stderr,
	       "detach - orphan a sound source.\n\n"
	       "Synopsis:\n"
	       "\tdetach id\n\n"
	       "This causes the source id to stop feeding whatever\n"
	       "it is feeding.  It usually dies thereafter (unless\n"
	       "it is not attached or is still otherwise\n" "accessible)\n");
      return;
    }
  if (strcmp (topic, "add") == 0)
    {
      fprintf (stderr,
	       "add - add a source to a mixer\n\n"
	       "Synopsis:\n"
	       "\tadd source_id [mixer_id]\n\n"
	       "Add the given source to the mixer (or the default\n"
	       "mixer if no mix is specified).\n\n");
      return;
    }
  if (strcmp (topic, "help") == 0)
    {
      fprintf (stderr,
	       "help - online help\n\n"
	       "Synopsis:\n"
	       "\thelp [topic]\n\n"
	       "If no topic is specified, " "a list of topics is printed.\n");
      return;
    }
  if (strcmp (topic, "call") == 0)
    {
      fprintf (stderr,
	       "call - call a \"function\" on a source\n"
	       "Synopsis:\n"
	       "\tcall source_id function_name arg_name_1 val_1 ...\n"
	       "\n"
	       "This calls a source-defined function.\n"
	       "Currently there is no way to find a list of available "
	       "functions,\n"
	       "except to read the source code.\n"
	       "\n"
	       "The `function_name' is the registered name\n"
	       "of the function.  And any number of named arguments\n"
	       "may follow.\n");
      return;
    }
  if (strcmp (topic, "buffer_call") == 0)
    {
      fprintf (stderr,
	       "buffer_call - call a \"function\" on a buffer\n"
	       "Synopsis:\n"
	       "\tbuffer_call source_id function_name "
	       "arg_name_1 val_1 ...\n"
	       "\n"
	       "This calls a cachebuffer-defined function.\n"
	       "Currently there is no way to find a list of available "
	       "functions,\n"
	       "except to read the source code.\n"
	       "\n"
	       "The `function_name' is the registered name\n"
	       "of the function.  And any number of named arguments\n"
	       "may follow.\n");
      return;
    }
}
