/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdambind.h"
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <netdb.h>

#ifndef UNIX_PATH_MAX
#define UNIX_PATH_MAX	108
#endif

int gdam_bind_tcp(int port_number, gboolean localhost_only)
{
  struct sockaddr_in sin;
  int rv;
  int one = 1;
  bzero (&sin, sizeof (sin));
  sin.sin_family = AF_INET;
  sin.sin_port = htons (port_number);
  /*sin.sin_addr = htonl(INADDR_ANY); */
  rv = socket (PF_INET, SOCK_STREAM, 0);
  if (rv < 0)
    {
      perror ("error creating listening socket??");
      return -1;
    }
  fcntl (rv, F_SETFD, FD_CLOEXEC);
  if (setsockopt (rv, SOL_SOCKET, SO_REUSEADDR, &one, sizeof (one)) < 0)
    perror ("warning: couldn't make sockaddr reuseable");
  if (localhost_only)
    {
      struct hostent *hent;
      hent = gethostbyname ("localhost");
      memcpy (&sin.sin_addr, hent->h_addr_list[0], hent->h_length);
    }
  if (bind (rv, (struct sockaddr *) (&sin), sizeof (struct sockaddr)) < 0)
    {
      fprintf (stderr, "bind: failed to bind to port %d: %s.\n\n",
	       port_number, strerror (errno));
      if (port_number < 1024 && getuid ())
	{
	  fprintf (stderr, "A likely reason is that only root "
		   "may bind to low-numbered ports.\n");
	}
      close (rv);
      return -1;
    }
  if (listen (rv, SOMAXCONN) < 0)
    {
      fprintf (stderr, "error listening: %s\n", strerror (errno));
      close (rv);
      return -1;
    }
  return rv;
}
int gdam_bind_unix(const char* unix_path)
{
  struct sockaddr_un un_addr;
  int rv;
  bzero (&un_addr, sizeof (un_addr));
  un_addr.sun_family = AF_UNIX;
  strncpy (un_addr.sun_path, unix_path, UNIX_PATH_MAX);
  rv = socket (PF_UNIX, SOCK_STREAM, 0);
  if (rv < 0)
    {
      perror ("error creating listening socket??");
      return -1;
    }
  fcntl (rv, F_SETFD, FD_CLOEXEC);
  if (bind (rv, (struct sockaddr *) (&un_addr), sizeof (struct sockaddr)) < 0)
    {
      fprintf (stderr, "bind: failed to bind to %s: %s.\n\n",
	       unix_path, strerror (errno));
      close (rv);
      return -1;
    }
  if (listen (rv, SOMAXCONN) < 0)
    {
      fprintf (stderr, "error listening: %s\n", strerror (errno));
      close (rv);
      return -1;
    }
  return rv;
}
