/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamsample.h"

/* For convenience below */
#define SAMPLE_MIN     4 * GDAM_SAMPLE_MIN
#define SAMPLE_MAX     4 * GDAM_SAMPLE_MAX
#define SAMPLE_SIGNBIT GDAM_SAMPLE_SIGNBIT

void gdam_sample_scale(GdamSample* regain, int nsample, GdamFixed factor)
{
  GdamSampleValue *s = (GdamSampleValue *) regain;
  nsample += nsample;
  if (-GdamFixed_ONE < factor && factor < GdamFixed_ONE)
    {
      while (nsample--)
	{
	  *s = (((GdamSampleProduct) * s) * factor) / GdamFixed_ONE;
	  s++;
	}
    }
  else
    {
      while (nsample--)
	{
	  int i = (int) ((GdamSampleProduct) (*s) * factor / GdamFixed_ONE);
	  if (i < SAMPLE_MIN)
	    *s++ = SAMPLE_MIN;
	  else if (i > SAMPLE_MAX)
	    *s++ = SAMPLE_MAX;
	  else
	    *s++ = (GdamSampleValue) i;
	}
    }
}

void gdam_sample_addto(GdamSample* accum, const GdamSample* term, int len)
{
  const GdamSampleValue *s = (const GdamSampleValue *) term;
  GdamSampleValue *d = (GdamSampleValue *) accum;
  len += len;
  while (len--)
    {
#if 0
      int sign_s = SAMPLE_SIGNBIT & (*s);
      int sign_d = SAMPLE_SIGNBIT & (*d);
      /*
       * If the sign of accumulator and term are different,
       * no possibility of overflow.
       *
       * If they are the same, overflow occurs iff the sum
       * has a different sign.
       */
      if (sign_s ^ sign_d)
	*d += *s;
      else
	{
	  if (((*d += *s) & SAMPLE_SIGNBIT) ^ sign_s)
	    *d = (sign_s) ? SAMPLE_MIN : SAMPLE_MAX;
	}
#else
      int new_val = (int) (*s) + (*d);
      if (new_val < SAMPLE_MIN)
	new_val = SAMPLE_MIN;
      else if (new_val > SAMPLE_MAX)
	new_val = SAMPLE_MAX;
      *d = (new_val);
#endif
      d++;
      s++;
    }
}

void gdam_sample_add_scaled(GdamSample* accum, const GdamSample* term,
			    int len, GdamFixed factor)
{
  const GdamSampleValue *s = (const GdamSampleValue *) term;
  GdamSampleValue *d = (GdamSampleValue *) accum;
  int shift_right;
  if (len <= 0)
    return;
  len += len;
  shift_right = GdamFixed_LOG_ONE;
  if (factor < 0)
    {
      while (factor < -GdamFixed_ONE)
	{
	  factor /= 2;
	  shift_right--;
	}
    }
  else
    {
      while (factor > GdamFixed_ONE)
	{
	  factor /= 2;
	  shift_right--;
	}
    }
  while (len--)
    {
      GdamSampleProduct t = *s;
      t *= (GdamSampleProduct) factor;
      t >>= shift_right;
      t += *d;
      if (t < SAMPLE_MIN)
	*d = SAMPLE_MIN;
      else if (t > SAMPLE_MAX)
	*d = SAMPLE_MAX;
      else
	*d = (GdamSampleValue) t;
      s++;
      d++;
    }
}

void gdam_sample_copy_scaled(GdamSample* accum, const GdamSample* term,
			    int len, GdamFixed factor)
{
  gdam_sample_copy (accum, term, len);
  if (factor != GdamFixed_ONE)
    gdam_sample_scale (accum, len, factor);
}

void gdam_sample_modulate(GdamSample* dst, const GdamSample* m,int len)
{
  GdamSampleValue *d = (GdamSampleValue *) dst;
  const GdamSampleValue *s = (const GdamSampleValue *) m;
  len += len;
  while (len--)
    {
      GdamSampleProduct a = *d;
      a *= *s++;
      *d++ = (GdamSampleValue) (a >> GDAM_SAMPLE_LOG);
    }
}

#ifdef gdam_sample_copy
#undef gdam_sample_copy
#endif

void gdam_sample_copy(GdamSample* dst, const GdamSample* src, int len)
{
  memcpy (dst, src, len * sizeof (GdamSample));
}
