#! /usr/bin/perl -w


#=====================================================================
#            PART ZERO --  Preamble.
#=====================================================================

print <<'EOF';
/*
 * Generated by make-gdamsampleconverter.pl.
 */

#include "gdamsampleconverter.h"
#include <common/gdamsample.h>
EOF


#=====================================================================
#            PART ONE  --  Write the int-to-int sample converters
#=====================================================================

# Describe the sample types that we will convert to and from.
#
# The follow are the byte-by-byte layouts
#                       [hence endian-independant code results]
#
# A - signed high byte
# a - unsigned high byte
# b - byte 2
# c - byte 3
# S - sign extend the high-byte [must include 'A' in string]
# 0 - ignore on input; zeroed on output.
#
%TYPES = (
	'uint8'		=> 'a',
	'int8'		=> 'A',
	'uint16_be'	=> 'ab',
	'int16_be'	=> 'Ab',
	'uint16_le'	=> 'ba',
	'int16_le'	=> 'bA',
	'uint24_be'	=> '0abc',
	'int24_be'	=> 'SAbc',
	'uint24_le'	=> 'cba0',
	'int24_le'	=> 'cbAS',
	'uint32_be'	=> 'abcd',
	'int32_be'	=> 'Abcd',
	'uint32_le'	=> 'dcba',
	'int32_le'	=> 'dcbA',
);

sub write_converter($$) {
	my ($src, $dst) = @_;
	my $src_spec = $TYPES{$src};
	my $dst_spec = $TYPES{$dst};
	my $forward = (length($src_spec) >= length($dst_spec));
	print "static void convert_", $src, "_to_", $dst, "\n";
	print "                        (void*       dst_ptr,\n";
	print "                         const void* src_ptr,\n";
	print "                         int         num_samples)\n";
	if ($src eq $dst) {
		print "{\n";
		print "\tif (dst_ptr == src_ptr)\n";
		print "\t\treturn;\n";
		print "\tg_memmove(dst_ptr, src_ptr, num_samples * ",
			length($src_spec), ");\n";
		print "}\n";
		return;
	}
	print "{\n";

	print "\tconst unsigned char* src;\n";
	print "\tunsigned char* dst;\n";
	print "\tint i;\n";
	print "\tsrc = src_ptr;\n";
	print "\tdst = dst_ptr;\n";
	if ($forward) {
		print "\tfor (i = 0; i < num_samples; i++) {\n";
	} else {
		print "\tsrc += (num_samples - 1) * ", length($src_spec), ";\n";
		print "\tdst += (num_samples - 1) * ", length($dst_spec), ";\n";
		print "\tfor (i = 0; i < num_samples; i++) {\n";
	}

	@dst_vars = split //, $dst_spec;
	$needs_S = 0;

	for (@dst_vars) {
		next if ($_ eq '0');
		if ($_ eq 'S') {
			$needs_S = 1;
			next;
		}
		print "\t\tunsigned char $_ = ";
		if (($_ eq 'a') || ($_ eq 'A')) {
			my $index = index($src_spec, $_);
			if ($index >= 0) {
				print "src[$index]";
			} else {
				$index = index($src_spec, ($_ eq 'a')?'A':'a');
				die if $index == -1;
				print "src[$index] ^ 0x80";
			}
		} elsif (($_ eq 'b') || ($_ eq 'c') || ($_ eq 'd')) {
			my $index = index($src_spec, $_);
			if ($index == -1) {
				print "0";
			} else {
				print "src[$index]";
			}
		}
		print ";\n";
	}

	if ($needs_S) {
		print "\t\tunsigned char S = (0x80 & A) ? 0xff : 0x00;\n";
	}

	for ($i = 0; $i < scalar(@dst_vars); $i++) {
		$_ = $dst_vars[$i];
		print "\t\tdst[$i] = $_;\n";
	}

	if ($forward) {
		print "\t\tsrc += ", length($src_spec), ";\n";
		print "\t\tdst += ", length($dst_spec), ";\n";
	} else {
		print "\t\tsrc -= ", length($src_spec), ";\n";
		print "\t\tdst -= ", length($dst_spec), ";\n";
	}
	print "\t}\n";
	print "}\n";
}

@keys = sort keys %TYPES;
for $a (@keys) {
	for $b (@keys) {
		write_converter($a, $b);
	}
}

print "\nstatic GdamSampleConverter sample_converters[",
	scalar(@keys) * scalar(@keys),
	"] = {\n";

for $a (@keys) {
	for $b (@keys) {
		print "\tconvert_${a}_to_${b},\n";
	}
}

print "};\n\n";

print "enum {\n";
for (@keys) {
	print "\tSAMPLE_INDEX_$_,\n";
}
print "LAST_SAMPLE_INDEX\n";
print "};\n\n";

print <<"EOF";
static inline int get_sample_type_index(int bits_per_sample,
                                        int is_signed,
				        int big_endian)
{
	if (bits_per_sample == 8)
		return is_signed ? SAMPLE_INDEX_int8 : SAMPLE_INDEX_uint8;

	if (bits_per_sample == 16)
		return big_endian
		   ? (is_signed?SAMPLE_INDEX_int16_be:SAMPLE_INDEX_uint16_be)
		   : (is_signed?SAMPLE_INDEX_int16_le:SAMPLE_INDEX_uint16_le);

	if (bits_per_sample == 24)
		return big_endian
		   ? (is_signed?SAMPLE_INDEX_int24_be:SAMPLE_INDEX_uint24_be)
		   : (is_signed?SAMPLE_INDEX_int24_le:SAMPLE_INDEX_uint24_le);

	if (bits_per_sample == 32)
		return big_endian
		   ? (is_signed?SAMPLE_INDEX_int32_be:SAMPLE_INDEX_uint32_be)
		   : (is_signed?SAMPLE_INDEX_int32_le:SAMPLE_INDEX_uint32_le);
	return -1;
}

GdamSampleConverter  gdam_sample_converter_get(int      in_bits_per_samp,
					       gboolean in_signed,
					       gboolean in_big_endian,
                                               int      out_bits_per_samp,
					       gboolean out_signed,
					       gboolean out_big_endian)
{
	int src_type_index, dst_type_index;
	int index;

	src_type_index = get_sample_type_index(in_bits_per_samp,
	                                       in_signed,
	                                       in_big_endian);
	dst_type_index = get_sample_type_index(out_bits_per_samp,
	                                       out_signed,
	                                       out_big_endian);
	
	if (src_type_index < 0 || dst_type_index < 0)
		return NULL;

	index = src_type_index * LAST_SAMPLE_INDEX + dst_type_index;
	return sample_converters[index];
}
EOF

#=====================================================================
#            PART TWO  --  Write the GdamSampleValue channel converters
#=====================================================================
$LAST_SUPPORTED_CHANNEL = 4;

sub write_channel_converter($$) {
	my ($num_out, $num_in) = @_;
	my $forward = ($num_in >= $num_out);
	print "static void convert_channels_", $num_in, "_to_", $num_out, "\n";
	print "                        (void*       dst_ptr,\n";
	print "                         const void* src_ptr,\n";
	print "                         int         num_samples)\n";

	if ($num_in == $num_out) {
		print "{\n";
		print "\tif ((char*) dst_ptr != (char*) src_ptr)\n";
		print "\t\tg_memmove(dst_ptr, src_ptr,\n";
		print "\t\t\tnum_samples * $num_in * sizeof(GdamSampleValue));\n";
		print "}\n";
		return;
	}

	print "{\n";
	print "\tGdamSampleValue* src = (GdamSampleValue*) src_ptr;\n";
	print "\tGdamSampleValue* dst = (GdamSampleValue*) dst_ptr;\n";
	print "\tint i;\n";
	if ($forward) {
		print "\tfor (i = 0; i < num_samples; i++) {\n";
	} else {
		print "\tsrc += (num_samples - 1) * $num_in;\n";
		print "\tdst += (num_samples - 1) * $num_out;\n";
		print "\tfor (i = 0; i < num_samples; i++) {\n";
	}


	@COUNTS = ();
	for ($i = 0; $i < $num_in; $i++) {
		if ($i < $num_out) {
			print "\t\tint out$i = src[$i];\n";
			$COUNTS[$i] = 1;
		} else {
			print "\t\tout", ($i % $num_out), " += src[$i];\n";
			$COUNTS[$i % $num_out]++;
		}
	}
	for ($i = 0; $i < $num_out; $i++) {
		if ($i >= $num_in) {
			;
		} elsif ($COUNTS[$i] > 1) {
			print "\t\tout$i /= $COUNTS[$i];\n";
		}
	}
	for ($i = 0; $i < $num_out; $i++) {
		print "\t\tdst[$i] = out", ($i % $num_in), ";\n";
	}

	if ($forward) {
		print "\t\tsrc += $num_in;\n";
		print "\t\tdst += $num_out;\n";
	} else {
		print "\t\tsrc -= $num_in;\n";
		print "\t\tdst -= $num_out;\n";
	}
	print "\t}\n";

	print "}\n";
	print "\n";
}

for ($a = 1; $a <= $LAST_SUPPORTED_CHANNEL; $a++) {
	for ($b = 1; $b <= $LAST_SUPPORTED_CHANNEL; $b++) {
		write_channel_converter($a, $b);
	}
}
print "static GdamSampleConverter channel_converters[] = {\n";
for ($a = 1; $a <= $LAST_SUPPORTED_CHANNEL; $a++) {
	for ($b = 1; $b <= $LAST_SUPPORTED_CHANNEL; $b++) {
		print "\tconvert_channels_${a}_to_${b},\n";
	}
}
print "};\n\n";

print <<"EOF";
GdamSampleConverter  gdam_sample_converter_get_channel_changer
                                              (int      num_channels_in,
					       int      num_channels_out)
{
	int src_type_index, dst_type_index;
	int index;

	if (num_channels_in < 1 || num_channels_in > $LAST_SUPPORTED_CHANNEL)
		return NULL;
	if (num_channels_out < 1 || num_channels_out > $LAST_SUPPORTED_CHANNEL)
		return NULL;

	src_type_index = num_channels_in - 1;
	dst_type_index = num_channels_out - 1;
	
	index = src_type_index * $LAST_SUPPORTED_CHANNEL + dst_type_index;
	return channel_converters[index];
}
EOF

