/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdambeatinfo.h"
#include <glib.h>


GdamBeatInfo*	gdam_beat_info_new	()
{
  return g_new0 (GdamBeatInfo, 1);
}

double		gdam_beat_info_get_bpm	(const GdamBeatInfo*	beat_info,
				       	 float			position)
{
  return beat_info ? beat_info->abpm : 0;
}

double		gdam_beat_info_get_first_beat	
					(const GdamBeatInfo*	beat_info,
				       	 float			position)
{
  return beat_info ? beat_info->afirst_beat : 0;
}

double		gdam_beat_info_get_beat_length	
					(const GdamBeatInfo*	beat_info,
				       	 float			position)
{
  gdouble rv = beat_info ? (60.0 / beat_info->abpm) : 0;
  if (rv < 0.01)
    {
      g_warning ("beat_length unreasonably small...");
      return 0;
    }
  return rv;
}

void		gdam_beat_info_set_bpm	(GdamBeatInfo*		beat_info,
					 double		new_bpm)
{
  g_return_if_fail (beat_info != NULL);
  beat_info->abpm = new_bpm;
}

void		gdam_beat_info_set_first_beat	
					(GdamBeatInfo*		beat_info,
					 double		new_first_beat)
{
  g_return_if_fail (beat_info != NULL);
  beat_info->afirst_beat = new_first_beat;
}

static void copy_one_bir(gpointer data, gpointer udata)
{
  GdamBeatInfoRange *bir;
  GdamBeatInfoRange *newbir;
  GList **newrl;
  //g_return_if_fail(data != NULL);
  //g_return_if_fail(udata != NULL);
  bir = (GdamBeatInfoRange *) data;
  newrl = (GList **) (udata);

  newbir = g_new0 (GdamBeatInfoRange, 1);
  *newbir = *bir;
  *newrl = g_list_append (*newrl, newbir);
}

GdamBeatInfo*	gdam_beat_info_copy	(const GdamBeatInfo*	beat_info)
{
  GdamBeatInfo *rv = g_new0 (GdamBeatInfo, 1);
  GList *new_rl = NULL;
  g_return_val_if_fail (beat_info != NULL, NULL);
  rv->abpm = beat_info->abpm;
  rv->afirst_beat = beat_info->afirst_beat;
  if (beat_info->range_list)
    {
      g_list_foreach (beat_info->range_list, copy_one_bir, &new_rl);
      rv->range_list = new_rl;
    }
  return rv;
}

void free_range(gpointer data, gpointer udata){
	GdamBeatInfoRange* bir;
	bir = (GdamBeatInfoRange*)data;
	/* TODO: need to free this, but it is causing a seg fault...*/
	g_free(bir);
}

void		gdam_beat_info_destroy	(GdamBeatInfo*		beat_info)
{
  if (beat_info->range_list)
    {
      g_list_foreach (beat_info->range_list, free_range, NULL);
      g_list_free (beat_info->range_list);
      beat_info->range_list = NULL;
    }
  g_free (beat_info);
}


