/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include <glib.h>

#ifndef __GDAM_BEAT_INFO_H_
#define __GDAM_BEAT_INFO_H_

/*  Describes a range of time (in seconds) for which a certain bpm and
 *  firstbeat value describe the tempo and phase of the music.  */
typedef struct _GdamBeatInfoRange GdamBeatInfoRange;
struct _GdamBeatInfoRange {
	double        		bpm;
	double        		first_beat;
	double        		range_start;
	double        		range_end;
};

/* Describes the arrangement of beats in the output of a model. 
 * DO NOT ACCESS DIRECTLY under normal circumstances, use the 
 * get_* functions which return data appropriate to a position
 * within the song.  */
typedef struct _GdamBeatInfo GdamBeatInfo;
struct _GdamBeatInfo {
	double        		afirst_beat;
	double        		abpm;
	GList*			range_list;
};

GdamBeatInfo*	gdam_beat_info_new	();
GdamBeatInfo*	gdam_beat_info_copy	(const GdamBeatInfo*	beat_info);
double		gdam_beat_info_get_bpm	(const GdamBeatInfo*	beat_info,
				       	 float			position);
double		gdam_beat_info_get_first_beat	
					(const GdamBeatInfo*	beat_info,
				       	 float			position);
double		gdam_beat_info_get_beat_length	
					(const GdamBeatInfo*	beat_info,
				       	 float			position);
void		gdam_beat_info_set_bpm	(GdamBeatInfo*		beat_info,
					 double			new_bpm);
void		gdam_beat_info_set_first_beat	
					(GdamBeatInfo*		beat_info,
					 double			new_first_beat);
void		gdam_beat_info_destroy	(GdamBeatInfo*		beat_info);
				       


#endif
