/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdaminputdevicefile.h"
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

static void gdam_input_device_file_init(GdamInputDeviceFile* input_device_file)
{
}
void            gdam_input_device_file_close(GdamInputDevice* input_device)
{
  GdamInputDeviceFile *device_file = GDAM_INPUT_DEVICE_FILE (input_device);
  if (device_file->io_channel)
    {
      /* XXX: verify that the watch we added is destroyed
         automatically. */
      g_io_channel_unref (device_file->io_channel);
      device_file->io_channel = NULL;
    }
}

gboolean        gdam_input_device_file_open(GdamInputDevice* input_device)
{
  int fd;
  GdamInputDeviceFile *device_file = GDAM_INPUT_DEVICE_FILE (input_device);
  GdamInputDeviceFileClass *klass;
  klass = GDAM_INPUT_DEVICE_FILE_CLASS (GTK_OBJECT (input_device)->klass);

  if (device_file->io_channel != NULL)
    gdam_input_device_file_close (input_device);

  fd = open (input_device->device_name, O_RDONLY);
  if (fd < 0)
    {
      g_warning ("couldn't open %s: %s",
		 input_device->device_name, strerror (errno));
      return FALSE;
    }
  g_message ("device %s opened on fd %d", input_device->device_name, fd);
  device_file->io_channel = g_io_channel_unix_new (fd);
  g_io_add_watch (device_file->io_channel,
		  G_IO_IN, klass->parse_raw, input_device);
  return TRUE;
}

static void
gdam_input_device_file_class_init(GdamInputDeviceClass* input_device_class)
{
  input_device_class->open = gdam_input_device_file_open;
  input_device_class->close = gdam_input_device_file_close;
}

GtkType gdam_input_device_file_get_type()
{
  static GtkType input_device_file_type = 0;
  if (!input_device_file_type)
    {
      static const GtkTypeInfo input_device_file_info = {
	"GdamInputDeviceFile",
	sizeof (GdamInputDeviceFile),
	sizeof (GdamInputDeviceFileClass),
	(GtkClassInitFunc) gdam_input_device_file_class_init,
	(GtkObjectInitFunc) gdam_input_device_file_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_INPUT_DEVICE;
      input_device_file_type = gtk_type_unique (parent,
						&input_device_file_info);
    }
  return input_device_file_type;
}
