/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_SEQUENCER_H_
#define __GDAM_SEQUENCER_H_


typedef struct _GdamSequencer GdamSequencer;
typedef struct _GdamSequencerClass GdamSequencerClass;
typedef struct _GdamSequencerElement GdamSequencerElement;

#include "gdamproducer.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_sequencer_get_type();
#define GDAM_TYPE_SEQUENCER			(gdam_sequencer_get_type ())
#define GDAM_SEQUENCER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_SEQUENCER, GdamSequencer))
#define GDAM_SEQUENCER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_SEQUENCER, GdamSequencerClass))
#define GDAM_IS_SEQUENCER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_SEQUENCER))
#define GDAM_IS_SEQUENCER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_SEQUENCER))

struct _GdamSequencerElement {
	int				buffer_id;

	/* First sample in sequence to fill, in beat numbers. */
	gdouble                         beat_offset;

	/* Resample rate. */
	gdouble				rate;

	/* Gain. */
	gdouble				gain;

	/* The first sample in buffer_id to use. */
	int				sample_offset;

	/* The number of samples to output,
	 * or -1 for "as many as possible" */
	int				output_length;

	/* Used to apply temporary changes to sets of samples. */
	gdouble				rate_factor;
	gdouble				gain_factor;

	/* Group elements into patterns, audible one at a time.  */
	int				pattern_number;
};

struct _GdamSequencerClass {
	GdamProducerClass		producer_class;

	void                          (*sequence_changed)(GdamSequencer*);
	void                          (*set_num_beats)(GdamSequencer* seq,
	                                               gdouble       num_beats);
};

struct _GdamSequencer {
	GdamProducer		producer;
	GList*			elements;

	/* Number of beats in the sequence. */
	gdouble 		num_beats;

	/* Number of subdivisions w/in each beat.  Used to determine
         * exact times for samples. (only used for guis)  */
	int			num_sub_beats;
	int			frozen;
	unsigned		making_request : 1;

	/* If this value is set, it is used to account for 
	 * lag when adding immediately. */
	gdouble			immediate_lag;

	/* Group elements into patterns, audible one at a time.  */
	int			current_pattern;
};

GdamSequencer*          gdam_sequencer_new        ();

GdamSequencerElement*	gdam_sequencer_add_element(GdamSequencer* sequencer,
	                                           int            buffer_id,
	                                           gdouble        beat_number,
	                                           gdouble        rate,
	                                           gdouble        gain,
	                                           int            sample_offset,
	                                           int            output_len);

GdamSequencerElement*	gdam_sequencer_add_element_immediate
                                                  (GdamSequencer* sequencer,
	                                           int            buffer_id,
	                                           gdouble        rate,
	                                           gdouble        gain,
	                                           int            sample_offset,
	                                           int            output_len);

void                    gdam_sequencer_remove_element
                                                  (GdamSequencer* sequencer,
						   GdamSequencerElement*);

void                    gdam_sequencer_set_num_beats
                                                  (GdamSequencer* sequencer,
                                                   gdouble        num_beats);

/* Refresh the server's notion of the sequence. */
void                    gdam_sequencer_update_sequence
                                                  (GdamSequencer* sequencer);

void    	        gdam_sequencer_set_elements
                                                  (GdamSequencer* sequencer,
		                                   GList*         new_elements);
void                    gdam_sequencer_load       (GdamSequencer* sequencer,
                                                   const char*    filename);
void                    gdam_sequencer_save       (GdamSequencer* sequencer,
                                                   const char*    filename);

void                    gdam_sequencer_freeze     (GdamSequencer* sequencer);
void                    gdam_sequencer_thaw       (GdamSequencer* sequencer);
void                    gdam_sequencer_set_pattern(GdamSequencer* sequencer,
						   int		  pattern_no);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
