/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __COMPONENT_FILTER_BENDER_H_
#define __COMPONENT_FILTER_BENDER_H_

#include <client/gdamchannel.h>
#include <common/gdamsample.h>
#include "../gdamfilter.h"
#include "../gdaminfomanager.h"

typedef struct _GdamBender GdamBender;
typedef struct _GdamBenderClass GdamBenderClass;

GtkType gdam_bender_get_type();
#define GDAM_TYPE_BENDER			(gdam_bender_get_type ())
#define GDAM_BENDER(obj)			(GTK_CHECK_CAST ((obj), GDAM_TYPE_BENDER, GdamBender))
#define GDAM_BENDER_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_BENDER, GdamBenderClass))
#define GDAM_IS_BENDER(obj)		(GTK_CHECK_TYPE ((obj), GDAM_TYPE_BENDER))
#define GDAM_IS_BENDER_CLASS(klass)	(GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_BENDER))

typedef enum _GDAM_BENDER_MODE{
	GDAM_BENDER_CONTINUOUS,
	GDAM_BENDER_KEEP_TIME,
	GDAM_BENDER_SNAP_TIME,
	GDAM_BENDER_RETURN 
} GDAM_BENDER_MODE; 

struct _GdamBenderClass {
	GdamFilterClass		filter_class;
	void			(*slow_to_stall) (GdamBender*	bender,
						  gdouble	slow_time);
};

struct _GdamBender {
	GdamFilter		filter;
	guint 			bender_id;
	float			beat_factor;   /* Used to compute the length
						  of certain effects. */
	float			rate_factor;   /* Used to compute the rate
						  of certain effects. */
	float			last_rate;
	  /* server-side mode, i.e. continuous, keeptime... */
	int			mode;
	  /* arg input mapping  mode; defines how sound is bent on input. */
	int			bend_mode;
	GTimeVal		bend_adj_time;
	gboolean		is_bending;
	float			backbuffer;
	  /* TODO: use prefab decongestive request method? */
	gboolean		making_request;
	GdamRequest*		pending_request;
	int			rub_mode;
};

GdamBender*	gdam_bender_new		();
void		gdam_bender_free	(GdamBender*		bender);
void		gdam_bender_set_mode	(GdamBender*		bender,
					 GDAM_BENDER_MODE	mode);
/* num_points==0, changes=NULL resets changes */
void		gdam_bender_set_changes	(GdamBender*		bender,
					 int			num_points,
					 int*			changes);
void		gdam_bender_set_back_buffer	
					(GdamBender*		bender,
					 float			length);
void		gdam_bender_jump_to_mark(GdamBender*		bender);
void		gdam_bender_set_mark    (GdamBender*		bender);
void		gdam_bender_slow_to_stall	
					(GdamBender*		bender,
					 float 			slow_time);
	/* If length argument is zero, length will be based on beat_factor.
	 * if length argument is negative, length will be long. */
void		gdam_bender_rewind	(GdamBender*		bender,
					 float			rewind_length);
	/* If width argument is zero, width will be based on beat_factor */
	/* If length argument is zero, rub will be started with a long length */
void		gdam_bender_rub		(GdamBender*		bender,
					 float			rub_width,
					 float			rub_length);
#endif
