/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamcompressor.h"
#include <common/gdamsample.h>
#include <math.h>

static void gdam_compressor_init(GdamCompressor* compressor)
{
  compressor->threshold = 1.0;
  compressor->ratio = 1.0;
}

static inline void maybe_dispense_request(GdamCompressor* compressor)
{
  GdamArg arg[2];

  arg[0].name = "threshold";
  arg[0].type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (arg + 0) = compressor->threshold;

  arg[1].name = "ratio";
  arg[1].type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (arg + 1) = compressor->ratio;

  gdam_model_set_args (GDAM_MODEL (compressor), 0, 2, arg);
}

static int gdam_compressor_append_start_request(GdamFilter* filter,
				                int new_source_id,
				                GdamRequestBuilder* builder)
{
  GdamCompressor *compressor = GDAM_COMPRESSOR (filter);
  GdamModel *model = GDAM_MODEL (filter);
  GdamRequest *request;
  gtk_object_ref (GTK_OBJECT (compressor));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source ("GdasCompressor",
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					compressor,
					3,
					"threshold",
					GDAM_ARG_TYPE_DOUBLE,
					compressor->threshold,
					"ratio",
					GDAM_ARG_TYPE_DOUBLE,
					compressor->ratio,
					"filtered",
					GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}

static void gdam_compressor_arg_set_threshold(GtkObject* object, GdamArg* arg)
{
  GdamCompressor *compressor;
  compressor = GDAM_COMPRESSOR (object);
  compressor->threshold = GDAM_ARG_DOUBLE (arg);
  maybe_dispense_request (compressor);
}
static void gdam_compressor_arg_get_threshold(GtkObject* object, GdamArg* arg)
{
  GdamCompressor *compressor;
  compressor = GDAM_COMPRESSOR (object);
  GDAM_ARG_DOUBLE (arg) = compressor->threshold;
}

static void gdam_compressor_arg_set_ratio(GtkObject* object, GdamArg* arg)
{
  GdamCompressor *compressor;
  compressor = GDAM_COMPRESSOR (object);
  compressor->ratio = GDAM_ARG_DOUBLE (arg);
  maybe_dispense_request (compressor);
}
static void gdam_compressor_arg_get_ratio(GtkObject* object, GdamArg* arg)
{
  GdamCompressor *compressor;
  compressor = GDAM_COMPRESSOR (object);
  GDAM_ARG_DOUBLE (arg) = compressor->ratio;
}

static void gdam_compressor_class_init(GdamModelClass* model_class)
{
  GdamFilterClass *filter_class;
  GtkObjectClass *object_class;
  filter_class = GDAM_FILTER_CLASS (model_class);
  object_class = GTK_OBJECT_CLASS (model_class);
  model_class->num_arg_sets = 1;
  filter_class->append_start_request = gdam_compressor_append_start_request;
  gdam_arg_register (object_class,
		     "threshold",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_compressor_arg_set_threshold,
		     gdam_compressor_arg_get_threshold,
		     NULL, "0,2", GDAM_ARG_VITAL);
  gdam_arg_register (object_class,
		     "ratio",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_compressor_arg_set_ratio,
		     gdam_compressor_arg_get_ratio,
		     NULL, "0,2", GDAM_ARG_VITAL);
}

GtkType gdam_compressor_get_type()
{
  static GtkType compressor_type = 0;
  if (!compressor_type)
    {
      static const GtkTypeInfo compressor_info = {
	"GdamCompressor",
	sizeof (GdamCompressor),
	sizeof (GdamCompressorClass),
	(GtkClassInitFunc) gdam_compressor_class_init,
	(GtkObjectInitFunc) gdam_compressor_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      compressor_type = gtk_type_unique (parent, &compressor_info);
    }
  return compressor_type;
}
