/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_FADER_H_
#define __GDAM_FADER_H_


typedef struct _GdamFader GdamFader;
typedef struct _GdamFaderClass GdamFaderClass;
typedef struct _GdamFaderElement GdamFaderElement;

#include "../gdamfilter.h"
#include <common/gdamsample.h>
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_fader_get_type();
#define GDAM_TYPE_FADER			(gdam_fader_get_type ())
#define GDAM_FADER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_FADER, GdamFader))
#define GDAM_FADER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_FADER, GdamFaderClass))
#define GDAM_IS_FADER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_FADER))
#define GDAM_IS_FADER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_FADER))

struct _GdamFaderClass {
	GdamFilterClass		filter_class;

	void                  (*fades_changed)(GdamFader*);
};

struct _GdamFaderElement {
	GdamTime		time;
	GdamFixed		volume;
};

struct _GdamFader {
	GdamFilter		filter;

	float			beat_factor;
	int			num_fades;
	gboolean		faded;
	GdamFaderElement*	fade_array;
	/*  Used for the "Perforate" effect.  */
	gboolean		perforated;
	float			base_width;
	float			peak_width;
	float 			depth;
};

gboolean gdam_fader_set_fades       (GdamFader*              fader,
                                     int                     num_fades,
			             const GdamFaderElement* fade_keypoints);
void     gdam_fader_remove_fade     (GdamFader*              fader,
                                     int                     which_fade);

void     gdam_fader_set_passthrough (GdamFader*              fader,
                                     int                     period_length);

/* Private helper routine that is useful in writing skins too... */
void     gdam_fader_sort_fade_array (int                     num_fades,
                                     GdamFaderElement*       array);
void 	 gdam_fader_phase_synch	    (GdamFader*		     fader);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
