/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamfiltermodulator.h"
#include <common/gdamplugin.h>
static int
gdam_filter_modulator_append_start_request(GdamFilter*          filter,
				           int                  new_source_id,
				           GdamRequestBuilder*  builder);
static void gdam_filter_modulator_handle_new_modifier
				(GdamFilterWavetable* filter,
			         int wavetable_local_id,
	                         GdamRequestBuilder* builder);


static void gdam_filter_modulator_init(GdamFilterModulator* filter_modulator)
{
}

static void gdam_filter_modulator_class_init(GdamFilterModulatorClass* klass)
{
  GDAM_FILTER_CLASS (klass)->append_start_request
    = gdam_filter_modulator_append_start_request;
  GDAM_FILTER_WAVETABLE_CLASS (klass)->handle_new_modifier
    = gdam_filter_modulator_handle_new_modifier;
}

GtkType gdam_filter_modulator_get_type()
{
  static GtkType filter_modulator_type = 0;
  if (!filter_modulator_type)
    {
      static const GtkTypeInfo filter_modulator_info = {
	"GdamFilterModulator",
	sizeof (GdamFilterModulator),
	sizeof (GdamFilterModulatorClass),
	(GtkClassInitFunc) gdam_filter_modulator_class_init,
	(GtkObjectInitFunc) gdam_filter_modulator_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER_WAVETABLE;
      filter_modulator_type = gtk_type_unique (parent,
					       &filter_modulator_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return filter_modulator_type;
}

static int
gdam_filter_modulator_append_start_request(GdamFilter*          filter,
				           int                  new_source_id,
				           GdamRequestBuilder*  builder)
{
  gint wavetable_local_id;
  GdamRequest *request;
  GdamModel *model = GDAM_MODEL (filter);

  wavetable_local_id = gdam_filter_wavetable_make_wavetable_request
    (GDAM_FILTER_WAVETABLE (filter), builder);
  gtk_object_ref (GTK_OBJECT (filter));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source
    ("GdasModulator",
     model->source_id,
     gdam_model_got_subsource,
     gdam_model_didnt_get_subsource,
     filter,
     3,
     "mode",
     GDAM_ARG_TYPE_STRING,
     "first_ended",
     "modulator",
     GDAM_ARG_TYPE_INT,
     wavetable_local_id, "filtered", GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}

static void gdam_filter_modulator_handle_new_modifier
				(GdamFilterWavetable* filter,
			         int wavetable_local_id,
	                         GdamRequestBuilder* builder)
{
  GdamArg arg[1];
  int source_id = GDAM_MODEL (filter)->source_id;
  g_return_if_fail (source_id != 0);
  arg[0].name = "modulator";
  arg[0].type = GDAM_ARG_TYPE_INT;
  GDAM_ARG_INT (arg + 0) = wavetable_local_id;
  gdam_request_builder_append_generic
    (builder, gdam_request_set_args (source_id, 1, arg, NULL, NULL, NULL));;
}

