/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_FIR_FILTER_H_
#define __GDAM_FIR_FILTER_H_


typedef struct _GdamFirFilter GdamFirFilter;
typedef struct _GdamFirFilterClass GdamFirFilterClass;

#include "../gdamechobase.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _GdamFirFilterElement GdamFirFilterElement;

GtkType gdam_fir_filter_get_type();
#define GDAM_TYPE_FIR_FILTER			(gdam_fir_filter_get_type ())
#define GDAM_FIR_FILTER(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_FIR_FILTER, GdamFirFilter))
#define GDAM_FIR_FILTER_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_FIR_FILTER, GdamFirFilterClass))
#define GDAM_IS_FIR_FILTER(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_FIR_FILTER))
#define GDAM_IS_FIR_FILTER_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_FIR_FILTER))

struct _GdamFirFilterElement {
	gdouble			offset;
	gdouble			volume;
};


struct _GdamFirFilterClass {
	GdamEchoBaseClass		echo_base_class;
};
struct _GdamFirFilter {
	GdamEchoBase			echo_base;

	int				num_elements;
	gboolean			echoed;
	GdamFirFilterElement*		elements;
	gdouble				grid_unit;	/* in samples */
	gdouble				last_grid_unit;	/* in samples */


	/* When normalizing, the total volume.
	 * Otherwise, the amount to multiply each volume by. */
	gdouble				gain_scale;

	/* If set, normalize the echo_base's array to have
	 * the sum of the volumes == gain_scale. */
	gboolean			normalize;

	/*  Sometimes used to scale grid_unit.  In seconds.  */
	gdouble				beat_factor;
};

void         gdam_fir_filter_add       (GdamFirFilter*        fir_filter,
                                        gdouble               gain,
			                gdouble               delay);
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
