/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamgate.h"
#include <common/gdamsample.h>
#include <math.h>

static void gdam_gate_init(GdamGate* gate)
{
  gate->threshold = 0.0;
}

static inline void maybe_dispense_request(GdamGate* gate)
{
  GdamArg arg[2];

  arg[0].name = "threshold";
  arg[0].type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (arg + 0) = gate->threshold;

  arg[1].name = "inverted";
  arg[1].type = GDAM_ARG_TYPE_INT;
  GDAM_ARG_INT (arg + 1) = gate->inverted;

  gdam_model_set_args (GDAM_MODEL (gate), 0, 2, arg);
}

static int gdam_gate_append_start_request(GdamFilter* filter,
				                int new_source_id,
				                GdamRequestBuilder* builder)
{
  GdamGate *gate = GDAM_GATE (filter);
  GdamModel *model = GDAM_MODEL (filter);
  GdamRequest *request;
  gtk_object_ref (GTK_OBJECT (gate));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source ("GdasGate",
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					gate,
					3,
					"threshold",
					GDAM_ARG_TYPE_DOUBLE,
					gate->threshold,
					"inverted",
					GDAM_ARG_TYPE_INT,
					gate->inverted,
					"filtered",
					GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}

static void gdam_gate_arg_set_threshold(GtkObject* object, GdamArg* arg)
{
  GdamGate *gate;
  gate = GDAM_GATE (object);
  gate->threshold = GDAM_ARG_DOUBLE (arg);
  maybe_dispense_request (gate);
}
static void gdam_gate_arg_get_threshold(GtkObject* object, GdamArg* arg)
{
  GdamGate *gate;
  gdouble threshold;
  gate = GDAM_GATE (object);
  threshold = (gdouble) gate->threshold;
  GDAM_ARG_DOUBLE (arg) = gate->threshold;
}

static void gdam_gate_arg_set_inverted(GtkObject* object, GdamArg* arg)
{
  GdamGate *gate;
  gate = GDAM_GATE (object);
  gate->inverted = GDAM_ARG_INT (arg);
  maybe_dispense_request (gate);
}
static void gdam_gate_arg_get_inverted(GtkObject* object, GdamArg* arg)
{
  GdamGate *gate;
  gate = GDAM_GATE (object);
  GDAM_ARG_INT (arg) = gate->inverted;
}

static void gdam_gate_class_init(GdamModelClass* model_class)
{
  GdamFilterClass *filter_class;
  GtkObjectClass *object_class;
  filter_class = GDAM_FILTER_CLASS (model_class);
  object_class = GTK_OBJECT_CLASS (model_class);
  model_class->num_arg_sets = 1;
  filter_class->append_start_request = gdam_gate_append_start_request;
  gdam_arg_register (object_class,
		     "threshold",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_gate_arg_set_threshold,
		     gdam_gate_arg_get_threshold,
		     NULL, "0,2", GDAM_ARG_VITAL);
  gdam_arg_register (object_class,
		     "inverted",
		     GDAM_ARG_TYPE_INT,
		     gdam_gate_arg_set_inverted,
		     gdam_gate_arg_get_inverted,
		     NULL, "boolean", GDAM_ARG_VITAL);
}

GtkType gdam_gate_get_type()
{
  static GtkType gate_type = 0;
  if (!gate_type)
    {
      static const GtkTypeInfo gate_info = {
	"GdamGate",
	sizeof (GdamGate),
	sizeof (GdamGateClass),
	(GtkClassInitFunc) gdam_gate_class_init,
	(GtkObjectInitFunc) gdam_gate_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      gate_type = gtk_type_unique (parent, &gate_info);
    }
  return gate_type;
}
