/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_MINI_NETWORK_H_
#define __GDAM_MINI_NETWORK_H_


typedef struct _GdamMiniNetwork GdamMiniNetwork;
typedef struct _GdamMiniNetworkClass GdamMiniNetworkClass;
typedef struct _GdamMiniNetworkNodeType GdamMiniNetworkNodeType;
typedef struct _GdamMiniNetworkNode GdamMiniNetworkNode;
typedef struct _GdamMiniNetworkPipe GdamMiniNetworkPipe;
typedef struct _GdamMiniNetworkInputType GdamMiniNetworkInputType;
typedef struct _GdamMiniNetworkParam GdamMiniNetworkParam;
typedef struct _GdamMiniNetworkParamMode GdamMiniNetworkParamMode;
typedef union _GdamMiniNetworkParamValue GdamMiniNetworkParamValue;

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include "../gdamfilter.h"

GtkType gdam_mini_network_get_type();
#define GDAM_TYPE_MINI_NETWORK			(gdam_mini_network_get_type ())
#define GDAM_MINI_NETWORK(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_MINI_NETWORK, GdamMiniNetwork))
#define GDAM_MINI_NETWORK_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_MINI_NETWORK, GdamMiniNetworkClass))
#define GDAM_IS_MINI_NETWORK(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_MINI_NETWORK))
#define GDAM_IS_MINI_NETWORK_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_MINI_NETWORK))
struct _GdamMiniNetworkInputType {
	char*		name;
	gboolean	one_only;	/* for inputs that accomodate infinite
	                                 * input connections. */
	int		x,y,width,height;	/* for skin's sake */
};

typedef enum _GdamMiniNetworkParamType {
	GDAM_MINI_NETWORK_TOGGLE,
	GDAM_MINI_NETWORK_RANGE,
	GDAM_MINI_NETWORK_GENERIC_RANGE,
	GDAM_MINI_NETWORK_BUFFER_ID,
	GDAM_MINI_NETWORK_STRING
} GdamMiniNetworkParamType;

struct _GdamMiniNetworkParam {
	char*		name;
	int		param_index;
	GdamMiniNetworkParamType param_type;

	/* For GDAM_MINI_NETWORK_RANGE-style parameters. */
	gdouble		min_value;
	gdouble		max_value;

	/* for entry-box params, should they continually
	 * update the value, or only when ENTER is pressed? */
	gboolean	enter_not_required;

	int		num_modes;
	GdamMiniNetworkParamMode* modes;
};

struct _GdamMiniNetworkParamMode {
	char*		name;
	int		param_index;
	gdouble		default_value;
	gdouble		min_value;
	gdouble		max_value;
	gdouble		scale;
	gboolean	logarithmic;
};

struct _GdamMiniNetworkNodeType {
	char*		type;
	char*		pixmap;
	int		num_inputs;
	GdamMiniNetworkInputType* input_types;

	/* Parameters, like `gain' */
	int		num_params;
	GdamMiniNetworkParam* params;
	GdamMiniNetworkParamValue* param_defaults;

	/* For skin's sake. */
	int		output_area_x;
	int		output_area_y;
	int		output_area_width;
	int		output_area_height;
};

GdamMiniNetworkNodeType* gdam_mini_network_get_node_type(const char* type);

struct _GdamMiniNetworkPipe {
	GdamMiniNetworkNode*	source;
	GdamMiniNetworkNode*	destination;
	char*			input_name;
	guint			pipe_id;
};

union _GdamMiniNetworkParamValue {
	gdouble			double_value;
	gboolean		boolean_value;
	gint			buffer_id_value;
	char*			string_value;
	struct {
		gdouble		value;
		gdouble		min_value;
		gdouble		max_value;
		unsigned	is_logarithmic : 1,
		        	is_integer : 1;
	} generic_range_value;
};

void gdam_mini_network_param_value_copy(GdamMiniNetworkParamValue* dst,
                                        const GdamMiniNetworkParamValue* src,
					int type);


struct _GdamMiniNetworkNode {
	GdamMiniNetworkNodeType* node_type;

	/* Must exactly match server!!! */
	int			node_id;

	int*			modes;
	GdamMiniNetworkParamValue* param_values;

	char*			options;
};

struct _GdamMiniNetworkClass {
	GdamFilterClass		filter_class;
};
struct _GdamMiniNetwork {
	GdamFilter		filter;
	GSList*			nodes;
	GSList*			pipes;
	GdamMiniNetworkNode*	input;
	GdamMiniNetworkNode*	output;
	guint			last_node_id;
	guint			last_pipe_id;

	/* When frozen, all the outgoing requests are just
	 * queued up -- but be sure you don't actually block
	 * while frozen. */
	int			freeze_count;
	GdamRequestBuilder*	builder;

	/* This number is incremented whenever the object
	 * changes -- this is used by view::synchronize
	 * methods to dropout quickly (by not doing anything
	 * if the synchronize-time seq_no and the current seq_no match).
	 */
	int			seq_no;
};

GdamMiniNetworkNode* gdam_mini_network_new_node
                                    (GdamMiniNetwork* network,
                                     const char*      type,
				     const char*      options);
void                 gdam_mini_network_configure_node
                                    (GdamMiniNetwork* network,
                                     GdamMiniNetworkNode* node,
				     const char*      options);
GdamMiniNetworkPipe* gdam_mini_network_new_pipe 
                                    (GdamMiniNetwork* network,
                                     GdamMiniNetworkNode* source,
                                     GdamMiniNetworkNode* dest,
				     const char*          input_name);
void                 gdam_mini_network_remove_node
                                    (GdamMiniNetwork*     network,
                                     GdamMiniNetworkNode*);
void                 gdam_mini_network_remove_pipe 
                                    (GdamMiniNetwork* network,
                                     GdamMiniNetworkPipe*);

char*                gdam_mini_network_make_param_string
                                    (GdamMiniNetworkNode* node);

/* flags to `make_one_param_string. */
#define GDAM_MINI_NETWORK_BUFFER_AS_STRING     1
void                 gdam_mini_network_make_one_param_string
                                    (GdamMiniNetworkParam*,
                                     GdamMiniNetworkParamValue*,
				     GString* output_buffer,
				     guint flags);

void gdam_mini_network_freeze(GdamMiniNetwork* network);
void gdam_mini_network_thaw(GdamMiniNetwork* network);

/* private: map from name to NodeType for internal use  */
GHashTable* gdam_mini_network_node_type_table;
#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
