#include "gdamrecorderexternal.h"
#include <common/gdamplugin.h>
#include <common/gdam-protocol.h>

static int gdam_recorder_external_append_start_request
				      (GdamFilter*         filter,
				       int                 new_source_id,
				       GdamRequestBuilder* builder);

static void gdam_recorder_external_init(GdamRecorderExternal* recorder_external)
{
}
static void gdam_recorder_external_class_init(GdamFilterClass* filter_class)
{
  filter_class->append_start_request
    = gdam_recorder_external_append_start_request;
}

GtkType gdam_recorder_external_get_type()
{
  static GtkType recorder_external_type = 0;
  if (!recorder_external_type)
    {
      static const GtkTypeInfo recorder_external_info = {
	"GdamRecorderExternal",
	sizeof (GdamRecorderExternal),
	sizeof (GdamRecorderExternalClass),
	(GtkClassInitFunc) gdam_recorder_external_class_init,
	(GtkObjectInitFunc) gdam_recorder_external_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      recorder_external_type = gtk_type_unique (parent,
						&recorder_external_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return recorder_external_type;
}

static int gdam_recorder_external_append_start_request
				      (GdamFilter*         filter,
				       int                 new_source_id,
				       GdamRequestBuilder* builder)
{
  GdamModel *model = GDAM_MODEL (filter);
  gtk_object_ref (GTK_OBJECT (filter));
  model->source_id = gdam_channel_new_local_id (model->channel);
  return gdam_request_builder_append_create
    (builder,
     gdam_request_create_source
     ("GdasRecorderExternal",
      model->source_id,
      gdam_model_got_subsource,
      gdam_model_didnt_get_subsource,
      filter, 1, "filtered", GDAM_ARG_TYPE_INT, new_source_id));
}
typedef struct _HandleAddOutput HandleAddOutput;
struct _HandleAddOutput {
	GdamRecorderHandleNewOutput user_success;
	GHookFunc              user_failure;
	gpointer               user_data;
};
	
static void
handle_add_output_success(int num_args, GdamArg* args, gpointer user_data)
{
  HandleAddOutput *add_output = user_data;
  if (num_args == 0
      || args->type != GDAM_ARG_TYPE_INT
      || strcmp (args->name, "output_id") != 0 || GDAM_ARG_INT (args) == 0)
    {
      if (add_output->user_failure)
	(*add_output->user_failure) (add_output->user_data);
      g_free (add_output);
      return;
    }
  if (add_output->user_success)
    (*add_output->user_success) (GDAM_ARG_INT (args), add_output->user_data);
  g_free (add_output);
}
static void handle_add_output_failure(gpointer user_data)
{
  HandleAddOutput *add_output = user_data;
  if (add_output->user_failure)
    (*add_output->user_failure) (add_output->user_data);
  g_free (add_output);
}

void gdam_recorder_external_start_connect(GdamRecorderExternal*  external,
                                          const char*            spec,
					  GdamRecorderHandleNewOutput handler,
					  GHookFunc              failure,
					  gpointer               user_data)
{
  GdamModel *model = GDAM_MODEL (external);
  GdamChannel *channel = model->channel;
  GdamRequest *request;
  HandleAddOutput *add_output;
  GdamArg arg;
  int source_id = model->source_id;

  if (source_id == 0)
    return;

  add_output = g_new (HandleAddOutput, 1);
  add_output->user_success = handler;
  add_output->user_failure = failure;
  add_output->user_data = user_data;

  arg.type = GDAM_ARG_TYPE_STRING;
  arg.name = "connect";
  GDAM_ARG_STRING (&arg) = (char *) spec;

  request = gdam_request_call_function (GDAM_PROTOCOL_SOURCE_ARG,
					source_id,
					"add_output",
					1,
					&arg,
					handle_add_output_success,
					handle_add_output_failure,
					add_output);
  gdam_channel_append_request (channel, request);
}

void gdam_recorder_external_start_file   (GdamRecorderExternal*  external,
                                          const char*            filename,
					  gboolean               truncate_exist,
					  GdamRecorderHandleNewOutput handler,
					  GHookFunc              failure,
					  gpointer               user_data)
{
  GdamModel *model = GDAM_MODEL (external);
  GdamChannel *channel = model->channel;
  GdamRequest *request;
  HandleAddOutput *add_output;
  GdamArg arg;
  int source_id = model->source_id;

  if (source_id == 0)
    return;

  add_output = g_new (HandleAddOutput, 1);
  add_output->user_success = handler;
  add_output->user_failure = failure;
  add_output->user_data = user_data;

  arg.type = GDAM_ARG_TYPE_STRING;
  arg.name = truncate_exist ? "truncate_file" : "file";
  GDAM_ARG_STRING (&arg) = (char *) filename;

  request = gdam_request_call_function (GDAM_PROTOCOL_SOURCE_ARG,
					source_id,
					"add_output",
					1,
					&arg,
					handle_add_output_success,
					handle_add_output_failure,
					add_output);
  gdam_channel_append_request (channel, request);
}

void gdam_recorder_external_stop         (GdamRecorderExternal*  external,
                                          int                    output_id)
{
  GdamModel *model = GDAM_MODEL (external);
  GdamChannel *channel = model->channel;
  GdamRequest *request;
  GdamArg arg;
  int source_id = model->source_id;

  if (source_id == 0)
    return;

  arg.type = GDAM_ARG_TYPE_INT;
  arg.name = "output_id";
  GDAM_ARG_INT (&arg) = output_id;

  request = gdam_request_call_function (GDAM_PROTOCOL_SOURCE_ARG,
					source_id,
					"add_output",
					1, &arg, NULL, NULL, NULL);
  gdam_channel_append_request (channel, request);
}
