/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

enum {
	SCRUB_LENGTH,
	INPUT_RATE,
	LAST_ARG_SET
};

#include "gdamscrub.h"

static int  gdam_scrub_append_start_request(GdamFilter*         filter,
				            int                 new_source_id,
				            GdamRequestBuilder* builder)
{
  GdamScrub *scrub = GDAM_SCRUB (filter);
  GdamModel *model = GDAM_MODEL (filter);
  GdamRequest *request;
  gtk_object_ref (GTK_OBJECT (scrub));
  model->source_id = gdam_channel_new_local_id (model->channel);
  request = gdam_request_create_source ("GdasScrub",
					model->source_id,
					gdam_model_got_subsource,
					gdam_model_didnt_get_subsource,
					scrub,
					3,
					"input_rate",
					GDAM_ARG_TYPE_DOUBLE,
					scrub->input_rate,
					"scrub_length",
					GDAM_ARG_TYPE_DOUBLE,
					scrub->scrub_length * 44100.0,
					"filtered",
					GDAM_ARG_TYPE_INT, new_source_id);
  return gdam_request_builder_append_create (builder, request);
}
static inline void maybe_dispense_scrub_length_request(GdamScrub* scrub)
{
  GdamArg arg[1];

  arg[0].name = "scrub_length";
  arg[0].type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (arg + 0) = scrub->scrub_length * 44100.0;

  gdam_model_set_args (GDAM_MODEL (scrub), SCRUB_LENGTH, 1, arg);
}

static inline void maybe_dispense_input_rate_request(GdamScrub* scrub)
{
  GdamArg arg[1];

  arg[0].name = "input_rate";
  arg[0].type = GDAM_ARG_TYPE_DOUBLE;
  GDAM_ARG_DOUBLE (arg + 0) = scrub->input_rate;

  gdam_model_set_args (GDAM_MODEL (scrub), INPUT_RATE, 1, arg);
}
static void gdam_scrub_arg_set_input_rate(GtkObject* object, GdamArg* arg)
{
  GdamScrub *scrub;
  gdouble input_rate;

  scrub = GDAM_SCRUB (object);
  input_rate = GDAM_ARG_DOUBLE (arg);
  scrub->input_rate = CLAMP (input_rate, 0.0, 1.0);
  maybe_dispense_input_rate_request (scrub);
}

static void gdam_scrub_arg_get_input_rate(GtkObject* object, GdamArg* arg)
{
  GdamScrub *scrub;

  scrub = GDAM_SCRUB (object);
  GDAM_ARG_DOUBLE (arg) = scrub->input_rate;
}

static void gdam_scrub_arg_set_scrub_length(GtkObject* object, GdamArg* arg)
{
  GdamScrub *scrub;
  gdouble scrub_length;

  scrub = GDAM_SCRUB (object);
  scrub_length = GDAM_ARG_DOUBLE (arg);
  g_return_if_fail (scrub_length >= 0.0);
  scrub->scrub_length = scrub_length;
  maybe_dispense_scrub_length_request (scrub);
}

static void gdam_scrub_arg_get_scrub_length(GtkObject* object, GdamArg* arg)
{
  GdamScrub *scrub;

  scrub = GDAM_SCRUB (object);
  GDAM_ARG_DOUBLE (arg) = scrub->scrub_length;
}

static void gdam_scrub_init(GdamScrub* scrub)
{
  scrub->scrub_length = 0.0;
  scrub->input_rate = 1.0;
}
static void gdam_scrub_class_init(GdamScrubClass* klass)
{
  GdamFilterClass *filter_class;
  GdamModelClass *model_class;
  GtkObjectClass *object_class;
  filter_class = GDAM_FILTER_CLASS (klass);
  model_class = GDAM_MODEL_CLASS (klass);
  object_class = GTK_OBJECT_CLASS (klass);
  model_class->num_arg_sets = LAST_ARG_SET;
  filter_class->append_start_request = gdam_scrub_append_start_request;
  gdam_arg_register (object_class,
		     "input_rate",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_scrub_arg_set_input_rate,
		     gdam_scrub_arg_get_input_rate,
		     NULL, "0,1", GDAM_ARG_VITAL);
  gdam_arg_register (object_class,
		     "scrub_length",
		     GDAM_ARG_TYPE_DOUBLE,
		     gdam_scrub_arg_set_scrub_length,
		     gdam_scrub_arg_get_scrub_length,
		     NULL, "0,", GDAM_ARG_VITAL);
}

GtkType gdam_scrub_get_type()
{
  static GtkType scrub_type = 0;
  if (!scrub_type)
    {
      static const GtkTypeInfo scrub_info = {
	"GdamScrub",
	sizeof (GdamScrub),
	sizeof (GdamScrubClass),
	(GtkClassInitFunc) gdam_scrub_class_init,
	(GtkObjectInitFunc) gdam_scrub_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_FILTER;
      scrub_type = gtk_type_unique (parent, &scrub_info);
    }
  return scrub_type;
}
