/* Gdamdsp -- a feed from a normal dsp device into a gdam-server.
 *
 * Based on:
 *
 * Vsound - a virtual audio loopback cable used for recording /dev/dsp streams
 * Copyright (C) 1999 James Henstridge <james@daa.com.au>
 *
 *   Based on the esddsp utility that is part of esound.  Esddsp's copyright:
 *   Copyright (C) 1998, 1999 Manish Singh <yosh@gimp.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <sys/un.h>
#include <sys/socket.h>
#include <errno.h>

#define guint32 unsigned

#include <dlfcn.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>

#define REAL_LIBC RTLD_NEXT

static int seqno = 1;
#define PREFIX		"/tmp"
#define FILE_PREFIX	"httpldso"
static int pid = 0;

static int is_wgetable (const char* pathname)
{ 
  return strncasecmp(pathname, "http://", 7) == 0;
}

int
open64 (const char *pathname, int flags, ...)
{
  static int (*func) (const char *, int, mode_t) = NULL;
  va_list args;
  mode_t mode;

  if (!func)
    func = (int (*) (const char *, int, mode_t)) dlsym (REAL_LIBC, "open64");
  if (pid == 0)
    pid = getpid();

  va_start (args, flags);
  mode = va_arg (args, mode_t);
  va_end (args);

  if (is_wgetable (pathname))
    {
      char tmp_name[2048];
      char command[2048];
      int rv;
      snprintf (tmp_name, sizeof(tmp_name),
                "%s/%s%d-%d", PREFIX, FILE_PREFIX, pid, seqno++);
      snprintf (command, sizeof(command),
                "wget --quiet -O %s %s", tmp_name, pathname);
      if (system (command) != 0)
        {
	  fprintf (stderr, "running %s failed.\n", command);
	  exit (1);
	}
      rv = (*func) (tmp_name, flags, mode);
      unlink (pathname);
      return rv;
    }
  else
    return (*func) (pathname, flags, mode);
}

int
open (const char *pathname, int flags, ...)
{
  static int (*func) (const char *, int, mode_t) = NULL;
  va_list args;
  mode_t mode;

  if (!func)
    func = (int (*) (const char *, int, mode_t)) dlsym (REAL_LIBC, "open");
  if (pid == 0)
    pid = getpid();

  va_start (args, flags);
  mode = va_arg (args, mode_t);
  va_end (args);

  if (is_wgetable (pathname))
    {
      char tmp_name[2048];
      char command[2048];
      int rv;
      snprintf (tmp_name, sizeof(tmp_name),
                "%s/%s%d-%d", PREFIX, FILE_PREFIX, pid, seqno++);
      snprintf (command, sizeof(command),
                "wget --quiet -O %s %s", tmp_name, pathname);
      if (system (command) != 0)
        {
	  fprintf (stderr, "running %s failed.\n", command);
	  exit (1);
	}
      rv = (*func) (tmp_name, flags, mode);
      unlink (pathname);
      return rv;
    }
  else
    return (*func) (pathname, flags, mode);
}


int
stat (const char *pathname, struct stat* stat_buf)
{
  static int (*func) (const char *, struct stat*) = NULL;

  if (!func)
    func = (int (*) (const char *, struct stat*)) dlsym (REAL_LIBC, "stat");

  if (is_wgetable (pathname))
    {
      bzero (stat_buf, sizeof(struct stat));
      stat_buf->st_mode = S_IRGRP | S_IRUSR;
      return 0;
    }
  return (*func) (pathname, stat_buf);
}
