/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <glib.h>
#include <string.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>
#include <sys/wait.h>
#include "gdamcleanup.h"
#include "gdamconfigfile.h"

/*
 * A hack file -- this attempts to cleanup after a desvasting exist:
 *                like Ctrl-C or exit().  (Nothing we can do about _exit())...
 *                Or kill -9 for that matter.)
 */

static          gboolean has_registered = FALSE;
static volatile gboolean has_cleaned_up = FALSE;
static volatile gint     num_pids       = 0;
static          gint*    pids           = NULL;
static volatile gint     max_pids       = 0;

static void     register_cleanup();

void    gdam_cleanup_init()
{
  if (!has_registered)
    register_cleanup ();
}
void    gdam_cleanup_register_child  (int pid)
{
  if (!has_registered)
    register_cleanup ();
  if (num_pids == max_pids)
    {
      max_pids = max_pids ? (2 * max_pids) : 16;
      pids = g_renew (gint, pids, max_pids);
    }
  pids[num_pids++] = pid;
}
void    gdam_cleanup_unregister_child(int pid)
{
  int hit = 0;
  while (hit < num_pids)
    {
      if (pids[hit] == pid)
	{
	  memmove (pids + hit,
		   pids + hit + 1, sizeof (gint32) * (num_pids - hit - 1));
	  num_pids--;
	  return;
	}
      hit++;
    }
}

static void do_cleanup()
{
  int i;
  if (has_cleaned_up)
    return;
  has_cleaned_up = TRUE;
  for (i = 0; i < num_pids; i++)
    {
      int status;

#ifdef SIGKILL
      (void) kill (pids[i], SIGKILL);
#endif
      (void) waitpid (pids[i], &status, 0);
    }
  if (gdam_pid_file != NULL)
    {
      unlink (gdam_pid_file);
    }
}

static void handle_signal(int signo)
{
  do_cleanup ();
  raise (signo);
  abort ();
  raise (SIGKILL);
}

static void     register_cleanup()
{
  struct sigaction action;
  has_registered = TRUE;
  g_atexit ((GVoidFunc) do_cleanup);
  bzero (&action, sizeof (action));
  action.sa_handler = handle_signal;
  action.sa_flags = SA_NODEFER
#ifdef SA_ONESHOT
    | SA_ONESHOT
#endif
#ifdef SA_RESETHAND
    | SA_RESETHAND
#endif
    ;
  sigaction (SIGINT, &action, NULL);
  sigaction (SIGTERM, &action, NULL);
  sigaction (SIGQUIT, &action, NULL);
}
