#include "gdamrealtime.h"
#include <sched.h>
#include <errno.h>
#include <glib.h>
#include <string.h>
#include <unistd.h>

#ifdef  HAVE_SYS_PRIOCNTL_H

#include <sys/priocntl.h>
#include <sys/rtpriocntl.h>
static void
switch_to_realtime_priority()
{
  pcinfo_t info;
  pcparms_t param;
  rtinfo_t rtinfo;
  rtparms_t rtparam;
  int pid;

  pid = getpid ();

  /* get info */
  strcpy (info.pc_clname, "RT");
  if (-1 == priocntl (P_PID, pid, PC_GETCID, (void *) &info))
    comerr ("Cannot get priority class id priocntl(PC_GETCID)\n");

  movebytes (info.pc_clinfo, &rtinfo, sizeof (rtinfo_t));

  /* set priority not to the max */
  rtparam.rt_pri = rtinfo.rt_maxpri - 2;
  rtparam.rt_tqsecs = 0;
  rtparam.rt_tqnsecs = RT_TQDEF;
  param.pc_cid = info.pc_cid;
  movebytes (&rtparam, param.pc_clparms, sizeof (rtparms_t));
  //needroot(0);
  if (-1 == priocntl (P_PID, pid, PC_SETPARMS, (void *) &param))
    comerr ("Cannot set priority class parameters priocntl(PC_SETPARMS)\n");
  //dontneedroot();
}
#else
#if      defined _POSIX_PRIORITY_SCHEDULING
#include <sched.h>

static void
switch_to_realtime_priority()
{
#ifdef  _SC_PRIORITY_SCHEDULING
  if (sysconf (_SC_PRIORITY_SCHEDULING) == -1)
    {
      g_warning ("WARNING: RR-scheduler not available, disabling.\n");
    }
  else
#endif
    {
      int sched_fifo_min, sched_fifo_max;
      int rv;
      struct sched_param sched_parms;

      memset (&sched_parms, 0, sizeof (sched_parms));
      sched_fifo_min = sched_get_priority_min (SCHED_FIFO);
      sched_fifo_max = sched_get_priority_max (SCHED_FIFO);
      sched_parms.sched_priority = sched_fifo_max - 1;
      //g_message("sched_fifo_max=%d, sched_fifo_min=%d", sched_fifo_max, 
      //sched_fifo_min);
      rv = sched_setscheduler (getpid (), SCHED_FIFO, &sched_parms);
      if (rv == -1)
	{
	  if (errno == EPERM && getuid () != 0)
	    {
	      g_message ("Since you were not root, "
			 "could not set posix \"realtime\" mode");
	      return;
	    }
	  g_warning ("sched_setscheduler error: %s", strerror (errno));
	}
      else if (rv >= 0)
	{
	  /*g_message("set posix \"realtime\" mode, rv=%d", rv); */
	}
      else
	{
	  g_warning
	    ("unexpected negative return value from realtime request");
	}
    }
}
#else
static void
switch_to_realtime_priority()
{
  g_warning ("Cannot switch to realtime priority, not compiled in.");
}
#endif
#endif


#ifdef  HAVE_SYS_PRIOCNTL_H

static void
check_priority()
{
  g_message ("priocntl check_realtime");
}
#else
#if      defined _POSIX_PRIORITY_SCHEDULING

static void
check_priority()
{
#ifdef  _SC_PRIORITY_SCHEDULING
  if (sysconf (_SC_PRIORITY_SCHEDULING) == -1)
    {
      g_warning ("WARNING: RR-scheduler not available, disabling.\n");
    }
#if 0
  else
    {
      int sched_fifo_min, sched_fifo_max;
      int rv;
      struct sched_param sched_parms;
      rv = sched_getparam (getpid (), &sched_parms);
      if (rv < 0)
	{
	  g_warning ("error getting sched params.");
	}
      else
	{
	  g_message ("fifo priority: %d.", sched_parms.sched_priority);
	}
      rv = sched_getscheduler (getpid ());
      if (rv == SCHED_FIFO)
	{
	  g_message ("realtime algorithm: SCHED_FIFO");
	}
      else
	{
	  g_message ("unknown realtime algorithm");
	}
    }
#endif
#endif
}

#else
static void
check_priority()
{
  g_warning ("Cannot check realtime priority, not compiled in.");
}
#endif
#endif




void gdam_realtime_set()
{
  int pid;
  pid = getpid ();
  switch_to_realtime_priority ();
  check_priority ();
}

void gdam_realtime_unset()
{
  struct sched_param param;
  int pid;
  pid = getpid ();
  //g_warning("realtime_unset may not match realtime_set code, pid=%d...",pid);
  param.sched_priority = 0;
  sched_setscheduler (0, SCHED_OTHER, &param);
}




