/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdasdynamicrange.h"

static void gdas_dynamic_range_init(GdasDynamicRange* dynamic_range)
{
  dynamic_range->first_time = TRUE;
}
static int gdas_dynamic_range_process_block(GdasBlockFilter*   filter,
	                                    GdamSample*        output,
					    const GdamSample*  input)
{
  GdasDynamicRange *range;
  int max_sample;
  int block_size;
  int i;
  GdasDynamicRangeClass *klass;
  range = GDAS_DYNAMIC_RANGE (filter);
  block_size = filter->input_block_size;
  klass = (GdasDynamicRangeClass *) ((GtkObject *) filter)->klass;
  range->last_max_sample = range->max_sample;
  max_sample = 0;
  for (i = 0; i < block_size; i++)
    {
      int tmp;
      tmp = ABS (input[i].s[0]);
      if (tmp > max_sample)
	max_sample = tmp;
      tmp = ABS (input[i].s[1]);
      if (tmp > max_sample)
	max_sample = tmp;
    }
  range->max_sample = GDAM_SAMPLE_CLAMP (max_sample);
  if (range->first_time)
    range->last_max_sample = range->max_sample;
  if (klass->process != NULL)
    (*klass->process) (range, output, input, block_size);
  return block_size;
}

static void gdas_dynamic_range_class_init(GdasBlockFilterClass* klass)
{
  klass->process_block = gdas_dynamic_range_process_block;
}

GtkType gdas_dynamic_range_get_type()
{
  static GtkType dynamic_range_type = 0;
  if (!dynamic_range_type)
    {
      static const GtkTypeInfo dynamic_range_info = {
	"GdasDynamicRange",
	sizeof (GdasDynamicRange),
	sizeof (GdasDynamicRangeClass),
	(GtkClassInitFunc) gdas_dynamic_range_class_init,
	(GtkObjectInitFunc) gdas_dynamic_range_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAS_TYPE_BLOCK_FILTER;
      dynamic_range_type = gtk_type_unique (parent, &dynamic_range_info);
    }
  return dynamic_range_type;
}
