/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdasfestival.h"
#include <common/gdamplugin.h>
#include <sys/types.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>

static void gdas_festival_init(GdasFestival* festival)
{
  GDAS_EXTERNAL (festival)->ignore_miss_data = 1;
}

static char* mktmpname(const char* base)
{
  static int seqno = 1;
  return g_strdup_printf ("/tmp/gdam-%s-%d-%d", base, getpid (), seqno++);
}

static void gdas_festival_start(GdasFestival* festival)
{
  char *command[128];
  char *tmp_name;
  int tmp_fd;
  int i = 0;
  GdasExternal *external = GDAS_EXTERNAL (festival);
  tmp_name = mktmpname ("festival");
  tmp_fd = open (tmp_name, O_CREAT | O_RDWR, 0644);
  if (tmp_fd < 0)
    {
      g_warning ("open tmp file failed: %s", strerror (errno));
      g_free (tmp_name);
      return;
    }
  if (festival->text != NULL)
    {
      write (tmp_fd, festival->text, strlen (festival->text));
      write (tmp_fd, "\n", 1);
    }
  lseek (tmp_fd, 0, SEEK_SET);
  unlink (tmp_name);
  g_free (tmp_name);

  gdas_external_set_num_channels (external, 1);

  external->stdin_redirect_fd = tmp_fd;

  command[i++] = "text2wave";
  command[i++] = "-otype";
  command[i++] = "raw";
  command[i++] = "-F";
  command[i++] = "44100";
  command[i] = NULL;
  if (!gdas_external_run_command (external, command))
    {
      g_warning ("running text2wave failed");
      return;
    }
  close (tmp_fd);
  external->stdin_redirect_fd = -1;
}
static void gdas_festival_done_constructing(GdasSource*      source)
{
  if (GDAS_EXTERNAL (source)->process_id <= 0)
    {
      gdas_festival_start (GDAS_FESTIVAL (source));
    }
}

static void gdas_festival_arg_set_text(GtkObject* object, GdamArg* arg)
{
  GdasFestival *festival;
  festival = GDAS_FESTIVAL (object);
  g_free (festival->text);
  festival->text = g_strdup (GDAM_ARG_STRING (arg));
}
static void gdas_festival_arg_get_text(GtkObject* object, GdamArg* arg)
{
  GdasFestival *festival;
  festival = GDAS_FESTIVAL (object);
  GDAM_ARG_STRING (arg) = g_strdup (festival->text);
}
static void gdas_festival_class_init(GdasFestivalClass* festival_class)
{
  GtkObjectClass *object_class;
  GdasSourceClass *source_class;
  object_class = GTK_OBJECT_CLASS (festival_class);
  source_class = GDAS_SOURCE_CLASS (festival_class);
  source_class->done_constructing = gdas_festival_done_constructing;
  gdam_arg_register (object_class,
		     "text",
		     GDAM_ARG_TYPE_STRING,
		     gdas_festival_arg_set_text,
		     gdas_festival_arg_get_text, NULL, NULL, GDAM_ARG_VITAL);
}

GtkType gdas_festival_get_type()
{
  static GtkType festival_type = 0;
  if (!festival_type)
    {
      static const GtkTypeInfo festival_info = {
	"GdasFestival",
	sizeof (GdasFestival),
	sizeof (GdasFestivalClass),
	(GtkClassInitFunc) gdas_festival_class_init,
	(GtkObjectInitFunc) gdas_festival_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAS_TYPE_EXTERNAL;
      festival_type = gtk_type_unique (parent, &festival_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return festival_type;
}
