/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAS_GDAM_DSP_H_
#define __GDAS_GDAM_DSP_H_

typedef struct _GdasGdamDsp GdasGdamDsp;
typedef struct _GdasGdamDspClass GdasGdamDspClass;

#include "../gdassource.h"
#include <common/gdamsampleconverter.h>

GtkType gdas_gdam_dsp_get_type();
#define GDAS_TYPE_GDAM_DSP   gdas_gdam_dsp_get_type()
#define GDAS_GDAM_DSP(ptr) ((GdasGdamDsp*) gtk_type_check_object_cast ((GtkTypeObject*) (ptr), GDAS_TYPE_GDAM_DSP))
#define GDAS_GDAM_DSP_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAS_TYPE_GDAM_DSP, GdasGdamDspClass))

struct _GdasGdamDspClass { 
	GdasSourceClass		source_class;
};

struct _GdasGdamDsp {
	GdasSource		base;

	/* The process id of the thread we started. */
	int			process_id;

	/* The file descriptor we're reading from. */
	GPollFD			poll_fd;

	/* The source-id in the GMainLoop. */
	guint			reader_source;

	/* Data buffered from the gdamdsp source. */
	GdamBuffer		buffer;

	/* Maximum of data to buffer from source. */
	int			max_buffer_size;

	/* The 4-byte length that prefixes each packet is stored
	 * here; header_length is the number of bytes of this 4-byte chunk
	 * that we've received. */
	int			header_length;
	union { /* XXX: is this portable? */
		char		buf[4];
		gint32		big_endian_num;
	} length_union;

	/* if header_length == 4, then body_length will be set,
	 * and body will be large enough to contain the whole packet.
	 * cur_length is the number of bytes of `body' that have been
	 * filled. */
	int			body_length;
	int			body_max_size;	/* allocated size of body */
	char*			body;
	int			cur_length;

	/* does the client think the device is opened? */
	gboolean		remote_open;

	/* Data we must discard b/c we were behind. */
	int			skipped;

	int			num_channels;
	int			bytes_per_sample;
	int			rate;
	GdamSampleConverter	sample_converter;
	GdamSampleConverter	channel_converter;
	unsigned		is_big_endian : 1;
	unsigned		is_signed : 1;

	gboolean		has_poll_fd;

	/*
	 * The next sample we are about to output.
	 * (So this is incremented entirely by `read' and `miss_data').
	 */
	GdamTime		samples_output;

	unsigned		has_been_ready : 1,
				ignore_miss_data : 1;
};


GdasGdamDsp*   gdas_gdam_dsp_new          (int           accepted_fd);


int  gdas_gdam_dsp_get_default_buffer_size();
void gdas_gdam_dsp_set_default_buffer_size(int buffer_size);


/* unnecessary and private */
void           gdas_gdam_dsp_has_opened   (GdasGdamDsp*  gdam_dsp,
                                           int           fd);
#endif
