/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamcrossfadeskin.h"
#include <common/gdamsample.h>
#include <glade/glade.h>
#include <gtk/gtksignal.h>
#include <stdlib.h>
#include "gdamskinutil.h"
#include "gdampacker.h"

static void vols_to_fade_amt(gdouble* fade_amt,
			     gdouble  lvol,
			     gdouble  rvol);

gboolean    gdam_crossfade_skin_set_child (GdamCrossfadeSkin* crossfade,
		                           int 		      child_num,
			                   GdamGain*	      gain);

/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 

static void gdam_crossfade_skin_class_init(GdamCrossfadeSkinClass* klass);
static void gdam_crossfade_skin_init(GdamCrossfadeSkin* skin)
{
  skin->max_gain = 1.0;
}

GtkType gdam_crossfade_skin_get_type()
{
  static GtkType crossfade_skin_type = 0;
  if (!crossfade_skin_type)
    {
      static const GtkTypeInfo crossfade_skin_info = {
	"GdamCrossfadeSkin",
	sizeof (GdamCrossfadeSkin),
	sizeof (GdamCrossfadeSkinClass),
	(GtkClassInitFunc) gdam_crossfade_skin_class_init,
	(GtkObjectInitFunc) gdam_crossfade_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      crossfade_skin_type = gtk_type_unique (GDAM_TYPE_SKIN,
					     &crossfade_skin_info);
    }
  return crossfade_skin_type;
}

/*  ____  _    _         _____                 _   _                 
 * / ___|| | _(_)_ __   |  ___|   _ _ __   ___| |_(_) ___  _ __  ___ 
 * \___ \| |/ / | '_ \  | |_ | | | | '_ \ / __| __| |/ _ \| '_ \/ __|
 *  ___) |   <| | | | | |  _|| |_| | | | | (__| |_| | (_) | | | \__ \
 * |____/|_|\_\_|_| |_| |_|   \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
 */                                                                 


typedef struct _AdjustmentChangeInfo AdjustmentChangeInfo;
struct _AdjustmentChangeInfo {
	int index1, index2;
	GtkAdjustment* adjustment;
	void (*update_crossfade)(AdjustmentChangeInfo* adj_info, 
			         GdamGain* 	       gain1,
				 GdamGain* 	       gain2);
	void (*update_gains)    (AdjustmentChangeInfo* adj_info, 
			         GdamGain* 	       gain1,
				 GdamGain* 	       gain2);
	GdamCrossfadeSkin* crossfade;
};

static gboolean find_by_index(gpointer info, gpointer index)
{
  int info_index = ((GdamCrossfadeGainInfo *) info)->index;
  return info_index == GPOINTER_TO_INT (index) ? 0 : 1;
}

static void refresh_adj(AdjustmentChangeInfo* info){
	GSList* list;
	GdamGain* gain1;
	GdamGain* gain2;
	gdouble total_volume;
	gdouble value;

	list = g_slist_find_custom(info->crossfade->gains,
				   GINT_TO_POINTER(info->index1),
	                           (GCompareFunc) find_by_index);
	g_return_if_fail(list != NULL);
	gain1 = ((GdamCrossfadeGainInfo*)(list->data))->gain;
	list = g_slist_find_custom(info->crossfade->gains,
				   GINT_TO_POINTER(info->index2),
	                           (GCompareFunc) find_by_index);
	g_return_if_fail(list != NULL);
	gain2 = ((GdamCrossfadeGainInfo*)(list->data))->gain;

	vols_to_fade_amt(&value, gain1->volume, gain2->volume);

	info->adjustment->value = value;
	//g_message("refreshing crossfade adjustment to %f", value);
	gtk_adjustment_value_changed(info->adjustment);

}

/* If gain index is passed in, it is treated as `fixed',
 * you may pass in -1 if you just want to renormalize. */
void gdam_crossfade_gain_changed   (GdamCrossfadeSkin* crossfade,
                                    int                gain_index)
{
  GSList *adj_list;

  //g_message("gain %d changed", gain_index);

  if (crossfade->resetting_gains)
    return;

  crossfade->resetting_gains = TRUE;

  for (adj_list = crossfade->adjustment_changes;
       adj_list != NULL; adj_list = adj_list->next)
    {
      AdjustmentChangeInfo *info = adj_list->data;
      if (info->index1 == gain_index || info->index2 == gain_index)
	{
	  refresh_adj (info);
	}
    }

  crossfade->resetting_gains = FALSE;

#if 0
#endif
}

#if 0
void gdam_crossfade_set_max_gain(GdamCrossfadeSkin* crossfade,
			         gdouble            max_gain)
{
  crossfade->max_gain = max_gain;
  gdam_debug ("crossfade: setting max_gain to %f", max_gain);
  gdam_crossfade_gain_changed (crossfade, -1);
}
#endif

static void fade_amt_to_vols(gdouble fade_amt,
			     gdouble* lvol,
			     gdouble* rvol)
{
#if 1
  *lvol = (1.0 - fade_amt);
  *rvol = fade_amt;
#else
  if (fade_amt < (0.5))
    *lvol = 1.0;
  else
    *lvol = (1.0 - fade_amt) * 2.0;

  if (fade_amt < (1.0 - 0.5))
    *rvol = fade_amt * 2.0;
  else
    *rvol = 1.0;
#endif
}

static void vols_to_fade_amt(gdouble* fade_amt,
			     gdouble  lvol,
			     gdouble  rvol)
{
  gdouble total_volume;
  total_volume = ABS (lvol) + ABS (rvol);
  if (total_volume <= .0001 * ABS (rvol))
    *fade_amt = 0.0;
  else
    *fade_amt = ABS (rvol) / total_volume;
}


void gdam_crossfade_set_crossfade   (GdamCrossfadeSkin* crossfade,
                                     int                index1,
			             int                index2,
			             gdouble            fade_amt)
{
  GSList *list;
  GdamGain *gain1;
  GdamGain *gain2;
  gdouble total_volume;
  gdouble lvol, rvol;

  crossfade->current_crossfade = fade_amt;

  list = g_slist_find_custom (crossfade->gains,
			      GINT_TO_POINTER (index1),
			      (GCompareFunc) find_by_index);
  if (list == NULL)
    return;
  gain1 = ((GdamCrossfadeGainInfo *) (list->data))->gain;
  list = g_slist_find_custom (crossfade->gains,
			      GINT_TO_POINTER (index2),
			      (GCompareFunc) find_by_index);
  if (list == NULL)
    return;
  gain2 = ((GdamCrossfadeGainInfo *) (list->data))->gain;

  crossfade->resetting_gains = TRUE;
  total_volume = ABS (gain1->volume) + ABS (gain2->volume);
  fade_amt_to_vols (fade_amt, &lvol, &rvol);
  gdam_gain_set_volume (gain1, total_volume * lvol);
  gdam_gain_set_volume (gain2, total_volume * rvol);

  crossfade->resetting_gains = FALSE;
}

gboolean gdam_crossfade_skin_remove_child(GdamCrossfadeSkin* crossfade,
                                          int                child_num)
{
  GSList *list;
  GdamGain *gain;
  GdamCrossfadeGainInfo *info;
  GtkObject *gain_object;
  list = g_slist_find_custom (crossfade->gains,
			      GINT_TO_POINTER (child_num),
			      (GCompareFunc) find_by_index);
  if (list == NULL)
    {
      return FALSE;
    }
  info = list->data;
  gain = info->gain;
  crossfade->gains = g_slist_remove (crossfade->gains, info);

  gain_object = GTK_OBJECT (info->gain);
  gdam_arg_disconnect (gain_object, info->update_volume_handler);
  gtk_signal_disconnect (gain_object, info->destroy_handler);
  g_free (info);

  return TRUE;
}

static void subgain_volume_changed(GdamCrossfadeGainInfo* info)
{
  /* TODO: this should be made to not have to relookup info. */
  gdam_crossfade_gain_changed (info->crossfade, info->index);
}

static void subgain_destroy(GdamCrossfadeGainInfo* info)
{
  GdamCrossfadeSkin *skin = info->crossfade;
  /* The signals are automatically freed... */
  skin->gains = g_slist_remove (skin->gains, info);
  g_free (info);
}

gboolean gdam_crossfade_skin_set_child(GdamCrossfadeSkin* crossfade,
		                       int 		  child_num,
			               GdamGain*	  gain)
{
  GdamCrossfadeGainInfo *info;
  GtkObject *gain_object = GTK_OBJECT (gain);
  gdam_crossfade_skin_remove_child (GDAM_CROSSFADE_SKIN (crossfade),
				    child_num);

  info = g_new (GdamCrossfadeGainInfo, 1);

  info->index = child_num;
  info->gain = gain;
  info->crossfade = crossfade;
  info->locked = FALSE;
  info->update_volume_handler
    = gdam_arg_connect (gain_object,
			"volume",
			GDAM_ARG_FUNC (subgain_volume_changed), info);
  info->destroy_handler
    = gtk_signal_connect_object (gain_object,
				 "destroy",
				 GTK_SIGNAL_FUNC (subgain_destroy),
				 (GtkObject *) info);

  crossfade->gains = g_slist_prepend (crossfade->gains, info);
  return TRUE;
}

/*
	gdam_crossfade_adjustment_handler:index1:index2

	      Makes this widget the adjustment fader from the child
	      at index1 to the child at index2, over the range of 0 .. 1.
 */


static void gdam_crossfade_value_changed(AdjustmentChangeInfo* info)
{
  //g_message("crossfade adjustment set to %f", info->adjustment->value);
  gdam_crossfade_set_crossfade (info->crossfade,
				info->index1,
				info->index2, info->adjustment->value);

}
 
static gboolean gdam_crossfade_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adjustment;
  AdjustmentChangeInfo *change_info;
  GdamCrossfadeSkin *crossfade = GDAM_CROSSFADE_SKIN (skin);
  int index1, index2;
  adjustment = util_get_widget_adjustment (GTK_WIDGET (object));
  g_return_val_if_fail (adjustment != NULL, TRUE);

  if (num_handler_names < 3)
    {
      g_warning ("too few arguments to %s", handler_names[0]);
      return TRUE;
    }
  index1 = atoi (handler_names[1]);
  index2 = atoi (handler_names[2]);
  if (index1 == index2)
    {
      g_warning ("%s: crossfade was attached between the same gains",
		 handler_names[0]);
      return TRUE;
    }
  change_info = g_new (AdjustmentChangeInfo, 1);
  change_info->index1 = index1;
  change_info->index2 = index2;
  change_info->adjustment = adjustment;
  change_info->crossfade = crossfade;
  gtk_signal_connect_full (GTK_OBJECT (adjustment),
			   "value_changed",
			   GTK_SIGNAL_FUNC (gdam_crossfade_value_changed),
			   NULL, change_info, g_free, TRUE, FALSE);
  crossfade->adjustment_changes =
    g_slist_prepend (crossfade->adjustment_changes, change_info);
  gdam_delayed_adjustment_init (adjustment);
  return TRUE;
}

static void alloc_colors(GdamCrossfadeSkin* skin)
{
  int state = GTK_WIDGET_STATE (skin->drawing_area);
  //GdkGC* gc = gdk_gc_new(skin->backing_pixmap);
  //gdk_gc_set_foreground(gc, util_alloc_color_cached(65535, 0, 0));
  //skin->circle_gc = gc;
  skin->background = skin->drawing_area->style->bg_gc[state];
  skin->left_gc = skin->drawing_area->style->fg_gc[GTK_STATE_NORMAL];
  skin->right_gc = skin->drawing_area->style->fg_gc[GTK_STATE_INSENSITIVE];
}

static void copy_backing_area(GdamCrossfadeSkin* skin, const GdkRectangle* area)
{
  gdk_draw_pixmap (skin->drawing_area->window,
		   skin->drawing_area->
		   style->fg_gc[GTK_WIDGET_STATE (skin->drawing_area)],
		   skin->backing_pixmap, area->x, area->y, area->x, area->y,
		   area->width, area->height);
}

static void redraw_area(GdamCrossfadeSkin* skin, const GdkRectangle* area)
{
  int num_points, i;
  //int circle_radius = skin->circle_radius;
  //gdouble pix_per_grid = skin->draw_width / skin->area_length_in_grid;
  GdkRectangle rect;
  //GdamFirFilter* fir_filter = get_fir_filter(skin);
  if (area == NULL)
    {
      rect.width = skin->drawing_area->allocation.width;
      rect.height = skin->drawing_area->allocation.height;
      rect.x = rect.y = 0;
      area = &rect;
    }
  if (!skin->left_gc)
    alloc_colors (skin);

  gdk_draw_rectangle (skin->backing_pixmap,
		      skin->background,
		      TRUE, area->x, area->y, area->width, area->height);

  for (i = area->x; i < area->x + area->width; i++)
    {
      gdouble perc = (gdouble) i / (gdouble) skin->draw_width;
      gdouble lval;
      gdouble rval;
      int x = i;
      int y;
      fade_amt_to_vols (perc, &lval, &rval);
      y = (skin->draw_height - 2) * lval / 1.0;
      /*  Flip display  */
      y = skin->draw_height - y;
      gdk_draw_line (skin->backing_pixmap, skin->left_gc, x, y, x, y - 1);
      y = (skin->draw_height - 2) * rval / 1.0;
      /*  Flip display  */
      y = skin->draw_height - y;
      gdk_draw_line (skin->backing_pixmap, skin->right_gc, x, y, x, y - 1);
    }

  copy_backing_area (skin, area);
}

static gint
crossfade_configure(GtkWidget* widget,
                    GdkEventConfigure* configure_event,
		    GdamCrossfadeSkin* skin)
{
  if (skin->backing_pixmap)
    gdk_pixmap_unref (skin->backing_pixmap);
  skin->backing_pixmap = gdk_pixmap_new (widget->window,
					 widget->allocation.width,
					 widget->allocation.height, -1);
  skin->draw_width = widget->allocation.width;
  skin->draw_height = widget->allocation.height;
  return TRUE;
}



static gint
crossfade_expose(GtkDrawingArea*    area,
                 GdkEventExpose*    expose_event,
                 GdamCrossfadeSkin* skin)
{
  redraw_area (skin, &expose_event->area);
  return 0;
}

static void connect_drawing_area(GtkDrawingArea*    drawing_area,
                                 GdamCrossfadeSkin* skin)
{
  skin->drawing_area = GTK_WIDGET (drawing_area);
  //gtk_drawing_area_size(GTK_DRAWING_AREA(drawing_area), 800, 300);
#if 0
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "button_press_event",
		      GTK_SIGNAL_FUNC (crossfade_button_press), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "button_release_event",
		      GTK_SIGNAL_FUNC (crossfade_button_release), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "motion_notify_event",
		      GTK_SIGNAL_FUNC (crossfade_button_motion), skin);
#endif
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "configure_event",
		      GTK_SIGNAL_FUNC (crossfade_configure), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "expose_event",
		      GTK_SIGNAL_FUNC (crossfade_expose), skin);
  gtk_widget_show (GTK_WIDGET (drawing_area));
  gtk_widget_add_events (GTK_WIDGET (drawing_area), GDK_EXPOSURE_MASK
			 //| GDK_LEAVE_NOTIFY_MASK
			 //| GDK_BUTTON_PRESS_MASK
			 //| GDK_BUTTON_RELEASE_MASK
			 //| GDK_POINTER_MOTION_MASK
			 //| GDK_POINTER_MOTION_HINT_MASK
    );
}



static gboolean gdam_crossfade_drawing_area_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  if (!GTK_IS_DRAWING_AREA (object))
    {
      g_warning ("%s may only be connected to a drawing area",
		 handler_names[0]);
      return TRUE;
    }
  connect_drawing_area (GTK_DRAWING_AREA (object),
			GDAM_CROSSFADE_SKIN (skin));
  return TRUE;
}




/*
    gdam_packer_set_crossfade_gain:crossfade_name:index:gain_name


    Attach a gain to a crossfader at a certain index.
 */

typedef struct _SetGainInfo SetGainInfo;
struct _SetGainInfo {
	char* crossfade_name;
	char* gain_name;
	int gain_index;
	GdamPacker* packer;
};
static gboolean deferred_set_gain(GdamSkin* skin, gpointer ptr)
{
  SetGainInfo *info = ptr;
  GdamGain *gain;
  GdamCrossfadeSkin *crossfade;
  int gain_index;
  GdamPacker *packer = GDAM_PACKER (skin);

  gain = GDAM_GAIN (gdam_packer_get_model (packer, info->gain_name));
  crossfade = GDAM_CROSSFADE_SKIN (gdam_packer_get_skin (packer,
							 info->crossfade_name));
  if (gain == NULL)
    {
      g_warning ("set_crossfade_gain: gain `%s' not found", info->gain_name);
      return TRUE;
    }
  if (crossfade == NULL)
    {
      g_warning ("set_crossfade_gain: crossfade `%s' not found",
		 info->crossfade_name);
      return TRUE;
    }
  gain_index = info->gain_index;

  if (!gdam_crossfade_skin_set_child (crossfade, gain_index, gain))
    {
      g_warning ("gdam_crossfade_skin_set_child failed");
      return TRUE;
    }
  return TRUE;
}

static void free_set_gain(gpointer ptr)
{
  SetGainInfo *info = ptr;
  g_free (info->crossfade_name);
  g_free (info->gain_name);
  g_free (info);
}

static gboolean gdam_packer_set_crossfade_gain_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SetGainInfo *info;
  if (num_args < 4)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new (SetGainInfo, 1);
  info->gain_index = atoi (args[2]);
  info->gain_name = args[3];
  info->crossfade_name = args[1];
  info->packer = GDAM_PACKER (skin);
  args[1] = args[3] = NULL;

  gdam_skin_add_initializer (skin, deferred_set_gain, free_set_gain, info);
  return TRUE;
}


static GdamSkinGenericConnect connect_infos[] =
{
  {
  "gdam_crossfade_adjustment", gdam_crossfade_adjustment_handler}
  ,
  {
  "gdam_crossfade_drawing_area", gdam_crossfade_drawing_area_handler}
,};
static GdamSkinGenericConnect packer_connect_infos[] =
{
  {
  "gdam_packer_set_crossfade_gain", gdam_packer_set_crossfade_gain_handler}
,};

static void gdam_crossfade_skin_class_init(GdamCrossfadeSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  gdam_skin_class_register_generic_handlers
    (skin_class, G_N_ELEMENTS (connect_infos), connect_infos);
  gdam_skin_class_register_generic_handlers
    (gdam_packer_class (),
     G_N_ELEMENTS (packer_connect_infos), packer_connect_infos);
}

