
#include "gdamdeviceswitcherskin.h"
#include "gdampacker.h"
#include "gdamskinutil.h"
#include <common/gdam-protocol.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <gtk/gtksignal.h>
#include <gtk/gtktogglebutton.h>

static void gdam_device_switcher_skin_init(GdamDeviceSwitcherSkin* device_switcher_skin)
{
  device_switcher_skin->call = g_strdup ("flustered:6060");
}
static void gdam_device_switcher_skin_class_init(GdamDeviceSwitcherSkinClass* klass);

GtkType gdam_device_switcher_skin_get_type()
{
  static GtkType device_switcher_skin_type = 0;
  if (!device_switcher_skin_type)
    {
      static const GtkTypeInfo device_switcher_skin_info = {
	"GdamDeviceSwitcherSkin",
	sizeof (GdamDeviceSwitcherSkin),
	sizeof (GdamDeviceSwitcherSkinClass),
	(GtkClassInitFunc) gdam_device_switcher_skin_class_init,
	(GtkObjectInitFunc) gdam_device_switcher_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      device_switcher_skin_type = gtk_type_unique (parent,
						   &device_switcher_skin_info);
    }
  return device_switcher_skin_type;
}

typedef struct _SwitchInfo{
	char*			device_name;
	GdamDeviceSwitcherSkin* skin;
	int			recording_id;
	int			device_index;
	gboolean		simple_record;
} SwitchInfo;

static void on_gdam_device_switcher_skin_select_device(GtkObject* object,
						       SwitchInfo* info)
{
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (object)))
    {
      GdamToplevel *toplevel;
      GdamModel *model;
      GdamSkin *skin;
      const char* device_name;
      skin = GDAM_SKIN (info->skin);
      toplevel = gdam_toplevel_get_global (skin->channel);
      g_return_if_fail (info->skin->child_skin != 0);

      device_name = info->device_name;
      if (!device_name)
	device_name = (char*)g_list_nth_data(skin->channel->device_list, 
	    				     info->device_index);
      if (!device_name) {
	      g_warning("missing device name or device num out of range.");
	      return;
      }

      model = GDAM_SKIN (info->skin->child_skin)->model;
      gdam_debug ("SWITCHER set device: %s", info->device_name);
      gdam_toplevel_set_child_device (toplevel, model, device_name);
    }
}

static void
process_stop_record_success(int num_args, GdamArg* args, gpointer user_data)
{
  SwitchInfo *info = (SwitchInfo *) user_data;
  info->recording_id = 0;
  gdam_debug ("Stopped recording.");
}

static void
process_record_success(int num_args, GdamArg* args, gpointer user_data)
{
  int i;
  SwitchInfo *info = (SwitchInfo *) user_data;
  for (i = 0; i < num_args; i++)
    {
      GdamArg *arg = args + i;
      if (strcmp (arg->name, "output_id") == 0
	  && arg->type == GDAM_ARG_TYPE_INT)
	{
	  info->recording_id = GDAM_ARG_INT (arg);
	  gdam_debug ("get recording id of %d", GDAM_ARG_INT (arg));
	}
      else
	{
	  char *arg_value;
	  arg_value = gdam_arg_val_as_string (args + i);
	  fprintf (stderr, "  arg %d: %s => %s\n",
		   i, args[i].name, arg_value);
	  g_free (arg_value);
	}
    }
}

static void process_call_failure(gpointer unused)
{
  gdam_debug ("call command failed");
}

static void on_gdam_device_switcher_skin_record_device(GtkObject* object,
						       SwitchInfo* info)
{
  GdamSkin *skin;
  GdamToplevel *toplevel;
  GdamToplevelDevice *device;
  const char* device_name;
  skin = GDAM_SKIN (info->skin);
  toplevel = gdam_toplevel_get_global (skin->channel);
  device_name = info->device_name;
  if (!device_name)
    device_name = g_list_nth_data (skin->channel->device_list, 
		    		   info->device_index);
  if (!device_name) 
    {
      g_warning("missing device_name or device index out of range.");
      return;
    }
  device = g_hash_table_lookup (toplevel->devices_by_name, device_name);
  g_return_if_fail (device != NULL);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (object)))
    {
      int recorder_id = device->recorder_id;
      GdamRequest *request;
      GdamArg arg[1];
      gdam_debug ("SWITCHER recording device: %s", device_name);
      g_return_if_fail (info->recording_id == 0);
      if (info->simple_record) 
        {
	  arg[0].name = "filename";
	  arg[0].type = GDAM_ARG_TYPE_STRING;
	  GDAM_ARG_STRING (arg + 0) = "/mnt/yurt/temp/testrecord";
	  request = gdam_request_call_function (GDAM_PROTOCOL_SOURCE_ARG,
						recorder_id,
						"record",
						1,
						arg,
						process_record_success,
						process_call_failure, 
						info);
        }
      else
	{
	  arg[0].name = "connect";
	  arg[0].type = GDAM_ARG_TYPE_STRING;
	  GDAM_ARG_STRING (arg + 0) =
	    g_strdup (GDAM_DEVICE_SWITCHER_SKIN (skin)->call);
	  request = gdam_request_call_function (GDAM_PROTOCOL_SOURCE_ARG,
						recorder_id,
						"add_output",
						1,
						arg,
						process_record_success,
						process_call_failure, info);
	}
      gdam_channel_append_request (skin->channel, request);
    }
  else
    {
      int recorder_id = device->recorder_id;
      GdamRequest *request;
      GdamArg arg[1];
      g_return_if_fail (info->recording_id != 0);
      gdam_debug ("Switcher: stopping recording on device: %s",
		  device_name);
      arg[0].name = "output_id";
      arg[0].type = GDAM_ARG_TYPE_INT;
      GDAM_ARG_INT (arg + 0) = info->recording_id;
      request = gdam_request_call_function (GDAM_PROTOCOL_SOURCE_ARG,
					    recorder_id,
					    "remove_output",
					    1,
					    arg,
					    process_stop_record_success,
					    process_call_failure, info);
      gdam_channel_append_request (skin->channel, request);
    }
}
	
	

typedef struct _SetChildInfo SetChildInfo;
struct _SetChildInfo {
	char* switcher_name;
	char* child_name;
	int index;
	GdamPacker* packer;
};

static void gdam_device_switcher_skin_set_child(
				GdamDeviceSwitcherSkin* skin,
				guint index,
				GdamSkin* child_skin)
{
  if (index > 0)
    {
      g_warning ("device switcher set child: %d out of range", index);
      return;
    }
  gdam_debug ("Setting child skin");

  skin->child_skin = child_skin;
}

static gboolean deferred_set_child(GdamSkin* skin, gpointer ptr)
{
  SetChildInfo *info = ptr;
  GdamDeviceSwitcherSkin *switcher;
  GdamSkin *child_skin;
  GdamPacker *packer = GDAM_PACKER (skin);

  switcher = GDAM_DEVICE_SWITCHER_SKIN (gdam_packer_get_skin (packer,
							      info->
							      switcher_name));
  child_skin = gdam_packer_get_skin (packer, info->child_name);
  if (switcher == NULL)
    {
      g_warning ("set_child: switcher `%s' not found", info->switcher_name);
      return FALSE;
    }

  gdam_device_switcher_skin_set_child (switcher, info->index, child_skin);
  return TRUE;
}

static void free_set_child(gpointer ptr)
{
  SetChildInfo *info = ptr;
  g_free (info->switcher_name);
  g_free (info->child_name);
  g_free (info);
}

static gboolean gdam_device_switcher_select_device_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SwitchInfo *info;
  if (num_args < 2)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new0 (SwitchInfo, 1);
  info->skin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  info->device_name = strdup (args[1]);
  gtk_signal_connect (object,
		      signal_name,
		      on_gdam_device_switcher_skin_select_device, info);
  return TRUE;
}

static gboolean gdam_device_switcher_select_device_num_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SwitchInfo *info;
  if (num_args < 2)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new0 (SwitchInfo, 1);
  info->skin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  info->device_index = atoi (args[1]);
  gtk_signal_connect (object,
		      signal_name,
		      on_gdam_device_switcher_skin_select_device, info);
  return TRUE;
}



static gboolean gdam_device_switcher_record_device_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SwitchInfo *info;
  if (num_args < 2)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new0 (SwitchInfo, 1);
  info->skin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  info->device_name = strdup (args[1]);
  gtk_signal_connect (object,
		      signal_name,
		      on_gdam_device_switcher_skin_record_device, info);
  return TRUE;
}

static gboolean gdam_device_switcher_record_device_num_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SwitchInfo *info;
  if (num_args < 2)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new0 (SwitchInfo, 1);
  info->skin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  info->device_index = atoi (args[1]);
  gtk_signal_connect (object,
		      signal_name,
		      on_gdam_device_switcher_skin_record_device, info);
  return TRUE;
}

static gboolean gdam_device_switcher_simple_record_device_num_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SwitchInfo *info;
  if (num_args < 2)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new0 (SwitchInfo, 1);
  info->skin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  info->device_index = atoi (args[1]);
  info->simple_record = TRUE;
  gtk_signal_connect (object,
		      signal_name,
		      on_gdam_device_switcher_skin_record_device, info);
  return TRUE;
}



static void toggle_misses_data(GtkToggleButton* toggle, 
                               GdamSkin* skin)
{
  GdamDeviceSwitcherSkin *switcher_skin;
  GdamModel *component;
  GdamToplevel *toplevel;
  switcher_skin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  component = GDAM_SKIN (switcher_skin->child_skin)->model;
  toplevel = gdam_toplevel_get_global (skin->channel);
  g_return_if_fail (toplevel != NULL);
  gdam_toplevel_set_misses_data (toplevel, component, toggle->active);
}

static gboolean gdam_device_switcher_toggle_misses_data_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  g_return_val_if_fail (GTK_IS_TOGGLE_BUTTON (object), TRUE);
  gtk_signal_connect (object,
		      signal_name,
		      GTK_SIGNAL_FUNC (toggle_misses_data), skin);
  return TRUE;
}

static gboolean gdam_packer_device_switcher_set_child_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_args,
				 char**             args)
{
  SetChildInfo *info;
  if (num_args < 4)
    {
      g_warning ("%s: too few arguments", args[0]);
      return TRUE;
    }
  info = g_new (SetChildInfo, 1);
  info->child_name = args[3];
  info->index = atoi (args[2]);
  info->switcher_name = args[1];
  args[1] = args[3] = NULL;

  gdam_skin_add_initializer (skin, deferred_set_child, free_set_child, info);
  return TRUE;
}

static gboolean is_blank(const char* string)
{
  while (*string && isspace (*string))
    string++;
  return *string == 0 ? TRUE : FALSE;
}

static void on_call(GdamDeviceSwitcherSkin* skin)
{
  const char *txt;
  g_return_if_fail (skin->call_entry != NULL);
  txt = gtk_entry_get_text (skin->call_entry);
  g_return_if_fail (txt != NULL && !is_blank (txt));
  skin->call = g_strdup (txt);
}

static gboolean gdam_device_switcher_call_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamDeviceSwitcherSkin *devskin = GDAM_DEVICE_SWITCHER_SKIN (skin);
  devskin->call_entry = GTK_ENTRY (object);
  gtk_signal_connect_object (GTK_OBJECT (devskin->call_entry),
			     "activate", on_call, (GtkObject *) skin);
  if (devskin->call)
    gtk_entry_set_text (devskin->call_entry, devskin->call);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] =
{
  {
    "gdam_device_switcher_select_device",
     gdam_device_switcher_select_device_handler },
  { "gdam_device_switcher_select_device_num",
     gdam_device_switcher_select_device_num_handler },
  { "gdam_device_switcher_record_device",
     gdam_device_switcher_record_device_handler },
  { "gdam_device_switcher_record_device_num",
     gdam_device_switcher_record_device_num_handler },
  { "gdam_device_switcher_simple_record_device_num",
     gdam_device_switcher_simple_record_device_num_handler },
  { "gdam_device_switcher_call_entry",
     gdam_device_switcher_call_entry_handler },
  { "gdam_device_switcher_toggle_misses_data",
     gdam_device_switcher_toggle_misses_data_handler }
};

static GdamSkinGenericConnect packer_connect_infos[] =
{
  { "gdam_packer_device_switcher_set_child",
     gdam_packer_device_switcher_set_child_handler },
};



static void gdam_device_switcher_skin_class_init(GdamDeviceSwitcherSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  //skin_class->build_new_model = gdam_gain_build_new_model;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
  gdam_skin_class_register_generic_handlers (gdam_packer_class (),
					     G_N_ELEMENTS
					     (packer_connect_infos),
					     packer_connect_infos);
}

