/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include <component/gdamproducermp3.h>
#include <component/gdamsongdatabase.h>
//#include "gdamfactory.h"
#include <gtk/gtkdnd.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtksignal.h>
#include <gtk/gtkentry.h>
#include <glade/glade.h>
#include <string.h>
#include "gdamproducermp3skin.h"
#include "gdamsongselectorskin.h"
#include "gdampacker.h"
#include "gdamskinutil.h"

static void gdam_producer_mp3_skin_class_init(GdamProducerMp3SkinClass* klass);
/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 

static void gdam_producer_mp3_skin_init(GdamProducerMp3Skin* skin)
{
}

GtkType gdam_producer_mp3_skin_get_type()
{
  static GtkType producermp3skin_type = 0;
  if (!producermp3skin_type)
    {
      static const GtkTypeInfo producermp3skin_info = {
	"GdamProducerMp3Skin",
	sizeof (GdamProducerMp3Skin),
	sizeof (GdamProducerMp3SkinClass),
	(GtkClassInitFunc) gdam_producer_mp3_skin_class_init,
	(GtkObjectInitFunc) gdam_producer_mp3_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      producermp3skin_type = gtk_type_unique (GDAM_TYPE_SKIN,
					      &producermp3skin_info);
    }
  return producermp3skin_type;
}



/* TODO: parse list of filenames. */
static void on_mp3_filename_received(
			    const char* 	  filenames,
			    gpointer          data)
{
  GdamProducerMp3 *mp3 = GDAM_PRODUCER_MP3 (data);
  char filename[1024];
  char *nl;
  strncpy (filename, filenames, 1024);
  nl = strchr (filename, '\n');
  if (nl)
    *nl = '\0';
  gdam_debug ("filename %s dropped", filename);
  gdam_producer_mp3_set_filename (mp3, filename);
  return;
}

static void on_mp3_filename_dragged(GtkSelectionData* 	selection_data,
				     gpointer 		data)
{
  GdamProducerMp3 *mp3 = GDAM_PRODUCER_MP3 (data);
  if (!mp3->filename)
    return;
  gtk_selection_data_set (selection_data, selection_data->target, 8,
			  (guchar *) strdup (mp3->filename),
			  strlen (mp3->filename));
}

static gboolean gdam_mp3_frame_drag_data_get_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gdam_debug ("producermp3: setting filename source");
  gdam_widget_string_source_set (GTK_WIDGET (object),
				 on_mp3_filename_dragged,
				 "gdam-filename-text",
				 GDAM_PRODUCER_MP3 (skin->model));
  return TRUE;
}


static gboolean gdam_mp3_frame_drag_data_received_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gdam_debug ("producermp3: setting filename dest");
  gdam_widget_string_dest_set (GTK_WIDGET (object),
			       on_mp3_filename_received,
			       "gdam-filename-text",
			       GDAM_PRODUCER_MP3 (skin->model));
  return TRUE;
}

static void do_import_files(GdamProducerMp3Skin* skin)
{
  GdamSongSelectorSkin *songselskin;
  GdamSkin *packer;
  GdamProducerMp3 *mp3;
  char filename[1024];
  char *filenames;
  char *nl;

  packer = gdam_skin_try_get_unique ("GdamSongSelector");
  if (!packer)
    {
      gdam_debug ("producermp3skin: no songsel open, can't import.");
      return;
    }
  songselskin =
    GDAM_SONG_SELECTOR_SKIN (gdam_packer_get_skin
			     (GDAM_PACKER (packer), "songsel"));
  filenames = gdam_song_selector_export_selection_string (songselskin);
  strncpy (filename, filenames, 1024);
  nl = strchr (filename, '\n');
  if (nl)
    *nl = '\0';

  gdam_debug ("filename %s imported", filename);

  mp3 = GDAM_PRODUCER_MP3 (GDAM_SKIN (skin)->model);
  gdam_producer_mp3_set_filename (mp3, filename);

  return;
}

static gboolean gdam_mp3_import_songsel_files_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (do_import_files),
			     GTK_OBJECT (skin));
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] =
{
  { "gdam_mp3_frame_drag_data_received",
     gdam_mp3_frame_drag_data_received_handler  },
  { "gdam_mp3_frame_drag_data_get",
     gdam_mp3_frame_drag_data_get_handler },
  { "gdam_mp3_import_songsel_files",
     gdam_mp3_import_songsel_files_handler},
};

static void gdam_producer_mp3_skin_class_init(GdamProducerMp3SkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->model_type = GDAM_TYPE_PRODUCER_MP3;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}

