/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamrespeedskin.h"
#include <component/gdamrespeed.h>
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>
#include <stdlib.h>
#include "gdamskinutil.h"

static void gdam_respeed_skin_class_init(GdamRespeedSkinClass* klass);

/*    ____ _                 ____  _          __  __ 
 *   / ___| | __ _ ___ ___  / ___|| |_ _   _ / _|/ _|
 *  | |   | |/ _` / __/ __| \___ \| __| | | | |_| |_ 
 *  | |___| | (_| \__ \__ \  ___) | |_| |_| |  _|  _|
 *   \____|_|\__,_|___/___/ |____/ \__|\__,_|_| |_|  
 */                                                 

static void gdam_respeed_skin_init(GdamRespeedSkin* skin)
{
}

GtkType gdam_respeed_skin_get_type()
{
  static GtkType respeed_skin_type = 0;
  if (!respeed_skin_type)
    {
      static const GtkTypeInfo respeed_skin_info = {
	"GdamRespeedSkin",
	sizeof (GdamRespeedSkin),
	sizeof (GdamRespeedSkinClass),
	(GtkClassInitFunc) gdam_respeed_skin_class_init,
	(GtkObjectInitFunc) gdam_respeed_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      respeed_skin_type = gtk_type_unique (GDAM_TYPE_SKIN,
					   &respeed_skin_info);
    }
  return respeed_skin_type;
}

static void  respeed_update_tempo(GtkLabel* label, gdouble new_tempo,
				   GtkObject* object)
{
  char text[63];
  sprintf (text, "%3.3f", new_tempo);
  gtk_label_set_text (label, text);
}

static void on_tempo_string_drop(const char* 		new_bpm_str,
				 gpointer		user_data)
{
  GdamRespeed *respeed;
  GdamModel *child;
  GdamBeatInfo child_bi;
  float new_bpm;
  float source_bpm;
  float new_rate = 0;
  respeed = GDAM_RESPEED (GDAM_SKIN (user_data)->model);
  new_bpm = atof (new_bpm_str);
  child = GDAM_FILTER (respeed)->child;
  g_return_if_fail (child != NULL);
  child_bi = child->beat_info;
  source_bpm = gdam_beat_info_get_bpm (&child_bi, 0);
  new_rate = new_bpm / source_bpm;
  if (new_rate > 0)
    gdam_respeed_set_rate (respeed, new_rate);
}


static gboolean gdam_respeed_skin_tempo_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkLabel *label = GTK_LABEL (object);
  gtk_signal_connect_object (GTK_OBJECT (skin->model),
			     "tempo_changed",
			     GTK_SIGNAL_FUNC (respeed_update_tempo),
			     GTK_OBJECT (label));
  return TRUE;
}

static gboolean gdam_respeed_skin_tempo_received_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gdam_widget_string_dest_set (GTK_WIDGET (object),
			       on_tempo_string_drop, "gdam-tempo-text", skin);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_respeed_skin_tempo_label",
	  gdam_respeed_skin_tempo_label_handler	},
	{ "gdam_respeed_skin_tempo_received",
	  gdam_respeed_skin_tempo_received_handler	},
};


static void gdam_respeed_skin_class_init(GdamRespeedSkinClass* klass)
{
  GdamSkinClass *skin_class = GDAM_SKIN_CLASS (klass);
  skin_class->model_type = GDAM_TYPE_RESPEED;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}

