/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamsamplepickerskin.h"
#include <gtk/gtkoptionmenu.h>
#include <gtk/gtkmenuitem.h>
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>
#include <string.h>
#include <component/gdambuffermanager.h>
#include "gdamhistory.h"
#include "gdamskinutil.h"

static void gdam_sample_picker_skin_class_init(GdamSkinClass* skin_class);


static void gdam_sample_picker_skin_init(GdamSamplePickerSkin* sample_picker_skin)
{
}

GtkType gdam_sample_picker_skin_get_type()
{
  static GtkType sample_picker_skin_type = 0;
  if (!sample_picker_skin_type)
    {
      static const GtkTypeInfo sample_picker_skin_info = {
	"GdamSamplePickerSkin",
	sizeof (GdamSamplePickerSkin),
	sizeof (GdamSamplePickerSkinClass),
	(GtkClassInitFunc) gdam_sample_picker_skin_class_init,
	(GtkObjectInitFunc) gdam_sample_picker_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      sample_picker_skin_type = gtk_type_unique (parent,
						 &sample_picker_skin_info);
    }
  return sample_picker_skin_type;
}

typedef struct _StartedCachingStruct StartedCachingStruct;
struct _StartedCachingStruct {
	GdamChannel*     channel;
	char*            description;
	int              buffer_id;
};

static void finished_caching(gpointer user_data)
{
  StartedCachingStruct *cache_info = (StartedCachingStruct *) user_data;
  GdamBufferManagerInfo *info;
  gdam_debug ("sample_picker: finished_caching");
  info = g_new0 (GdamBufferManagerInfo, 1);
  info->buffer_id = cache_info->buffer_id;
  info->description = cache_info->description;
  info->channel = cache_info->channel;
  gdam_buffer_manager_add_buffer (info);
  g_free (cache_info);
}

static void started_caching(int buffer_id,
                            int connection_id,
			    gpointer user_data)
{
  StartedCachingStruct *cache_info = (StartedCachingStruct *) user_data;
  cache_info->buffer_id = buffer_id;
  gdam_debug ("sample_picker: started_caching");
  gdam_channel_register_handler (cache_info->channel,
				 gdam_channel_simple_handler_new
				 (connection_id, TRUE, finished_caching,
				  cache_info));
}

static void failed_to_start_caching(gpointer user_data)
{
  StartedCachingStruct *cache_info = (StartedCachingStruct *) user_data;
  g_warning ("failed to start caching %s", cache_info->description);
  g_free (cache_info->description);
  g_free (cache_info);
}
static void created_wav_source(int wav_source_id, gpointer user_data)
{
  StartedCachingStruct *cache_info = user_data;
  gdam_debug ("sample_picker: created_wav_source");
  gdam_channel_append_request (cache_info->channel,
			       gdam_request_start_caching (wav_source_id,
							   started_caching,
							   failed_to_start_caching,
							   cache_info));
}
static void failed_to_create_wav(gpointer user_data)
{
  StartedCachingStruct *cache_info = user_data;
  g_warning ("error creating %s", cache_info->description);
  g_free (cache_info->description);
  g_free (cache_info);
}
static void start_caching(GdamSamplePickerSkin* picker_skin)
{
  StartedCachingStruct *cache_info;
  char *filename;
  GdamSkin *skin = GDAM_SKIN (picker_skin);
  gdam_debug ("sample_picker: start_caching");
  if (picker_skin->file_selection == NULL)
    {
      g_warning ("no file selection in this skin");
      return;
    }
  filename = gtk_file_selection_get_filename (picker_skin->file_selection);
  if (filename == NULL)
    {
      g_warning ("no filename in selection");
      return;
    }
  cache_info = g_new (StartedCachingStruct, 1);
  cache_info->channel = skin->channel;
  cache_info->description = g_strdup_printf ("Wav %s", filename);
  gdam_channel_append_request (skin->channel,
			       gdam_request_create_source ("GdasWav",
							   0,
							   created_wav_source,
							   failed_to_create_wav,
							   cache_info,
							   1,
							   "filename",
							   GDAM_ARG_TYPE_STRING,
							   filename));

}
static void set_last_sample_dir(GdamSamplePickerSkin* skin)
{
  const char *filename;
  /*
     const char* end_dir;
   */
  const char *dirname;
  filename = gtk_file_selection_get_filename (skin->file_selection);
  g_return_if_fail (filename != NULL);
  /*
     end_dir = strrchr(filename, '/');
     dirname = (char*) alloca(end_dir - filename + 1);
     memcpy(dirname, filename, end_dir - filename);
     dirname[end_dir - filename] = 0;
   */
  dirname = filename;
  gdam_history_set_entry ("SamplePicker", "LastPath", dirname);
}

static gboolean gdam_sample_picker_file_selection_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkFileSelection *file_sel = GTK_FILE_SELECTION (object);
  GdamSamplePickerSkin *picker_skin = GDAM_SAMPLE_PICKER_SKIN (skin);
  const char *last_dir;
  picker_skin->file_selection = file_sel;
  last_dir = gdam_history_get_entry ("SamplePicker", "LastPath");
  if (last_dir)
    gtk_file_selection_set_filename (file_sel, last_dir);
  gtk_signal_connect_object (GTK_OBJECT (file_sel->ok_button),
			     "clicked",
			     GTK_SIGNAL_FUNC (set_last_sample_dir),
			     GTK_OBJECT (skin));
  return TRUE;
}
static gboolean gdam_sample_picker_click_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (start_caching),
			     GTK_OBJECT (skin));
  return TRUE;
}


static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_sample_picker_file_selection",
	   gdam_sample_picker_file_selection_handler },
	{ "gdam_sample_picker_click",
	   gdam_sample_picker_click_handler }			};

static void gdam_sample_picker_skin_class_init(GdamSkinClass* skin_class)
{
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
