/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_SMOOTH_FUNCTION_H_
#define __GDAM_SMOOTH_FUNCTION_H_

typedef struct _GdamSmoothFunction GdamSmoothFunction;
typedef struct _GdamSmoothFunctionClass GdamSmoothFunctionClass;
typedef struct _GdamKeypointInfo GdamKeypointInfo;

#include <gtk/gtkdrawingarea.h>
#include <common/gdamspline.h>

struct _GdamKeypointInfo {
	gfloat			pos[2];
	guint			is_movable : 1;
	guint			is_removable : 1;
};

GtkType gdam_smooth_function_get_type();
#define GDAM_TYPE_SMOOTH_FUNCTION			(gdam_smooth_function_get_type ())
#define GDAM_SMOOTH_FUNCTION(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_SMOOTH_FUNCTION, GdamSmoothFunction))
#define GDAM_SMOOTH_FUNCTION_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_SMOOTH_FUNCTION, GdamSmoothFunctionClass))
#define GDAM_IS_SMOOTH_FUNCTION(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_SMOOTH_FUNCTION))
#define GDAM_IS_SMOOTH_FUNCTION_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_SMOOTH_FUNCTION))

struct _GdamSmoothFunctionClass {
	GtkDrawingAreaClass		drawing_area_class;
};
/*
 * Basically, a 1-d spline which maps from x (the spline parameter) to y.
 */

struct _GdamSmoothFunction {
	GtkDrawingArea		graph;
	gfloat			min_input;
	gfloat			max_input;
	gfloat			min_output;
	gfloat			max_output;
	int			draw_width;
	int			draw_height;

	/* should be x0,y0,x1,y1,... */
	int			num_keypoints;
	GdamKeypointInfo*	keypoints;

	/* taughtness of the GdamSpline */
	gfloat			taughtness;

	/* keypoint the user is dragging, or -1 */
	int			drag_keypoint;
	int			drag_offset_x;
	int			drag_offset_y;

	/* radius of a keypoint */
	int			keypoint_radius;

	GdamSpline*		function;

	int			num_cached_points;
	GdkPoint*		cached_points;
	gboolean		cached_points_invalid;
};

#endif
