/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamsongselectorskin.h"
#include <component/gdammodel.h>
#include "gdamdnd.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkfilesel.h>
#include "gdamskinutil.h"


GdamSongSelectorSkin* global_songsel = NULL;

typedef struct _CListInfo CListInfo;
struct _CListInfo {
	GdamSongSelectorSkin*       skin;

	/* We reference count the clist. */
	GtkCList*                  clist;
	int			   selected_connect_id;
	int			   realize_connect_id;
	int			   resize_column_connect_id;
	int			   unselected_connect_id;
	int			   button_press_connect_id;
	int			   size_alloc_connect_id;
	int			   click_column_connect_id;

	int                        num_columns;
	//CListEntryGetter*	   column_fillers;
};

static GtkObjectClass* parent_class;


/*   ____  _    _         ____  _          __  __ 
 *  / ___|| | _(_)_ __   / ___|| |_ _   _ / _|/ _|
 *  \___ \| |/ / | '_ \  \___ \| __| | | | |_| |_ 
 *   ___) |   <| | | | |  ___) | |_| |_| |  _|  _|
 *  |____/|_|\_\_|_| |_| |____/ \__|\__,_|_| |_|  
 */

static void gdam_song_selector_skin_destroy(GtkObject* object);

static void gdam_song_selector_skin_init(GdamSongSelectorSkin* skin)
{
  skin->database = gdam_song_database_system ();
  skin->last_w = 0;
  skin->last_h = 0;
  skin->amVisible = FALSE;
  skin->sort_by = GDAM_SORT_BY_FILENAME;
  skin->current = NULL;
  skin->max_bpm = 10000;
  skin->save_unknown_bpm = TRUE;
}

static void gdam_song_selector_skin_class_init(GdamSkinClass* skin_class);

GtkType gdam_song_selector_skin_get_type()
{
  static GtkType song_selector_skin_type = 0;
  if (!song_selector_skin_type)
    {
      static const GtkTypeInfo song_selector_skin_info = {
	"GdamSongSelectorSkin",
	sizeof (GdamSongSelectorSkin),
	sizeof (GdamSongSelectorSkinClass),
	(GtkClassInitFunc) gdam_song_selector_skin_class_init,
	(GtkObjectInitFunc) gdam_song_selector_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      parent_class = gtk_type_class (parent);
      song_selector_skin_type = gtk_type_unique (parent,
						 &song_selector_skin_info);
    }
  return song_selector_skin_type;
}

void gdam_song_selector_skin_remove(GdamSongSelectorSkin* songsel, const char* fname);
void gdam_song_selector_skin_select(GdamSongSelectorSkin* songsel, const char* fname);

void gdam_song_selector_skin_update_table(GdamSongSelectorSkin* songsel);
static void sort_table_by_column(GtkWidget* wid, int idx, 
		 	        GdamSongSelectorSkin* sel_skin);

static void song_selected(GtkWidget*      clist,
                          gint            row,
			  gint            col,
			  GdkEventButton* event,
                          CListInfo* info)
{
  GdamSongSelectorSkin *skin = info->skin;
  if (event && (event->state & GDK_SHIFT_MASK))
    {
    }
  else
    {
      const char *txt;
      GdamSongInfo *si;
      gpointer data;
      data = gtk_clist_get_row_data (GTK_CLIST (info->clist), row);
      si = (GdamSongInfo *) data;
      txt = gdam_song_info_get_filename (si);
      gdam_song_selector_skin_select (skin, txt);
    }
}

static gboolean song_unselected(GtkWidget* clist,
                            gint row,
		            gint col,
		            GdkEventButton* event,
                            CListInfo* info)
{
  if (event == NULL)
    return FALSE;
  return FALSE;
}

void songsel_skin_size_allocate(GtkWidget* wid,GtkAllocation* a,
				GdamSongSelectorSkin* skin)
{
  int n;
  int first_expander = -1;
  int free_size = a->width - 40;
  //if (!skin->respond_to_size_alloc)
  //return;
  for (n = 0; n < skin->scrlist->columns; n++)
    {
      if (skin->column_expands[n] && first_expander < 0)
	{
	  first_expander = n;
	}
      else if (skin->column_widths[n] > 0)
	{
	  int width = skin->column_widths[n];
	  gtk_clist_set_column_width (skin->scrlist, n, width);
	  free_size -= width;
	}
    }
  if (first_expander >= 0)
    {
      gtk_clist_set_column_width (skin->scrlist, first_expander, free_size);
    }
}

void songsel_skin_clist_realize(GtkCList* clist, GdamSongSelectorSkin* skin)
{
  int n;
  for (n = 0; n < skin->scrlist->columns; n++)
    {
      skin->column_widths[n] = skin->scrlist->column[n].width;
    }
  /* TODO: will this resize columns at start (we hope?) */
  gtk_widget_queue_resize (GTK_WIDGET (skin->scrlist));
}

void songsel_skin_column_resized(GtkCList* clist, int column, int width, 
				 GdamSongSelectorSkin* skin)
{
  skin->column_widths[column] = width;
}

/*static const char* filename_drag_xpm[] = {
"32 32 3 1",
"       c None",
".      c #000000",
"+      c #FFFFFF",
"                                ",
"                                ",
"                ..              ",
"              ..+.              ",
"             ..++..             ",
"           ...++++.             ",
"         ...++++++..            ",
"       ...+++++++++.            ",
"     ...+++++++++++..           ",
"    ..+.++++++++ +++..          ",
"     .++.++++++.+ +++..         ",
"     .+++.+++..+++ +++..        ",
"     .++++.+.++++++ +++.        ",
"     .+++.+.+++++++ +++..       ",
"     .++.+++++++++++ +++..      ",
"     .+.+++++++++++++ +++..     ",
"     ..+++++++++++++.+++++..    ",
"     ..+++++++++++..+++++++.    ",
"     .+++++++++...++++++++..    ",
"     ..++++++++..+++++++..      ",
"      .++++++++++++++++..       ",
"      ..+++++++++++++...        ",
"       .++++++++++++..          ",
"       ..+++++++++..            ",
"        .++++++++..             ",
"        ..++++++..              ",
"         .+++++..               ",
"          .++..                 ",
"           ...                  ",
"           ..                   ",
"                                ",
"                                "};
*/


static void on_scrlist_filename_drag(GtkSelectionData* 	selection_data,
				     gpointer 		data)
{
  CListInfo *info = (CListInfo *) data;
  GtkCList *clist = info->clist;
  GList *selected;
  char str[16384] = "";
  selected = clist->selection;

  while (selected)
    {
      int num = GPOINTER_TO_INT (selected->data);
      gpointer data = gtk_clist_get_row_data (clist, num);
      GdamSongInfo *si = (GdamSongInfo *) data;
      strcat (str, gdam_song_info_get_filename (si));
      strcat (str, "\n");
      selected = selected->next;
    }

  gtk_selection_data_set (selection_data, selection_data->target, 8,
			  (guchar *) str, strlen (str));
}

/* TODO: handle concatenated filename lists such as those we produce. */
static void on_scrlist_filename_drop (const char* filename_str,
				      gpointer	user_data)
{
  CListInfo *info = (CListInfo *) user_data;
  GdamSongSelectorSkin *skin = info->skin;
  GdamSongInfo *si;
  GdamSongDatabase *songdb = gdam_song_database_system ();
  gboolean keep_going = TRUE;
  char *cur_file;
  char *filenames = g_strdup (filename_str);
  char *filename;
  char *nl;
  g_return_if_fail (songdb != NULL);

  cur_file = filenames;
  gdam_song_database_freeze (skin->database);
  while (keep_going && *cur_file != '\0')
    {
      nl = strchr (cur_file, '\n');
      if (nl)
	*nl = '\0';
      else
	keep_going = FALSE;
      filename = cur_file;
      si = gdam_song_database_find (songdb, filename);
      if (!si)
	g_warning ("couldn't find songinfo for %s", filename);
      else
	{
	  if (skin->database != songdb)
	    {
	      gdam_debug ("adding %s to %p", filename, skin->database);
	      gdam_song_database_add (skin->database,
				      gdam_song_info_copy (si));
	    }
	  gdam_debug ("declining to add %s", filename);
	}
      cur_file = nl + 1;
    }
  gdam_song_database_thaw (skin->database);
}

static int scrlist_button_press(GtkWidget*      widget,
                                GdkEventButton* event, 
	                        CListInfo*      info)
{
  GdamSongSelectorSkin *skin = info->skin;
  if (skin == NULL)
    return FALSE;
  return TRUE;
}

static gint songsel_destroy_event()
{
  return FALSE;
}

GdamSongSelectorSkin* gdam_song_selector_skin_new(GdamSongDatabase* database)
{
  GdamSongSelectorSkin *rv = g_new (GdamSongSelectorSkin, 1);

  gtk_signal_connect (GTK_OBJECT (rv->toplevel),
		      "delete_event",
		      GTK_SIGNAL_FUNC (songsel_destroy_event), rv);

  return rv;
}

static void set_row_song_info(GtkCList* clist,
                              int index,
			      GdamSongInfo* song_info)
{
  GdamSongInfo *old;
  old = gtk_clist_get_row_data (clist, index);
  if (old == song_info)
    return;
  if (old != NULL)
    gdam_song_info_unref (old);
  if (song_info != NULL)
    gdam_song_info_ref (song_info);
  gtk_clist_set_row_data (clist, index, song_info);
}

static gboolean custom_decide_func(GdamSongInfo* si, gpointer ptr)
{
  GdamSongSelectorSkin *skin = GDAM_SONG_SELECTOR_SKIN (ptr);
  const GdamBeatInfo *bi;
  gdouble bpm;
  gboolean visible;
  const char *show_key = NULL;
  const char *show_value = NULL;

  visible = TRUE;
  g_return_val_if_fail (si != NULL, 0);
  bi = gdam_song_info_get_beat_info (si);
  bpm = bi ? gdam_beat_info_get_bpm (bi, 0) : 0;
  if (skin->show_key_entry)
    show_key = gtk_entry_get_text (skin->show_key_entry);
  if (skin->show_value_entry)
    show_value = gtk_entry_get_text (skin->show_value_entry);

  if (skin->hide_default)
    visible = FALSE;
  if (bpm <= 0)
    {
      if (skin->hide_unknown_bpm)
	visible = FALSE;
      else
	visible = TRUE;
    }
  if (skin->hide_songdb)
    {
      if (gdam_song_database_find (skin->hide_songdb,
				   gdam_song_info_get_filename (si)))
	visible = FALSE;
    }
  if (skin->show_songdb)
    {
      if (gdam_song_database_find (skin->show_songdb,
				   gdam_song_info_get_filename (si)))
	visible = TRUE;
    }
  if (show_key)
    {
      GHashTable *props;
      props = (GHashTable *) gdam_song_info_get_properties (si);
      if (props)
	{
	  const char *value;
	  value = g_hash_table_lookup (props, show_key);
	  if (value)
	    {
	      if (show_value && strcmp (value, show_value) == 0)
		visible = TRUE;
	      else if (show_value == NULL)
		visible = TRUE;
	      else if (strcmp (show_value, "") == 0)
		visible = TRUE;
	    }

	}
    }
  if (bpm < skin->min_bpm)
    {
      visible = FALSE;
    }
  if (bpm > skin->max_bpm)
    {
      visible = FALSE;
    }

  return visible;
}


/* Traverse func used to refresh the clist after the song database has changed*/
static int 
gdam_songsel_skin_traverse_func(gpointer key, gpointer value, gpointer data)
{
  GdamSongInfo *si = (GdamSongInfo *) value;
  GdamSongSelectorSkin *skin = (GdamSongSelectorSkin *) data;
  GtkCList *clist = skin->scrlist;
  gboolean visible;
  char **strings;
  int i, new_idx;
  int num_columns = clist->columns;

  visible = custom_decide_func (si, skin);

  if (!visible)
    return 0;

  strings = (char **) alloca (sizeof (char *) * num_columns);
  for (i = 0; i < num_columns; i++)
    {
      strings[i] = (*skin->column_fillers[i]) (si);
    }
  new_idx = gtk_clist_append (clist, strings);
  set_row_song_info (clist, new_idx, si);

  return 0;
}

void gdam_song_selector_skin_update_table(GdamSongSelectorSkin* sel_skin)
{
  GList *selected_songs = NULL;
  GList *selected;
  GList *selsong;
  int move_to;

  if (!(sel_skin->database))
    {
      g_message ("update_table called w/o database, creating...");
      sel_skin->database = gdam_song_database_system ();
    }
  g_return_if_fail (sel_skin->database != NULL);

  gdam_debug ("Songdb at %p changed", sel_skin->database);

  /* Save selection state. */
  selected = sel_skin->scrlist->selection;
  while (selected)
    {
      int num = GPOINTER_TO_INT (selected->data);
      gpointer data = gtk_clist_get_row_data (sel_skin->scrlist, num);
      selected_songs = g_list_append (selected_songs, data);
      selected = selected->next;
    }

  gtk_clist_freeze (sel_skin->scrlist);
  gtk_clist_clear (sel_skin->scrlist);
  if (sel_skin->sort_by == GDAM_SORT_BY_ORDER)
    {
      g_tree_traverse (sel_skin->database->by_order,
		       gdam_songsel_skin_traverse_func, G_IN_ORDER, sel_skin);
    }
  else if (sel_skin->sort_by == GDAM_SORT_BY_FILENAME)
    {
      gdam_debug ("sorting by name");
      g_tree_traverse (sel_skin->database->by_name,
		       gdam_songsel_skin_traverse_func, G_IN_ORDER, sel_skin);
    }
  else if (sel_skin->sort_by == GDAM_SORT_BY_BPM)
    {
      g_tree_traverse (sel_skin->database->by_bpm,
		       gdam_songsel_skin_traverse_func, G_IN_ORDER, sel_skin);
    }
  else
    {
      g_tree_traverse (sel_skin->database->by_order,
		       gdam_songsel_skin_traverse_func, G_IN_ORDER, sel_skin);
    }
  gtk_clist_thaw (sel_skin->scrlist);

  /* Restore selection state. */
  selsong = selected_songs;
  if (selected_songs)
    move_to = gtk_clist_find_row_from_data (sel_skin->scrlist, selsong->data);
  while (selsong)
    {
      int num = gtk_clist_find_row_from_data (sel_skin->scrlist,
					      selsong->data);
      if (num >= 0)
	gtk_clist_select_row (sel_skin->scrlist, num, -1);
      selsong = selsong->next;
    }
  if (selected_songs)
    gtk_clist_moveto (sel_skin->scrlist, move_to, 0, 0.5, 0);
  g_list_free (selected_songs);
  g_list_free (selected);
}


void gdam_song_selector_skin_remove(GdamSongSelectorSkin* songsel, const char* fname)
{
  g_warning ("songsel remove not complete");
}

static void compile_props(gpointer key, gpointer value, gpointer cbuf)
{
  char buf[128];
  sprintf (buf, "%s=%s  ", (char *) key, (char *) value);
  strcat (cbuf, buf);
}

void gdam_song_selector_skin_select(GdamSongSelectorSkin* songsel, const char* fname)
{

  if (!songsel->database)
    {
      g_message ("skin_select called w/o database, creating...");
      songsel->database = gdam_song_database_system ();
    }
  g_return_if_fail (songsel->database != NULL);

  songsel->current = gdam_song_database_find (songsel->database, fname);

  if (!songsel->current)
    return;
  if (songsel->info_label)
    {
      const GHashTable *prop_table;
      const GList *list;
      char cbuf[2048] = "";
      prop_table = gdam_song_info_get_properties (songsel->current);
      if (prop_table)
	{
	  g_hash_table_foreach ((GHashTable *) prop_table,
				compile_props, cbuf);
	}
      list = gdam_song_info_get_loops (songsel->current);
      if (list)
	{
	  char buf[16];
	  sprintf (buf, "%d loops", g_list_length ((GList *) list));
	}
      gtk_label_set_text (songsel->info_label, cbuf);
    }
}

static void gdam_song_selector_skin_destroy(GtkObject* object)
{
#if 0
  GdamBufferManagerSkin *skin = GDAM_BUFFER_MANAGER_SKIN (object);
  g_slist_foreach (skin->clist_list, disconnect_clist, NULL);
  g_slist_free (skin->clist_list);
  skin->clist_list = NULL;
  (*parent_class->destroy) (object);
#endif
  g_warning ("TODO: must unref songinfos");
}

static CListEntryGetter get_column_filler(const char* title);
static GtkJustification get_column_justification(const char* title);
static gboolean get_column_expand(const char* title);
static GdamSorting get_column_sorting(const char* title);

static void connect_clist(GdamSongSelectorSkin*  skin,
                          char**                 column_names,
			  GtkCList*              clist)
{
  int i;
  int id;
  int n;
  CListInfo *info = g_new (CListInfo, 1);
  skin->scrlist = clist;
  info->skin = skin;
  info->clist = clist;
  info->num_columns = clist->columns;

  /* Connect to relevant clist signals. */
  id = gtk_signal_connect (GTK_OBJECT (clist),
			   "select_row",
			   GTK_SIGNAL_FUNC (song_selected), info);
  info->selected_connect_id = id;
  id = gtk_signal_connect (GTK_OBJECT (clist),
			   "unselect_row",
			   GTK_SIGNAL_FUNC (song_unselected), info);
  info->unselected_connect_id = id;
  //id = gtk_signal_connect(GTK_OBJECT(clist), 
  //"button_press_event", 
  //GTK_SIGNAL_FUNC(scrlist_button_press), 
  //info);
  //info->button_press_connect_id = id;
  id = gtk_signal_connect (GTK_OBJECT (clist),
			   "size_allocate",
			   GTK_SIGNAL_FUNC (songsel_skin_size_allocate),
			   skin);
  info->size_alloc_connect_id = id;
  id = gtk_signal_connect (GTK_OBJECT (clist),
			   "click_column",
			   GTK_SIGNAL_FUNC (sort_table_by_column), skin);
  info->click_column_connect_id = id;
  id = gtk_signal_connect (GTK_OBJECT (clist),
			   "resize_column",
			   GTK_SIGNAL_FUNC (songsel_skin_column_resized),
			   skin);
  info->resize_column_connect_id = id;
  id = gtk_signal_connect (GTK_OBJECT (clist),
			   "realize",
			   GTK_SIGNAL_FUNC (songsel_skin_clist_realize),
			   skin);
  info->realize_connect_id = id;

  /* Set drag-n-drop options */
  gdam_widget_string_dest_set (GTK_WIDGET (clist),
			       on_scrlist_filename_drop,
			       "gdam-filename-text", info);
  gdam_widget_string_source_set (GTK_WIDGET (clist),
				 on_scrlist_filename_drag,
				 "gdam-filename-text", info);
  /*  Match up column titles with functions which fill them. */

  skin->column_fillers = g_new (CListEntryGetter, info->num_columns);
  skin->column_widths = g_new0 (int, info->num_columns);
  skin->column_sortings = g_new (GdamSorting, info->num_columns);
  skin->column_expands = g_new (gboolean, info->num_columns);
  for (i = 0; i < info->num_columns; i++)
    {
      char *title = column_names[i];
      GtkJustification just;
      if (title == NULL)
	break;
      gdam_debug ("filler for column %d based on %s", i, title);
      skin->column_fillers[i] = get_column_filler (title);
      skin->column_sortings[i] = get_column_sorting (title);
      skin->column_expands[i] = get_column_expand (title);
      just = get_column_justification (title);
      gtk_clist_set_column_justification (clist, i, just);
    }
  for (; i < info->num_columns; i++)
    skin->column_fillers[i] = get_column_filler (NULL);
  /* Reference the CList */
  gtk_object_ref (GTK_OBJECT (clist));
  /* Fill the clist with songs */
  gdam_song_selector_skin_update_table (skin);
  /* Collect widths.  */
  for (n = 0; n < skin->scrlist->columns; n++)
    {
      skin->column_widths[n] = skin->scrlist->column[n].width;
    }
#if 0
  skin->clist_list = g_slist_prepend (skin->clist_list, info);
#endif
}

static gboolean is_blank(const char* string)
{
  while (*string && isspace (*string))
    string++;
  return *string == 0 ? TRUE : FALSE;
}

static void save_database(GdamSongSelectorSkin* skin)
{
  const char *filename;
  g_return_if_fail (skin != NULL);
  g_return_if_fail (skin->filename_entry != NULL);
  filename = gtk_entry_get_text (skin->filename_entry);
  g_return_if_fail (filename != NULL && !is_blank (filename));

  g_return_if_fail (skin->database != NULL);
  if (!skin->save_unknown_bpm)
    gdam_song_database_write_ext (skin->database, filename,
				  custom_decide_func, skin);
  else
    gdam_song_database_write (skin->database, filename);
}

static void load_database(GdamSongSelectorSkin* skin)
{
  const char *filename;
  g_return_if_fail (skin != NULL);
  g_return_if_fail (skin->filename_entry != NULL);
  filename = gtk_entry_get_text (skin->filename_entry);
  g_return_if_fail (filename != NULL && !is_blank (filename));

  if (skin->database)
    gtk_object_destroy (GTK_OBJECT (skin->database));
  skin->database = gdam_song_database_new_from_file (filename);
  gdam_song_selector_skin_update_table (skin);
}

static void save_info(GdamSongSelectorSkin* skin)
{
  char info[2048];
  char *p;
  char *key;
  char *val;
  g_return_if_fail (skin != NULL);
  g_return_if_fail (skin->info_entry != NULL);
  sprintf (info, gtk_entry_get_text (skin->info_entry));
  g_return_if_fail (info != NULL && !is_blank (info));
  if (!skin->current)
    return;
  p = strchr (info, '=');
  if (!p)
    return;
  *p = '\0';
  key = info;
  val = p + 1;
  if (val && !is_blank (val))
    {
      gdam_debug ("parsed key %s, val %s", key, val);
      gdam_song_info_add_property (skin->current, key, val);
    }
  else
    gdam_song_info_add_property (skin->current, key, NULL);
  gdam_song_selector_skin_select (skin,
				  gdam_song_info_get_filename (skin->
							       current));
  gtk_entry_set_text (skin->info_entry, "");
}


static void on_sort_by_bpm(GdamSongSelectorSkin* skin)
{
  skin->sort_by = GDAM_SORT_BY_BPM;
  gdam_song_selector_skin_update_table (skin);
}

static void on_hide_unknown(GdamSongSelectorSkin* skin)
{
  skin->hide_unknown_bpm = skin->hide_unknown_bpm ? FALSE : TRUE;
  if (skin->hide_unknown_tbutton)
    gtk_toggle_button_set_active (skin->hide_unknown_tbutton,
				  skin->hide_unknown_bpm);
  gdam_song_selector_skin_update_table (skin);
}

static void hide_unknown_tbutton_toggled(GdamSongSelectorSkin* skin){
	gboolean hide;
	g_return_if_fail(skin->hide_unknown_tbutton != NULL);
	hide = gtk_toggle_button_get_active(skin->hide_unknown_tbutton);
	if (hide == skin->hide_unknown_bpm)
		return;
	skin->hide_unknown_bpm = hide;
	gdam_song_selector_skin_update_table(skin);
}

static void save_unknown_tbutton_toggled(GdamSongSelectorSkin* skin){
	gboolean save;
	g_return_if_fail(skin->save_unknown_tbutton != NULL);
	save = gtk_toggle_button_get_active(skin->save_unknown_tbutton);
	if (save == skin->save_unknown_bpm)
		return;
	skin->save_unknown_bpm = save;
}

static void hide_default_tbutton_toggled(GdamSongSelectorSkin* skin){
	gboolean hide;
	g_return_if_fail(skin->hide_default_tbutton != NULL);
	hide = gtk_toggle_button_get_active(skin->hide_default_tbutton);
	skin->hide_default = hide;
	gdam_song_selector_skin_update_table(skin);
}

static void on_delete(GdamSongSelectorSkin* skin)
{
  GtkCList *clist = skin->scrlist;
  GList *sel;
  sel = skin->scrlist->selection;

  gdam_debug ("deleting %d songs from database", g_list_length (sel));

  while (sel)
    {
      int num = GPOINTER_TO_INT (sel->data);
      gpointer data = gtk_clist_get_row_data (clist, num);
      GdamSongInfo *si = (GdamSongInfo *) data;
      gdam_debug ("deleting song %d from database", num);
      sel = sel->next;
      gdam_song_database_remove (skin->database,
				 gdam_song_info_get_filename (si));
    }

  gdam_song_selector_skin_update_table (skin);
}

static void max_bpm_adj_changed(GtkAdjustment* adj, gpointer ptr)
{
  GdamSongSelectorSkin *skin;
  skin = GDAM_SONG_SELECTOR_SKIN (ptr);
  skin->max_bpm = adj->value;
  gdam_song_selector_skin_update_table (skin);
}

static void min_bpm_adj_changed(GtkAdjustment* adj, gpointer ptr)
{
  GdamSongSelectorSkin *skin;
  skin = GDAM_SONG_SELECTOR_SKIN (ptr);
  skin->min_bpm = adj->value;
  gdam_song_selector_skin_update_table (skin);
}

static void on_find(GdamSongSelectorSkin* skin)
{
  const char *find;
  int n;
  int found = -1;
  int first_found = -1;
  GdamSongInfo *info = NULL;
  GdamSongInfo *first_info = NULL;
  GtkCList *clist = skin->scrlist;
  g_return_if_fail (skin->find_entry != NULL);
  find = gtk_entry_get_text (skin->find_entry);
  g_return_if_fail (find != NULL && !is_blank (find));

  for (n = 0; n < clist->rows; n++)
    {
      GdamSongInfo *si;
      gpointer data;
      data = gtk_clist_get_row_data (clist, n);
      si = (GdamSongInfo *) data;
      if (strstr (gdam_song_info_get_filename (si), find))
	{
	  if (n <= skin->last_found && first_found < 0)
	    {
	      first_found = n;
	      first_info = si;
	    }
	  else if (n > skin->last_found)
	    {
	      found = n;
	      info = si;
	      break;
	    }

	}
    }

  if (found < 0)
    {
      found = first_found;
      info = first_info;
    }
  if (found < 0)
    {
      return;
    }

  g_return_if_fail (info != NULL);

  gtk_clist_moveto (clist, found, 0, 0.5, 0);
  skin->last_found = found;
}

typedef struct _SourceInfo {
    	GtkFileSelection* filesel;
	GdamSkin* skin;
} SourceInfo;

static void do_source_hide_file(SourceInfo* source_info)
{
  const char *filename;
  GdamSongSelectorSkin *skin;
  skin = GDAM_SONG_SELECTOR_SKIN (source_info->skin);
  filename = gtk_file_selection_get_filename (source_info->filesel);
  if (skin->hide_file_entry)
    {
      char text[16384] = "";
      strcat (text, gtk_entry_get_text (skin->hide_file_entry));
      strcat (text, ", ");
      strcat (text, filename);
      gtk_entry_set_text (skin->hide_file_entry, text);
    }
  skin->hide_songdb = gdam_song_database_add_from_file (skin->hide_songdb,
							filename);
  gdam_song_selector_skin_update_table (skin);
  gtk_object_destroy (GTK_OBJECT (source_info->filesel));
}

static void do_reset_hide_file(GdamSongSelectorSkin* skin)
{
  if (skin->hide_songdb)
    gtk_object_destroy (GTK_OBJECT (skin->hide_songdb));
  skin->hide_songdb = NULL;
  if (skin->hide_file_entry)
    {
      char text[4] = "";
      gtk_entry_set_text (skin->hide_file_entry, text);
    }
  gdam_song_selector_skin_update_table (skin);
}

static void do_reset_show_file(GdamSongSelectorSkin* skin)
{
  if (skin->show_songdb)
    gtk_object_destroy (GTK_OBJECT (skin->show_songdb));
  skin->show_songdb = NULL;
  if (skin->show_file_entry)
    {
      char text[4] = "";
      gtk_entry_set_text (skin->show_file_entry, text);
    }
  gdam_song_selector_skin_update_table (skin);
}

static void pop_up_hide_filesel(GdamSkin* skin)
{
  GtkFileSelection *filesel;
  SourceInfo *si;
  filesel = (GtkFileSelection *)
    gtk_file_selection_new ("Pick a song database of songs to hide");
  si = g_new (SourceInfo, 1);
  si->filesel = filesel;
  si->skin = skin;
  gtk_object_ref ((GtkObject *) filesel);
  gtk_object_sink ((GtkObject *) filesel);
  /* TODO: close filesel on cancel, skin destroy */
  gtk_signal_connect_full (GTK_OBJECT (filesel->ok_button),
			   "clicked",
			   GTK_SIGNAL_FUNC (do_source_hide_file),
			   NULL, si, g_free, TRUE, FALSE);
  gtk_signal_connect_object_while_alive
    (GTK_OBJECT (filesel->cancel_button),
     "clicked", GTK_SIGNAL_FUNC (gtk_object_destroy), GTK_OBJECT (filesel));
  gtk_signal_connect_object_while_alive
    (GTK_OBJECT (skin),
     "destroy", GTK_SIGNAL_FUNC (gtk_object_destroy), GTK_OBJECT (filesel));
  gtk_widget_show (GTK_WIDGET (filesel));
}


static void do_source_show_file(SourceInfo* source_info)
{
  const char *filename;
  GdamSongSelectorSkin *skin;
  skin = GDAM_SONG_SELECTOR_SKIN (source_info->skin);
  filename = gtk_file_selection_get_filename (source_info->filesel);
  if (skin->show_file_entry)
    {
      char text[16384] = "";
      strcat (text, gtk_entry_get_text (skin->show_file_entry));
      strcat (text, ", ");
      strcat (text, filename);
      gtk_entry_set_text (skin->show_file_entry, text);
    }
  skin->show_songdb = gdam_song_database_add_from_file (skin->show_songdb,
							filename);
  gdam_song_selector_skin_update_table (skin);
  gtk_object_destroy (GTK_OBJECT (source_info->filesel));
}

static void pop_up_show_filesel(GdamSkin* skin)
{
  GtkFileSelection *filesel;
  SourceInfo *si;
  filesel = (GtkFileSelection *)
    gtk_file_selection_new ("Pick a song database of songs to show");
  si = g_new (SourceInfo, 1);
  si->filesel = filesel;
  si->skin = skin;
  gtk_object_ref ((GtkObject *) filesel);
  gtk_object_sink ((GtkObject *) filesel);
  /* TODO: close filesel on cancel, skin destroy */
  gtk_signal_connect_full (GTK_OBJECT (filesel->ok_button),
			   "clicked",
			   GTK_SIGNAL_FUNC (do_source_show_file),
			   NULL, si, g_free, TRUE, FALSE);
  gtk_signal_connect_object_while_alive
    (GTK_OBJECT (filesel->cancel_button),
     "clicked", GTK_SIGNAL_FUNC (gtk_object_destroy), GTK_OBJECT (filesel));
  gtk_signal_connect_object_while_alive
    (GTK_OBJECT (skin),
     "destroy", GTK_SIGNAL_FUNC (gtk_object_destroy), GTK_OBJECT (filesel));
  gtk_widget_show (GTK_WIDGET (filesel));
}


static gboolean idle_grab_focus(gpointer data)
{
  GdamSongSelectorSkin *skin;
  skin = GDAM_SONG_SELECTOR_SKIN (data);
  if (skin->find_entry)
    {
      gtk_widget_grab_focus (GTK_WIDGET (skin->find_entry));
      gtk_entry_set_text (skin->find_entry, "");
    }
  return FALSE;
}

void gdam_song_selector_grab_find_focus(GdamSongSelectorSkin* skin)
{
  if (GTK_WIDGET_HAS_FOCUS (skin->filename_entry))
    return;
  if (GTK_WIDGET_HAS_FOCUS (skin->find_entry))
    return;
  if (GTK_WIDGET_HAS_FOCUS (skin->info_entry))
    return;
  g_idle_add (idle_grab_focus, skin);
}

static gboolean gdam_song_selector_grab_find_focus_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     gdam_song_selector_grab_find_focus,
			     GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_save_database_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, save_database, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_load_database_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, load_database, GTK_OBJECT (skin));
  return TRUE;
}


static gboolean gdam_song_selector_find_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *songsel = GDAM_SONG_SELECTOR_SKIN (skin);
  songsel->find_entry = GTK_ENTRY (object);
  gtk_signal_connect_object (GTK_OBJECT (songsel->find_entry),
			     "activate", on_find, (GtkObject *) skin);
  return TRUE;
}

static gboolean gdam_song_selector_filename_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *songsel = GDAM_SONG_SELECTOR_SKIN (skin);
  songsel->filename_entry = GTK_ENTRY (object);
  gtk_entry_set_text (songsel->filename_entry, songsel->database->dbname);
  return TRUE;
}

static gboolean gdam_song_selector_info_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *songsel_skin;
  songsel_skin = GDAM_SONG_SELECTOR_SKIN (skin);
  songsel_skin->info_label = GTK_LABEL (object);
  return TRUE;
}

static gboolean gdam_song_selector_info_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *songsel = GDAM_SONG_SELECTOR_SKIN (skin);
  songsel->info_entry = GTK_ENTRY (object);
  gtk_signal_connect_object (GTK_OBJECT (songsel->info_entry),
			     "activate", save_info, (GtkObject *) skin);
  return TRUE;
}

static gboolean gdam_song_selector_find_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object, signal_name, on_find, GTK_OBJECT (skin));
  return TRUE;
}


static gboolean gdam_song_selector_min_bpm_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adj;
  adj = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adj),
		      "value_changed",
		      GTK_SIGNAL_FUNC (min_bpm_adj_changed), skin);
  return TRUE;
}

static gboolean gdam_song_selector_max_bpm_adjustment_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GtkAdjustment *adj;
  adj = util_get_widget_adjustment (GTK_WIDGET (object));
  gtk_signal_connect (GTK_OBJECT (adj),
		      "value_changed",
		      GTK_SIGNAL_FUNC (max_bpm_adj_changed), skin);
  return TRUE;
}


static gboolean gdam_song_selector_sort_by_bpm_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, on_sort_by_bpm, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_hide_unknown_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, on_hide_unknown, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_hide_unknown_bpm_tbutton_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_TOGGLE_BUTTON (object))
    {
      g_warning ("%s only valid with ToggleButton", handler_names[0]);
      return TRUE;
    }

  ssskin->hide_unknown_tbutton = GTK_TOGGLE_BUTTON (object);
  gtk_signal_connect_object (object,
			     "toggled",
			     hide_unknown_tbutton_toggled, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_save_unknown_bpm_tbutton_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_TOGGLE_BUTTON (object))
    {
      g_warning ("%s only valid with ToggleButton", handler_names[0]);
      return TRUE;
    }

  ssskin->save_unknown_tbutton = GTK_TOGGLE_BUTTON (object);
  gtk_signal_connect_object (object,
			     "toggled",
			     save_unknown_tbutton_toggled, GTK_OBJECT (skin));
  return TRUE;
}


static gboolean gdam_song_selector_show_key_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_ENTRY (object))
    {
      g_warning ("%s only valid with Entry", handler_names[0]);
      return TRUE;
    }
  ssskin->show_key_entry = GTK_ENTRY (object);
  return TRUE;
}

static gboolean gdam_song_selector_show_value_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_ENTRY (object))
    {
      g_warning ("%s only valid with Entry", handler_names[0]);
      return TRUE;
    }
  ssskin->show_value_entry = GTK_ENTRY (object);
  return TRUE;
}


static gboolean gdam_song_selector_show_file_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_ENTRY (object))
    {
      g_warning ("%s only valid with Entry", handler_names[0]);
      return TRUE;
    }
  ssskin->show_file_entry = GTK_ENTRY (object);
  return TRUE;
}

static gboolean gdam_song_selector_hide_file_entry_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_ENTRY (object))
    {
      g_warning ("%s only valid with Entry", handler_names[0]);
      return TRUE;
    }
  ssskin->hide_file_entry = GTK_ENTRY (object);
  return TRUE;
}

static gboolean gdam_song_selector_change_hide_file_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  gtk_signal_connect_object (object,
			     signal_name,
			     pop_up_hide_filesel, GTK_OBJECT (skin));
  return TRUE;
}


static gboolean gdam_song_selector_reset_hide_file_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  gtk_signal_connect_object (object,
			     signal_name,
			     do_reset_hide_file, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_reset_show_file_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  gtk_signal_connect_object (object,
			     signal_name,
			     do_reset_show_file, GTK_OBJECT (skin));
  return TRUE;
}


static gboolean gdam_song_selector_change_show_file_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  gtk_signal_connect_object (object,
			     signal_name,
			     pop_up_show_filesel, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_hide_default_tbutton_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin;
  ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!GTK_IS_TOGGLE_BUTTON (object))
    {
      g_warning ("%s only valid with ToggleButton", handler_names[0]);
      return TRUE;
    }

  ssskin->hide_default_tbutton = GTK_TOGGLE_BUTTON (object);
  gtk_signal_connect_object (object,
			     "toggled",
			     hide_default_tbutton_toggled, GTK_OBJECT (skin));
  return TRUE;
}


static gboolean gdam_song_selector_delete_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name, on_delete, GTK_OBJECT (skin));
  return TRUE;
}

static gboolean gdam_song_selector_clist_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  if (!GTK_IS_CLIST (object))
    {
      g_warning ("%s only valid for CList's", handler_names[0]);
      return TRUE;
    }
  connect_clist (GDAM_SONG_SELECTOR_SKIN (skin),
		 handler_names + 1, GTK_CLIST (object));
  return TRUE;
}

static gboolean gdam_song_selector_palette_clist_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *ssskin = GDAM_SONG_SELECTOR_SKIN (skin);
  ssskin->database = gdam_song_database_new ("palette");
  if (!GTK_IS_CLIST (object))
    {
      g_warning ("%s only valid for CList's", handler_names[0]);
      return TRUE;
    }
  gdam_debug ("Setting songdb to %p", ssskin->database);
  connect_clist (GDAM_SONG_SELECTOR_SKIN (skin),
		 handler_names + 1, GTK_CLIST (object));
  return TRUE;
}

static void sort_table_by_column(GtkWidget* wid, int idx, 
		 	        GdamSongSelectorSkin* sel_skin)
{
  sel_skin->sort_by = sel_skin->column_sortings[idx];
  gdam_song_selector_skin_update_table (sel_skin);
  return;
}

static gboolean gdam_song_selector_window_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamSongSelectorSkin *sel_skin = GDAM_SONG_SELECTOR_SKIN (skin);
  sel_skin->toplevel = GTK_WINDOW (object);
  gtk_signal_connect (object,
		      "delete_event",
		      GTK_SIGNAL_FUNC (songsel_destroy_event), skin);
  return TRUE;
}

static gboolean is_one_of(const char* key, ...)
{
  va_list args;
  char *cmp;
  va_start (args, key);
  while ((cmp = va_arg (args, char *)) != NULL)
    {
      if (g_strcasecmp (key, cmp) == 0)
	{
	  va_end (args);
	  return TRUE;
	}
    }
  va_end (args);
  return FALSE;
}

static char* song_bpm_getter(GdamSongInfo* info)
{
  return g_strdup_printf ("%3.3f",
			  gdam_beat_info_get_bpm (gdam_song_info_get_beat_info
						  (info), 0));
}

static char* song_length_getter(GdamSongInfo* info)
{
  int secs = (int) (gdam_song_info_get_length (info));
  return g_strdup_printf ("%d:%02d", secs / 60, secs % 60);
}

static char* song_loops_getter(GdamSongInfo* info)
{
  const GList *loop_list = gdam_song_info_get_loops (info);
  int num = loop_list ? g_list_length ((GList *) loop_list) : 0;
  return g_strdup_printf ("%d", num);
}

static char* song_ranges_getter(GdamSongInfo* info)
{
  const GdamBeatInfo *bi = gdam_song_info_get_beat_info (info);
  const GList *range_list = bi->range_list;
  int num = range_list ? g_list_length ((GList *) range_list) : 0;
  return g_strdup_printf ("%d", num);
}

static char* song_filename_getter(GdamSongInfo* info)
{
  return g_strdup (gdam_song_info_get_filename (info));
}
static char* unknown_getter(GdamSongInfo* info)
{
  return g_strdup ("??");
}

static CListEntryGetter get_column_filler(const char* title)
{
  if (is_one_of (title, "filename", "song name", (char *) NULL))
    return song_filename_getter;

  if (is_one_of (title, "bpm", "tempo", (char *) NULL))
    return song_bpm_getter;

  if (is_one_of (title, "length", (char *) NULL))
    return song_length_getter;

  if (is_one_of (title, "loops", (char *) NULL))
    return song_loops_getter;

  if (is_one_of (title, "ranges", (char *) NULL))
    return song_ranges_getter;

  return unknown_getter;
}

static GtkJustification get_column_justification(const char* title)
{
  if (is_one_of (title, "filename", "song name", (char *) NULL))
    return GTK_JUSTIFY_LEFT;

  if (is_one_of (title, "bpm", "tempo", (char *) NULL))
    return GTK_JUSTIFY_RIGHT;

  if (is_one_of (title, "length", (char *) NULL))
    return GTK_JUSTIFY_RIGHT;

  if (is_one_of (title, "loops", (char *) NULL))
    return GTK_JUSTIFY_RIGHT;

  if (is_one_of (title, "ranges", (char *) NULL))
    return GTK_JUSTIFY_RIGHT;

  return GTK_JUSTIFY_LEFT;
}

static gboolean get_column_expand(const char* title)
{
  if (is_one_of (title, "filename", "song name", (char *) NULL))
    return TRUE;

  return FALSE;
}

static GdamSorting get_column_sorting(const char* title)
{
  if (is_one_of (title, "filename", "song name", (char *) NULL))
    return GDAM_SORT_BY_FILENAME;

  if (is_one_of (title, "bpm", "tempo", (char *) NULL))
    return GDAM_SORT_BY_BPM;
  return GDAM_SORT_BY_FILENAME;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_song_selector_clist",
	   gdam_song_selector_clist_handler			},
	{ "gdam_song_selector_palette_clist",
	   gdam_song_selector_palette_clist_handler		},
	{ "gdam_song_selector_info_label",
	   gdam_song_selector_info_label_handler		},
	{ "gdam_song_selector_filename_entry",
	   gdam_song_selector_filename_entry_handler		},
	{ "gdam_song_selector_info_entry",
	   gdam_song_selector_info_entry_handler		},
	{ "gdam_song_selector_find_entry",
	   gdam_song_selector_find_entry_handler		},
	{ "gdam_song_selector_grab_find_focus",
	   gdam_song_selector_grab_find_focus_handler		},
	{ "gdam_song_selector_find",
	   gdam_song_selector_find_handler			},
	{ "gdam_song_selector_delete",
	   gdam_song_selector_delete_handler			},
	{ "gdam_song_selector_hide_unknown",
	   gdam_song_selector_hide_unknown_handler		},
	{ "gdam_song_selector_hide_unknown_bpm_tbutton",
	   gdam_song_selector_hide_unknown_bpm_tbutton_handler	},
	{ "gdam_song_selector_save_unknown_bpm_tbutton",
	   gdam_song_selector_save_unknown_bpm_tbutton_handler	},
	{ "gdam_song_selector_hide_default_tbutton",
	   gdam_song_selector_hide_default_tbutton_handler	},
	{ "gdam_song_selector_change_hide_file",
	   gdam_song_selector_change_hide_file_handler		},
	{ "gdam_song_selector_change_show_file",
	   gdam_song_selector_change_show_file_handler		},
	{ "gdam_song_selector_reset_hide_file",
	   gdam_song_selector_reset_hide_file_handler		},
	{ "gdam_song_selector_reset_show_file",
	   gdam_song_selector_reset_show_file_handler		},
	{ "gdam_song_selector_hide_file_entry",
	   gdam_song_selector_hide_file_entry_handler		},
	{ "gdam_song_selector_show_file_entry",
	   gdam_song_selector_show_file_entry_handler		},
	{ "gdam_song_selector_show_key_entry",
	   gdam_song_selector_show_key_entry_handler		},
	{ "gdam_song_selector_show_value_entry",
	   gdam_song_selector_show_value_entry_handler		},
	{ "gdam_song_selector_sort_by_bpm",
	   gdam_song_selector_sort_by_bpm_handler		},
	{ "gdam_song_selector_min_bpm_adjustment",
	   gdam_song_selector_min_bpm_adjustment_handler	},
	{ "gdam_song_selector_max_bpm_adjustment",
	   gdam_song_selector_max_bpm_adjustment_handler	},
	{ "gdam_song_selector_save_database",
	   gdam_song_selector_save_database_handler		},
	{ "gdam_song_selector_load_database",
	   gdam_song_selector_load_database_handler		},
	{ "gdam_song_selector_window",
	   gdam_song_selector_window_handler			}
};

static void gdam_song_selector_skin_post_connecting_handler(GdamSkin* skin)
{
  GdamSongSelectorSkin *selector_skin = GDAM_SONG_SELECTOR_SKIN (skin);
  if (!selector_skin->database)
    {
      g_warning ("post_connect_handler called w/o database, creating");
      selector_skin->database = gdam_song_database_system ();
    }
  g_return_if_fail (selector_skin->database != NULL);
  gdam_debug ("Conntecting to songdb changed for %p",
	      selector_skin->database);
  gtk_signal_connect_object_while_alive
    (GTK_OBJECT (selector_skin->database),
     "changed",
     GTK_SIGNAL_FUNC (gdam_song_selector_skin_update_table),
     GTK_OBJECT (skin));
}

static void gdam_song_selector_skin_class_init(GdamSkinClass* skin_class)
{
  skin_class->post_connecting_handler
    = gdam_song_selector_skin_post_connecting_handler;
  GTK_OBJECT_CLASS (skin_class)->destroy = gdam_song_selector_skin_destroy;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}

/* TODO: this string compilation code shouldn't be duplicated in drag */
/* TODO: switch to dynamically allocated character array, perhaps g_string?? */
char*  gdam_song_selector_export_selection_string(GdamSongSelectorSkin* skin)
{
  GtkCList *clist = skin->scrlist;
  GList *selected;
  char str[16384] = "";
  selected = clist->selection;

  while (selected)
    {
      int num = GPOINTER_TO_INT (selected->data);
      gpointer data = gtk_clist_get_row_data (clist, num);
      GdamSongInfo *si = (GdamSongInfo *) data;
      strcat (str, gdam_song_info_get_filename (si));
      strcat (str, "\n");
      selected = selected->next;
      if (strlen (str) >= 16383 && selected != NULL)
	{
	  g_warning ("OUt of string space in song selector" " export!");
	}
    }
  return g_strdup (str);
}
