/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gtkpixscrollbar.h"
#include "pixmapcache.h"
#include "gdamskin.h"		/* needed for path stuff. */
#include <gtk/gtksignal.h>
#include <gtk/gtkmain.h>
#include <stdlib.h>

/* 
 * XXX: currently the entire scrollbar is redrawn when
 *      the bauble moves.
 */

/* XXX: memory leak from pixmaps, names  in destroy ,set filenames. */

enum {
	MOTION,
	LAST_SIGNAL
};
static guint pix_scrollbar_signals[LAST_SIGNAL] = { 0 };

static int  gtk_pix_scrollbar_expose_event          (GtkWidget*       widget,
                                                     GdkEventExpose*  event);
static int  gtk_pix_scrollbar_configure_event       (GtkWidget*       widget,
                                                     GdkEventConfigure*event);
static void gtk_pix_scrollbar_draw                  (GtkWidget*       widget,
                                                     GdkRectangle*    area);
static void gtk_pix_scrollbar_draw_default          (GtkWidget *widget);
static void gtk_pix_scrollbar_draw_focus            (GtkWidget *widget);
static void gtk_pix_scrollbar_size_request          (GtkWidget*       widget,
                                                     GtkRequisition*  event);
static int  gtk_pix_scrollbar_press_event           (GtkWidget*       widget,
                                                     GdkEventButton*  event);
static int  gtk_pix_scrollbar_release_event         (GtkWidget*       widget,
                                                     GdkEventButton*  event);
static int  gtk_pix_scrollbar_enter_notify_event    (GtkWidget*       widget,
                                                     GdkEventCrossing*  event);
static int  gtk_pix_scrollbar_leave_notify_event    (GtkWidget*       widget,
                                                     GdkEventCrossing*event);
static int  gtk_pix_scrollbar_motion_event          (GtkWidget*       widget,
                                                     GdkEventMotion*  event);
static void update_adjustment(GtkPixScrollbar* pix_scrollbar,
                              int              cur_x,
			      int              cur_y,
			      int	       button);
static GtkObjectClass* parent_class;
static void load_images                     (GtkPixScrollbar* pix_scrollbar);
static void gtk_pix_scrollbar_init(GtkPixScrollbar* pix_scrollbar)
{
  GTK_WIDGET_SET_FLAGS (pix_scrollbar, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
  gtk_widget_add_events (GTK_WIDGET (pix_scrollbar),
			 GDK_EXPOSURE_MASK
			 | GDK_STRUCTURE_MASK
			 | GDK_ENTER_NOTIFY_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_BUTTON_PRESS_MASK | GDK_MOTION_NOTIFY);
  pix_scrollbar->drag_to_position = 1;
}
static void gtk_pix_scrollbar_class_init(GtkPixScrollbarClass* klass)
{
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
  GtkObjectClass *object_class = GTK_OBJECT_CLASS (klass);
  widget_class->expose_event = gtk_pix_scrollbar_expose_event;
  widget_class->configure_event = gtk_pix_scrollbar_configure_event;
  widget_class->button_press_event = gtk_pix_scrollbar_press_event;
  widget_class->button_release_event = gtk_pix_scrollbar_release_event;
  widget_class->motion_notify_event = gtk_pix_scrollbar_motion_event;
  widget_class->enter_notify_event = gtk_pix_scrollbar_enter_notify_event;
  widget_class->leave_notify_event = gtk_pix_scrollbar_leave_notify_event;
  widget_class->size_request = gtk_pix_scrollbar_size_request;
  widget_class->draw = gtk_pix_scrollbar_draw;
  widget_class->draw_focus = gtk_pix_scrollbar_draw_focus;
  widget_class->draw_default = gtk_pix_scrollbar_draw_default;
  pix_scrollbar_signals[MOTION] =
    gtk_signal_new ("motion",
		    GTK_RUN_FIRST,
		    object_class->type,
		    GTK_SIGNAL_OFFSET (GtkPixScrollbarClass, motion),
		    gtk_marshal_NONE__NONE, GTK_TYPE_NONE, 0);
  gtk_object_class_add_signals (object_class,
				pix_scrollbar_signals, LAST_SIGNAL);
}

GtkType gtk_pix_scrollbar_get_type()
{
  static GtkType pix_scrollbar_type = 0;
  if (!pix_scrollbar_type)
    {
      static const GtkTypeInfo pix_scrollbar_info = {
	"GtkPixScrollbar",
	sizeof (GtkPixScrollbar),
	sizeof (GtkPixScrollbarClass),
	(GtkClassInitFunc) gtk_pix_scrollbar_class_init,
	(GtkObjectInitFunc) gtk_pix_scrollbar_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GTK_TYPE_EVENT_BOX;
      parent_class = gtk_type_class (parent);
      pix_scrollbar_type = gtk_type_unique (parent, &pix_scrollbar_info);
    }
  return pix_scrollbar_type;
}

static inline gfloat get_fraction(GtkPixScrollbar* pix_scrollbar)
{
  gfloat fraction;
  if (pix_scrollbar->adjustment != NULL)
    {
      GtkAdjustment *adjustment = pix_scrollbar->adjustment;
      fraction = (adjustment->value - adjustment->lower)
	/ (adjustment->upper - adjustment->lower);
      fraction = CLAMP (fraction, 0.0, 1.0);
    }
  else
    {
      fraction = 0.5;
    }
  return fraction;
}
static void
gtk_pix_scrollbar_draw_focus (GtkWidget *widget)
{
  gtk_widget_draw (widget, NULL);
}

static void
gtk_pix_scrollbar_draw_default (GtkWidget *widget)
{
  gtk_widget_draw (widget, NULL);
}


static void gtk_pix_scrollbar_draw(GtkWidget* widget, GdkRectangle* area)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  GdkWindow *window = widget->window;
  GdkGCValues values;
  int window_width = widget->allocation.width;
  int window_height = widget->allocation.height;
  int x, y;
  gfloat fraction;
  int pix_width, pix_height;

  if (!GTK_WIDGET_DRAWABLE (widget))
    {
      return;
    }
  if (!pix_scrollbar->loaded_images)
    load_images (pix_scrollbar);

  gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
  gdk_window_clear_area (widget->window,
			 area->x, area->y, area->width, area->height);
  if (GTK_WIDGET_HAS_DEFAULT (widget) &&
      GTK_BUTTON (widget)->relief == GTK_RELIEF_NORMAL &&
      pix_scrollbar->background == NULL)
    {
      gtk_paint_box (widget->style, widget->window,
		     GTK_STATE_NORMAL, GTK_SHADOW_IN,
		     area, widget, "buttondefault",
		     area->x, area->y, area->width, area->height);
    }


  gdk_gc_get_values (widget->style->fg_gc[0], &values);

  if (pix_scrollbar->background)
    {
      pix_width = pix_scrollbar->background_size.x;
      pix_height = pix_scrollbar->background_size.y;
      x = (window_width - pix_width) / 2;
      y = (window_height - pix_height) / 2;
      gdk_gc_set_clip_origin (widget->style->bg_gc[0], x, y);
      gdk_gc_set_clip_mask (widget->style->bg_gc[0],
			    pix_scrollbar->background_mask);

      gdk_window_copy_area (window,
			    widget->style->bg_gc[0],
			    x,
			    y,
			    pix_scrollbar->background,
			    0, 0, pix_width, pix_height);
      gdk_gc_set_clip_origin (widget->style->bg_gc[0], 0, 0);
      gdk_gc_set_clip_mask (widget->style->bg_gc[0], NULL);
    }
  else
    {
      x = y = 0;
    }

  gdk_gc_set_clip_mask (widget->style->fg_gc[0], NULL);
  gdk_gc_set_clip_mask (widget->style->fg_gc[0], pix_scrollbar->floater_mask);
  fraction = get_fraction (pix_scrollbar);
  pix_width = pix_scrollbar->floater_size.x;
  pix_height = pix_scrollbar->floater_size.y;

  x += (fraction * pix_scrollbar->trough_delta_x);
  x += pix_scrollbar->trough_start_x;
  y += (fraction * pix_scrollbar->trough_delta_y);
  y += pix_scrollbar->trough_start_y;
  x -= pix_width / 2;
  y -= pix_height / 2;

  gdk_gc_set_clip_origin (widget->style->fg_gc[0], x, y);
  gdk_window_copy_area (window,
			widget->style->fg_gc[0],
			x,
			y,
			pix_scrollbar->floater, 0, 0, pix_width, pix_height);

  gdk_gc_set_clip_mask (widget->style->fg_gc[0], NULL);
  gdk_gc_set_clip_origin (widget->style->fg_gc[0],
			  values.clip_x_origin, values.clip_y_origin);
}

/* update:
 *     in_child
 *     drag_offset_x
 *     drag_offset_y
 */
static void recompute_in_child(GtkPixScrollbar* pix_scrollbar,
                               int              press_x,
			       int              press_y)
{
  gfloat fraction;
  GtkAdjustment *adj = pix_scrollbar->adjustment;
  int width, height;
  int floater_x, floater_y;
  GtkAllocation *allocation;

  /*
   * Make press_x, press_y relative to the centered
   * trough pixmap.
   */
  allocation = &GTK_WIDGET (pix_scrollbar)->allocation;
  press_x -= (allocation->width - pix_scrollbar->background_size.x) / 2;
  press_y -= (allocation->height - pix_scrollbar->background_size.y) / 2;

  fraction = (adj->value - adj->lower) / (adj->upper - adj->lower);
  floater_x = pix_scrollbar->trough_start_x
    + fraction * pix_scrollbar->trough_delta_x;
  floater_y = pix_scrollbar->trough_start_y
    + fraction * pix_scrollbar->trough_delta_y;
  press_x -= floater_x;
  press_y -= floater_y;
  width = pix_scrollbar->floater_size.x;
  height = pix_scrollbar->floater_size.y;
  if ((-width / 2 <= press_x && press_x <= +width / 2)
      && (-height / 2 <= press_y && press_y <= +height / 2))
    {
      pix_scrollbar->in_child = TRUE;
      pix_scrollbar->drag_offset_x = press_x;
      pix_scrollbar->drag_offset_y = press_y;
    }
  else
    {
      pix_scrollbar->in_child = FALSE;
    }
}

static int gtk_pix_scrollbar_press_event(GtkWidget*       widget,
                                         GdkEventButton*  event)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  gfloat fraction;
  int x_offset, y_offset;
  int width2, height2;
  int press_x = event->x;
  int press_y = event->y;

  gtk_grab_add (widget);

  recompute_in_child (pix_scrollbar, press_x, press_y);

  update_adjustment (pix_scrollbar, press_x, press_y, event->button);

  gtk_widget_draw_default (GTK_WIDGET (pix_scrollbar));
  return FALSE;
}

static void reset_adjustment(GtkPixScrollbar* pix_scrollbar)
{
  GtkAdjustment *adj;
  GtkObject *object;
  g_return_if_fail (pix_scrollbar->adjustment != NULL);
  adj = pix_scrollbar->adjustment;

  object = GTK_OBJECT (adj);
  gtk_signal_handler_block (object, pix_scrollbar->connect_id);
  gtk_signal_handler_block (object, pix_scrollbar->value_connect_id);
  gtk_adjustment_set_value (adj, pix_scrollbar->orig_value);
  gtk_signal_handler_unblock (object, pix_scrollbar->value_connect_id);
  gtk_signal_handler_unblock (object, pix_scrollbar->connect_id);
  gtk_widget_draw_default (GTK_WIDGET (pix_scrollbar));
}

static void update_adjustment(GtkPixScrollbar* pix_scrollbar,
                              int              cur_x,
			      int              cur_y,
			      int	       button)
{
  int x, y;
  int trough_x = pix_scrollbar->trough_delta_x;
  int trough_y = pix_scrollbar->trough_delta_y;
  gfloat value;
  gfloat press_value;
  GtkAdjustment *adj;
  GtkObject *object;
  GtkAllocation *allocation;
  gfloat fraction;
  g_return_if_fail (pix_scrollbar->adjustment != NULL);
  adj = pix_scrollbar->adjustment;
  allocation = &GTK_WIDGET (pix_scrollbar)->allocation;

  /*
   * Make cur_x, cur_y relative to the centered
   * trough pixmap.
   */
  cur_x -= (allocation->width - pix_scrollbar->background_size.x) / 2;
  cur_y -= (allocation->height - pix_scrollbar->background_size.y) / 2;

  if (!pix_scrollbar->drag_to_position && pix_scrollbar->in_child)
    {
      cur_x += pix_scrollbar->drag_offset_x;
      cur_y += pix_scrollbar->drag_offset_y;
    }
  x = cur_x - pix_scrollbar->trough_start_x;
  y = cur_y - pix_scrollbar->trough_start_y;
  fraction = x * trough_x + y * trough_y;
  fraction /= trough_x * trough_x + trough_y * trough_y;
  press_value = fraction * (adj->upper - adj->lower) + adj->lower;

  if (pix_scrollbar->drag_to_position)
    {
      /* This case moves the scrollbar to an absolute
       * mouse position. */
      value = press_value;
    }
  else if (button == 5)
    {
      /* TODO: check adjustment limits on button 4,5?? */
      /* TODO: are these bindings wrong for horizontal scrollbars? */
      value = adj->value + adj->page_size;
    }
  else if (button == 4)
    {
      value = adj->value - adj->page_size;
    }
  else if (pix_scrollbar->in_child)
    {
      value = press_value;
    }
  else if (button == 2)
    {
      value = press_value;
    }
  else
    {
      /* Otherwise, move by step_incr. */
      value = adj->value;
      if (press_value < value - adj->page_size)
	{
	  value -= adj->page_size;
	}
      else if (press_value > value - adj->page_size)
	{
	  value += adj->page_size;
	}
      else
	{
	  value = press_value;
	}
    }

  object = GTK_OBJECT (adj);
  gtk_signal_handler_block (object, pix_scrollbar->connect_id);
  gtk_signal_handler_block (object, pix_scrollbar->value_connect_id);
  gtk_adjustment_set_value (adj, value);
  gtk_signal_handler_unblock (object, pix_scrollbar->value_connect_id);
  gtk_signal_handler_unblock (object, pix_scrollbar->connect_id);
  gtk_widget_draw_default (GTK_WIDGET (pix_scrollbar));
}

static int gtk_pix_scrollbar_release_event(GtkWidget*       widget,
                                           GdkEventButton*  event)
{
  gboolean clicked = FALSE;
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  int x, y;
  guint state;
  gtk_grab_remove (widget);

  if (event->window != widget->window)
    {
      gdk_window_get_pointer (widget->window, &x, &y, &state);
    }
  else
    {
      x = event->x;
      y = event->y;
      state = event->state;
    }

  if (pix_scrollbar->do_reset)
    {
      reset_adjustment (pix_scrollbar);
    }
  else
    {
      update_adjustment (pix_scrollbar, x, y, event->button);
    }
  pix_scrollbar->in_child = FALSE;
  gtk_pix_scrollbar_motion (pix_scrollbar);

  return FALSE;
}
static void load_images(GtkPixScrollbar* pix_scrollbar)
{
  GdkWindow *window = GTK_WIDGET (pix_scrollbar)->window;
  PixmapInfo *info;
  const char *fname;
  char *xpm;
  if (pix_scrollbar->loaded_images)
    return;
  pix_scrollbar->loaded_images = TRUE;
  xpm = pix_scrollbar->floater_file;
  if (xpm == NULL)
    {
      g_warning ("error: no filename for floater in pixscrollbar");
    }
  else
    {
      fname = gdam_skin_search_for_file (xpm);
      if (fname == NULL)
	{
	  g_warning ("error: no image %s for pixscrollbar", xpm);
	}
      else
	{
	  info = pixmap_cache_get_pixmap (window, fname);
	  if (info == NULL)
	    {
	      g_warning ("error loading pixmap %s", fname);
	      return;
	    }
	  else
	    {
	      pix_scrollbar->floater = info->pixmap;
	      pix_scrollbar->floater_mask = info->mask;
	      pix_scrollbar->floater_size.x = info->width;
	      pix_scrollbar->floater_size.y = info->height;
	    }
	}
    }

  if (strcmp (pix_scrollbar->background_xpm_file, "none") != 0)
    {
      const char *fname = pix_scrollbar->background_xpm_file;
      PixmapInfo *info;
      fname = gdam_skin_search_for_file (fname);
      if (fname == NULL)
	{
	  g_warning ("error: no bg image %s for pixscrollbar", fname);
	  return;
	}
      info = pixmap_cache_get_pixmap (window, fname);
      if (info == NULL)
	{
	  g_warning ("error loading pixmap %s", fname);
	  return;
	}
      pix_scrollbar->background = info->pixmap;
      pix_scrollbar->background_mask = info->mask;
      pix_scrollbar->background_size.x = info->width;
      pix_scrollbar->background_size.y = info->height;
    }
  else
    {
      pix_scrollbar->background = NULL;
    }
}

static int gtk_pix_scrollbar_expose_event(GtkWidget*       widget,
                                          GdkEventExpose*  event)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  gtk_widget_draw_default (GTK_WIDGET (pix_scrollbar));
  return FALSE;
}
static int gtk_pix_scrollbar_configure_event(GtkWidget*       widget,
                                             GdkEventConfigure*event)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  if (!pix_scrollbar->loaded_images)
    load_images (pix_scrollbar);
  if (pix_scrollbar->lastw == event->width &&
      pix_scrollbar->lasth == event->height)
    return FALSE;
  pix_scrollbar->lastw = event->width;
  pix_scrollbar->lasth = event->height;
  gtk_widget_draw_default (GTK_WIDGET (pix_scrollbar));
  return FALSE;
}
static int gtk_pix_scrollbar_enter_notify_event(GtkWidget*       widget,
                                                GdkEventCrossing*event)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  GtkWidget *event_widget;
  return FALSE;
}
static int gtk_pix_scrollbar_leave_notify_event(GtkWidget*        widget,
                                                GdkEventCrossing* event)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  GtkWidget *event_widget;
  event_widget = gtk_get_event_widget ((GdkEvent *) event);
  if (event_widget != widget || event->detail == GDK_NOTIFY_INFERIOR)
    return FALSE;
  gtk_widget_draw_default (widget);
  return FALSE;
}
static void gtk_pix_scrollbar_size_request    (GtkWidget*       widget,
                                               GtkRequisition*  requisition)
{
  GtkPixScrollbar *pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  requisition->width = pix_scrollbar->background_size.x;
  requisition->height = pix_scrollbar->background_size.y;
}

static int  gtk_pix_scrollbar_motion_event          (GtkWidget*       widget,
                                                     GdkEventMotion*  event)
{
  int x, y;
  guint state;
  GtkPixScrollbar *pix_scrollbar;
  pix_scrollbar = GTK_PIX_SCROLLBAR (widget);
  if (event->is_hint)
    {
      gdk_window_get_pointer (widget->window, &x, &y, &state);
    }
  else
    {
      if (event->window != widget->window)
	{
	  gdk_window_get_pointer (widget->window, &x, &y, &state);
	}
      else
	{
	  x = event->x;
	  y = event->y;
	  state = event->state;
	}
    }

  update_adjustment (pix_scrollbar, x, y, 0);

  gtk_pix_scrollbar_motion (pix_scrollbar);
  return 0;
}

void gtk_pix_scrollbar_motion(GtkPixScrollbar* pix_scrollbar)
{
  gtk_signal_emit (GTK_OBJECT (pix_scrollbar), pix_scrollbar_signals[MOTION]);
}


void gtk_pix_scrollbar_set_pixmaps_by_filename(GtkPixScrollbar* pix_scrollbar,
                                               const char*      background,
                                               const char*      floater)
{
  pix_scrollbar->floater_file = g_strdup (floater);
  pix_scrollbar->background_xpm_file = g_strdup (background);
  if (GTK_WIDGET (pix_scrollbar)->window)
    load_images (pix_scrollbar);
}

/*
 *   string1:    background:floater
 *   string2:    startx:starty:deltax:deltay:cur_value:min_value:max_value
 */

GtkWidget* gtk_pix_scrollbar_create_func   (const char*   name,
                                            const char*   string1,
                                            const char*   string2,
                                            int           int1,
                                            int           int2)
{
  char **pixmaps = g_strsplit (string1, ":", 100);
  char **coordinates;
  gfloat cur_value, min_value, max_value;
  gfloat step_increment, page_increment, page_size;
  GtkAdjustment *adjustment;
  GtkPixScrollbar *pix_scrollbar;
  gboolean do_reset;
  pix_scrollbar = GTK_PIX_SCROLLBAR (gtk_type_new (GTK_TYPE_PIX_SCROLLBAR));
  gtk_pix_scrollbar_set_pixmaps_by_filename (pix_scrollbar,
					     pixmaps[0], pixmaps[1]);
  g_strfreev (pixmaps);

  coordinates = g_strsplit (string2, ":", 100);
  pix_scrollbar->trough_start_x = atoi (coordinates[0]);
  pix_scrollbar->trough_start_y = atoi (coordinates[1]);
  pix_scrollbar->trough_delta_x = atoi (coordinates[2]);
  pix_scrollbar->trough_delta_y = atoi (coordinates[3]);

  pix_scrollbar->do_reset = int1;

  cur_value = atof (coordinates[4]);
  min_value = atof (coordinates[5]);
  max_value = atof (coordinates[6]);
  if (coordinates[7] && atoi (coordinates[7]) != 0)
    {
      pix_scrollbar->drag_to_position = 0;
      g_return_val_if_fail (coordinates[8] != NULL, NULL);
      g_return_val_if_fail (coordinates[9] != NULL, NULL);
      g_return_val_if_fail (coordinates[10] != NULL, NULL);
      step_increment = atof (coordinates[8]);
      page_increment = atof (coordinates[9]);
      page_size = atof (coordinates[10]);
    }
  else
    {
      pix_scrollbar->drag_to_position = 1;
      step_increment = 0.01;
      page_increment = 0.01;
      page_size = 0.0;
    }
  g_strfreev (coordinates);

  pix_scrollbar->orig_value = cur_value;

  adjustment = (GtkAdjustment *) gtk_adjustment_new (cur_value,
						     min_value,
						     max_value,
						     step_increment,
						     page_increment,
						     page_size);
  gtk_pix_scrollbar_set_adjustment (pix_scrollbar, adjustment);

  return GTK_WIDGET (pix_scrollbar);
}

void gtk_pix_scrollbar_set_adjustment(GtkPixScrollbar*  pix_scrollbar,
                                      GtkAdjustment*    adjustment)
{
  GtkAdjustment *old_adj = pix_scrollbar->adjustment;
  if (old_adj == adjustment)
    return;
  if (old_adj != NULL)
    {
      gtk_signal_disconnect (GTK_OBJECT (old_adj), pix_scrollbar->connect_id);
      gtk_signal_disconnect (GTK_OBJECT (old_adj),
			     pix_scrollbar->value_connect_id);
      gtk_object_unref (GTK_OBJECT (old_adj));
    }
  pix_scrollbar->adjustment = adjustment;
  gtk_object_ref (GTK_OBJECT (adjustment));
  gtk_object_sink (GTK_OBJECT (adjustment));
  if (adjustment == NULL)
    {
      pix_scrollbar->value_connect_id = 0;
      pix_scrollbar->connect_id = 0;
    }
  else
    {
      pix_scrollbar->value_connect_id
	= gtk_signal_connect_object
	(GTK_OBJECT (adjustment),
	 "value_changed",
	 GTK_SIGNAL_FUNC (gtk_widget_queue_draw), GTK_OBJECT (pix_scrollbar));
      pix_scrollbar->connect_id
	= gtk_signal_connect_object
	(GTK_OBJECT (adjustment),
	 "changed",
	 GTK_SIGNAL_FUNC (gtk_widget_queue_draw), GTK_OBJECT (pix_scrollbar));
    }
}
