/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/

#include "gdamgongskin.h"
#include "../../component/plugins/gdamgong.h"
#include "../gdamskinutil.h"
#include <gtk/gtklabel.h>
#include <gtk/gtkhscrollbar.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtksignal.h>
#include <math.h>

static void gdam_gong_skin_init(GdamGongSkin* gong_skin)
{
}

static void
set_label_to_hertz(GtkAdjustment* adjustment, GtkLabel* label)
{
  char buf[50];
  g_snprintf (buf, sizeof (buf), "%4.4g", exp (adjustment->value));
  gtk_label_set_text (label, buf);
}

static void
set_gong_style_log_hertz(GtkAdjustment* adjustment, GdamGongStyle* style)
{
  style->hertz = exp (adjustment->value);
}

static void
set_gong_style_volume(GtkAdjustment* adjustment, GdamGongStyle* style)
{
  style->overall_volume = adjustment->value;
}

static void
set_gong_style_halflife(GtkAdjustment* adjustment, GdamGongStyle* style)
{
  style->halflife_scale = adjustment->value;
}
static void strike_gong(GdamGongStyle* style)
{
  gdam_gong_strike (style->gong, style);
}

static GtkWidget* make_gong_style_widget(GdamGongStyle* style)
{
  GtkAdjustment *adjustment;
  GtkBox *box;
  GtkBox *sub_box;
  GtkWidget *label;
  GtkWidget *scroll;
  GtkWidget *button;

  box = (GtkBox *) gtk_vbox_new (FALSE, 1);
  sub_box = (GtkBox *) gtk_hbox_new (FALSE, 1);
  adjustment
    = (GtkAdjustment *) gtk_adjustment_new (log (440.0),
					    log (20.0),
					    log (10000.0), 0.01, 0.01, 0.01);
  label = gtk_label_new ("Hertz");
  gtk_widget_show (label);
  gtk_box_pack_start_defaults (sub_box, label);

  label = gtk_label_new ("0");
  gtk_widget_show (label);
  gtk_box_pack_start_defaults (sub_box, label);
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (set_label_to_hertz), label);
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (set_gong_style_log_hertz), style);

  scroll = gtk_hscrollbar_new (adjustment);
  gtk_widget_show (scroll);
  gtk_box_pack_start_defaults (sub_box, scroll);

  gtk_widget_show (GTK_WIDGET (sub_box));
  gtk_box_pack_start_defaults (box, GTK_WIDGET (sub_box));

  sub_box = (GtkBox *) gtk_hbox_new (FALSE, 1);
  label = gtk_label_new ("Halflife");
  gtk_widget_show (label);
  gtk_box_pack_start_defaults (sub_box, label);
  adjustment
    = (GtkAdjustment *) gtk_adjustment_new (1.0, 0.1, 2.0, 0.01, 0.01, 0.01);
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (set_gong_style_halflife), style);
  scroll = gtk_hscrollbar_new (adjustment);
  gtk_widget_show (scroll);
  gtk_box_pack_start_defaults (sub_box, scroll);

  gtk_widget_show (GTK_WIDGET (sub_box));
  gtk_box_pack_start_defaults (box, GTK_WIDGET (sub_box));

  sub_box = (GtkBox *) gtk_hbox_new (FALSE, 1);
  label = gtk_label_new ("Volume");
  gtk_widget_show (label);
  gtk_box_pack_start_defaults (sub_box, label);
  adjustment
    = (GtkAdjustment *) gtk_adjustment_new (1.0, 0.0, 1.5, 0.01, 0.01, 0.01);
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value_changed",
		      GTK_SIGNAL_FUNC (set_gong_style_volume), style);
  scroll = gtk_hscrollbar_new (adjustment);
  gtk_widget_show (scroll);
  gtk_box_pack_start_defaults (sub_box, scroll);

  gtk_widget_show (GTK_WIDGET (sub_box));
  gtk_box_pack_start_defaults (box, GTK_WIDGET (sub_box));

  button = gtk_button_new_with_label ("Gong!");
  gtk_signal_connect_object (GTK_OBJECT (button),
			     "clicked",
			     GTK_SIGNAL_FUNC (strike_gong),
			     (GtkObject *) style);
  gtk_widget_show (button);
  gtk_box_pack_start_defaults (box, button);
  gtk_widget_show (GTK_WIDGET (box));
  return GTK_WIDGET (box);
}

typedef struct _GdamGongNewTypeMaker GdamGongNewTypeMaker;
struct _GdamGongNewTypeMaker {
	GdamGongSkin* skin;
	GdamGong* gong;
	GdamGongType type;
	guint connect_id;
};
static void new_type_of_gong(GdamGongNewTypeMaker* maker)
{
  GdamGongStyle *style;
  GtkWidget *widget;
  style = gdam_gong_add (maker->gong, maker->type);
  widget = make_gong_style_widget (style);
  g_return_if_fail (widget != NULL);
  gtk_container_add (maker->skin->gong_box, widget);
}

static gboolean gdam_gong_new_type_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  const char *type_name;
  GdamGongType type;
  GdamGongNewTypeMaker *maker;
  if (num_handler_names < 2)
    {
      g_warning ("usaged: %s:gong_type", handler_names[0]);
      return TRUE;
    }
  type_name = handler_names[1];
  if (strcmp (type_name, "free") == 0)
    {
      type = GDAM_GONG_FREE_OSCILLATION;
    }
  else if (strcmp (type_name, "clamped_circular") == 0)
    {
      type = GDAM_GONG_CLAMPED_CIRCULAR_PLATE;
    }
  else
    {
      g_warning ("unknown gong type: %s", type_name);
      return TRUE;
    }
  maker = g_new (GdamGongNewTypeMaker, 1);
  maker->type = type;
  maker->skin = GDAM_GONG_SKIN (skin);
  maker->gong = GDAM_GONG (skin->model);
  maker->connect_id
    =
    gtk_signal_connect_full (object, signal_name,
			     GTK_SIGNAL_FUNC (new_type_of_gong), NULL, maker,
			     g_free, TRUE,	/* is object_signal */
			     FALSE);
  return TRUE;
}

static gboolean gdam_gong_container_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GDAM_GONG_SKIN (skin)->gong_box = GTK_CONTAINER (object);
  return TRUE;
}


static GdamSkinGenericConnect connect_infos[] = { 	
	{ "gdam_gong_new_type",
	   gdam_gong_new_type_handler },
	{ "gdam_gong_container",
	   gdam_gong_container_handler },
};

static void gdam_gong_skin_class_init(GdamSkinClass* skin_class)
{
  skin_class->model_type = GDAM_TYPE_GONG;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}

GtkType gdam_gong_skin_get_type()
{
  static GtkType gong_skin_type = 0;
  if (!gong_skin_type)
    {
      static const GtkTypeInfo gong_skin_info = {
	"GdamGongSkin",
	sizeof (GdamGongSkin),
	sizeof (GdamGongSkinClass),
	(GtkClassInitFunc) gdam_gong_skin_class_init,
	(GtkObjectInitFunc) gdam_gong_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      gong_skin_type = gtk_type_unique (parent, &gong_skin_info);
    }
  return gong_skin_type;
}
