#include "gdamhelpskin.h"
#include "../gdamskinutil.h"
#include <common/gdamsearchpath.h>
#include <gtk/gtksignal.h>
#include <string.h>
#include <errno.h>
#include <common/gdamplugin.h>

static void gdam_help_post_connecting_handler (GdamSkin*      skin);
static void gdam_help_skin_class_init(GdamSkinClass* skin_class);
static void gdam_help_skin_init(GdamHelpSkin* help_skin)
{
}

static const char* help_path = "~/gdam/help"
                               ":./help"
			       ":/usr/share/gdam/help"
			       ":/usr/local/share/gdam/help"
			       ":" G_STRINGIFY(PREFIX) "/help";

static gboolean parse_bool(const char* arg)
{
  if (strchr ("ytYT1", *arg))
    return TRUE;
  if (strchr ("nfNF0", *arg))
    return FALSE;
  g_message ("unknown boolean string %s, in first line of help file", arg);
  return TRUE;
}

typedef struct _HelpTopicSetter HelpTopicSetter;
struct _HelpTopicSetter {
	char* str;
	GdamHelpSkin* help_skin;
};

static void set_help_topic(HelpTopicSetter* setter)
{
  gdam_help_skin_set_topic (setter->help_skin, setter->str);
}
static void free_help_topic(HelpTopicSetter* setter)
{
  g_free (setter->str);
  g_free (setter);
}


static void add_one_button(gpointer str, gpointer skin)
{
  GtkWidget *button;
  GdamHelpSkin *help_skin = GDAM_HELP_SKIN (skin);
  HelpTopicSetter *help_topic_setter;
  GtkBox *box = help_skin->see_also_box;
  help_topic_setter = g_new (HelpTopicSetter, 1);

  /* TODO: is the next line a memory leak? */
  help_topic_setter->str = g_strchomp (g_strdup (str));

  button = gtk_button_new_with_label (help_topic_setter->str);
  gtk_widget_show (button);
  help_topic_setter->help_skin = help_skin;
  gtk_signal_connect_full (GTK_OBJECT (button),
			   "clicked",
			   GTK_SIGNAL_FUNC (set_help_topic),
			   NULL,
			   help_topic_setter,
			   (GDestroyNotify) free_help_topic, TRUE, FALSE);
  gtk_box_pack_start (GTK_BOX (box), button, FALSE,	/* fill */
		      FALSE,	/* expand */
		      1);	/* padding */
}

static gboolean process_first_line(GdamHelpSkin* skin, const char* buf)
{
  char **options;
  char **at;
  int num_see_also = 0;
  GSList *see_also = NULL;
  if (buf[0] != ':')
    return FALSE;
  options = g_strsplit (buf + 1, ":", 0);
  if (options == NULL)
    return TRUE;
  skin->cur_font = NULL;
  for (at = options; *at != NULL; at++)
    {
      const char *arg = *at;
      const char *opt = strchr (arg, '=');
      if (opt != NULL)
	opt++;
      if (strncasecmp ("word_wrap=", arg, 10) == 0)
	{
	  gtk_text_set_word_wrap (skin->text_area, parse_bool (opt));
	  continue;
	}
      if (strncasecmp ("line_wrap=", arg, 10) == 0)
	{
	  gtk_text_set_line_wrap (skin->text_area, parse_bool (opt));
	  continue;
	}
      if (strncasecmp ("fixed_font=", arg, 11) == 0)
	{
	  gboolean fixed = parse_bool (opt);
	  if (fixed)
	    {
	      /* TODO: make this fixed. */
	      if (!skin->fixed_font)
		skin->fixed_font = gdk_font_load ("fixed");
	      skin->cur_font = skin->fixed_font;
	    }
	  continue;
	}
      if (strncasecmp ("see_also=", arg, 9) == 0)
	{
	  see_also = g_slist_prepend (see_also, (gpointer) arg + 9);
	  num_see_also++;
	  continue;
	}
      g_message ("unknown option in help file: %s", arg);
    }


  if (skin->see_also_box != NULL)
    {
      GList *children;
      /* Destroy all the current help topic buttons. */
      children = gtk_container_children (GTK_CONTAINER (skin->see_also_box));
      if (children)
	{
	  GtkWidget **widgets;
	  int num_child = g_list_length (children);
	  GList *tmp;
	  int i;
	  widgets = alloca (sizeof (GtkWidget *) * num_child);
	  for (i = 0, tmp = children; i < num_child; i++, tmp = tmp->next)
	    {
	      widgets[i] = tmp->data;
	    }
	  for (i = 0; i < num_child; i++)
	    {
	      gtk_widget_destroy (widgets[i]);
	    }
	}

      if (num_see_also == 0)
	{
	  GtkWidget *label = gtk_label_new ("No other topics.");
	  gtk_widget_show (label);
	  gtk_box_pack_start_defaults (skin->see_also_box, label);
	}
      else
	{
	  /* Add new buttons. */
	  g_slist_foreach (see_also, add_one_button, skin);
	}
    }
  g_slist_free (see_also);
  g_strfreev (options);
  return TRUE;
}

static void reload_text(GdamHelpSkin* help_skin)
{
  GString *text = g_string_sized_new (4096);

  GtkText *textarea;
  GtkEditable *editable;

  FILE *fp;
  char *filename;
  char buf[4096];
  gboolean is_first_line = TRUE;

  g_return_if_fail (help_skin->topic != NULL);
  g_return_if_fail (help_skin->text_area != NULL);

  filename = gdam_search_path (help_skin->topic,
			       help_path, GDAM_SEARCH_PATH_FIND_FILE);
  if (filename == NULL)
    {
      g_warning ("no help on `%s' found", help_skin->topic);
      return;
    }
  fp = fopen (filename, "r");
  if (fp == NULL)
    {
      g_warning ("error opening %s: %s", filename, strerror (errno));
      g_free (filename);
      return;
    }
  g_free (filename);
  while (fgets (buf, sizeof (buf), fp))
    {
      if (is_first_line)
	{
	  is_first_line = FALSE;
	  if (process_first_line (help_skin, buf))
	    continue;
	}
      text = g_string_append (text, buf);
    }
  fclose (fp);

  textarea = help_skin->text_area;
  editable = GTK_EDITABLE (textarea);
  gtk_text_freeze (textarea);
  gtk_editable_delete_text (editable, 0, gtk_text_get_length (textarea));
  gtk_text_insert (textarea,
		   help_skin->cur_font, NULL, NULL, text->str, text->len);
  gtk_text_thaw (textarea);
  g_string_free (text, TRUE);
}

void gdam_help_skin_set_topic(GdamHelpSkin* help_skin, const char* topic)
{
  /* Safe even if topic == help_skin->topic */
  char *old_topic = help_skin->topic;
  g_return_if_fail (topic != NULL);
  help_skin->topic = g_strdup (topic);

  if (help_skin->topic_label != NULL)
    gtk_label_set_text (help_skin->topic_label, topic);
  if (help_skin->text_area != NULL)
    reload_text (help_skin);

  g_free (old_topic);
}

GtkType gdam_help_skin_get_type()
{
  static GtkType help_skin_type = 0;
  if (!help_skin_type)
    {
      static const GtkTypeInfo help_skin_info = {
	"GdamHelpSkin",
	sizeof (GdamHelpSkin),
	sizeof (GdamHelpSkinClass),
	(GtkClassInitFunc) gdam_help_skin_class_init,
	(GtkObjectInitFunc) gdam_help_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      help_skin_type = gtk_type_unique (parent, &help_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return help_skin_type;
}

static gboolean gdam_help_skin_text_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamHelpSkin *help_skin = GDAM_HELP_SKIN (skin);
  if (!GTK_IS_TEXT (object))
    {
      g_warning ("%s may only be connected to a text widget",
		 handler_names[0]);
    }
  help_skin->text_area = GTK_TEXT (object);
  if (help_skin->topic != NULL)
    reload_text (help_skin);
  return TRUE;
}

static gboolean gdam_help_skin_topic_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GdamHelpSkin *help_skin = GDAM_HELP_SKIN (skin);
  if (!GTK_IS_LABEL (object))
    {
      g_warning ("%s may only be connected to a text widget",
		 handler_names[0]);
    }
  help_skin->topic_label = GTK_LABEL (object);
  if (help_skin->topic != NULL)
    gtk_label_set_text (help_skin->topic_label, help_skin->topic);
  return TRUE;
}
static gboolean gdam_help_skin_see_also_box_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GDAM_HELP_SKIN (skin)->see_also_box = GTK_BOX (object);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_help_skin_text",
           gdam_help_skin_text_handler			},
	{ "gdam_help_skin_topic_label",
           gdam_help_skin_topic_label_handler		},
	{ "gdam_help_skin_see_also_box",
           gdam_help_skin_see_also_box_handler		},
};
static void gdam_help_post_connecting_handler (GdamSkin*      skin)
{
  GdamHelpSkin *help_skin = GDAM_HELP_SKIN (skin);
  if (help_skin->topic != NULL)
    {
      gdam_help_skin_set_topic (GDAM_HELP_SKIN (skin), help_skin->topic);
    }
}

static gboolean gdam_help_build_new_model(GdamSkin*      skin,
                                              GdamChannel*   channel,
                                              int            num_options,
                                              char**         options)
{
  if (num_options >= 1)
    {
      GDAM_HELP_SKIN (skin)->topic = g_strdup (options[0]);
    }
  return TRUE;
}

static void gdam_help_evaluate_options(GdamSkin*      skin,
                                       int            num_options,
                                       char**         options)
{
  if (num_options >= 1)
    gdam_help_skin_set_topic (GDAM_HELP_SKIN (skin), options[0]);
}

static void gdam_help_skin_class_init(GdamSkinClass* skin_class)
{
  skin_class->build_new_model = gdam_help_build_new_model;
  skin_class->evaluate_options = gdam_help_evaluate_options;
  skin_class->post_connecting_handler = gdam_help_post_connecting_handler;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
