#include "gdammininetworkskin.h"
#include "../gdamskinutil.h"
#include <gtk/gtkdrawingarea.h>
#include <string.h>
#include "../../component/plugins/gdammininetwork.h"
#include <common/gdamsample.h>
#include <gtk/gtksignal.h>
#include "../gdamskinutil.h"
#include "../pixmapcache.h"
#include "../gdambuffercombo.h"
#include <gtk/gtkvbox.h>
#include <gtk/gtkhbox.h>
#include <gtk/gtkhscrollbar.h>
#include <gtk/gtktogglebutton.h>
#include <gtk/gtklabel.h>
#include <gtk/gtkentry.h>
#include <gtk/gtkcombo.h>
#include <common/gdamplugin.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
typedef struct _SkinNodeType SkinNodeType;
struct _SkinNodeType {
	char*			node_type;
	GdkPixmap*		pixmap;
	GdkBitmap*		pixmap_mask;
	int			width, height;
	int			num_input_types;
	GdkRectangle*           input_areas;
	char**			input_names;
	GdkRectangle		output_area;
};

typedef struct _SkinNode SkinNode;
struct _SkinNode {
	SkinNodeType*		type;
	int			x, y;
	int*			units;
};
static void queue_redraw(GdamMiniNetworkSkin* skin);
static SkinNodeType* get_skin_node_type(const char* type,
                                        GdkWindow*  window);

static gboolean lookup_input_offset(SkinNodeType* type, const char* name,
                                    int* x_out, int* y_out);
static void get_output_offset(SkinNodeType* type, int* x_out, int* y_out);

static GtkObjectClass* parent_class = NULL;

static gboolean create_input_output_nodes(GdamMiniNetworkSkin* network_skin)
{
  SkinNode *skin_input;
  SkinNode *skin_output;
  GdamMiniNetwork *network;
  GdkWindow *window;
  GdamSkin *skin = GDAM_SKIN (network_skin);
  if (skin->model == NULL)
    return FALSE;
  if (skin->main_widget == NULL || skin->main_widget->window == NULL)
    return FALSE;
  window = skin->main_widget->window;
  network = GDAM_MINI_NETWORK (skin->model);
  g_return_val_if_fail (network != NULL, FALSE);
  g_return_val_if_fail (network->input != NULL, FALSE);
  g_return_val_if_fail (network->output != NULL, FALSE);

  network_skin->has_io_nodes = TRUE;

  skin_input = g_new (SkinNode, 1);
  skin_input->type = get_skin_node_type ("input", window);
  skin_input->x = 100;
  skin_input->y = 0;
  g_hash_table_insert (network_skin->node_table, network->input, skin_input);

  skin_output = g_new (SkinNode, 1);
  skin_output->type = get_skin_node_type ("output", window);
  skin_output->x = 100;
  skin_output->y = 50;
  g_hash_table_insert (network_skin->node_table,
		       network->output, skin_output);
  queue_redraw (network_skin);
  return TRUE;
}

/*
 * Synchronize with the underlying model (add and delete nodes
 * as needed.)
 */
/* This structure stores the difference between the
 * model and the skin. */
typedef struct _UpdateLists UpdateLists;
struct _UpdateLists {
	GSList* model_nodes_to_add;
	GdamMiniNetworkSkin* network_skin;
};
static void find_new_skin_nodes(gpointer node_ptr, gpointer update_lists_ptr)
{
  UpdateLists *update_lists = update_lists_ptr;
  GdamMiniNetworkNode *node = node_ptr;
  GdamMiniNetworkSkin *network_skin = update_lists->network_skin;

  if (g_hash_table_lookup (network_skin->node_table, node) != NULL)
    return;

  update_lists->model_nodes_to_add
    = g_slist_prepend (update_lists->model_nodes_to_add, node);
}


static void gdam_mini_network_skin_synchronize(GdamView*    view)
{
  GdamSkin *skin;
  GdamMiniNetworkSkin *network_skin;
  GdamMiniNetwork *network;
  UpdateLists update_lists = { NULL, NULL };
  GSList *list;
  GdkWindow *window;

  skin = GDAM_SKIN (view);
  network_skin = GDAM_MINI_NETWORK_SKIN (skin);
  if (g_hash_table_size (network_skin->node_table) != 0)
    return;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  update_lists.network_skin = network_skin;

  /*g_message("mini_network: "
     "synchronize: model has %d nodes; skin has %d nodes",
     g_slist_length(network->nodes),
     g_hash_table_size(network_skin->node_table)); */

  /* First, revamp the skin node hashtable.
   * We don't attempt to remove old nodes -- if you want to
   * do that, just clear the network_skin before starting... */
  g_slist_foreach (network->nodes, find_new_skin_nodes, &update_lists);

  window = (skin->main_widget != NULL) ? skin->main_widget->window : NULL;
  for (list = update_lists.model_nodes_to_add;
       list != NULL; list = list->next)
    {
      GdamMiniNetworkNode *node_to_add = list->data;
      GdamMiniNetworkNodeType *node_type = node_to_add->node_type;
      SkinNode *skin_node;
      SkinNodeType *skin_node_type;

      skin_node_type = get_skin_node_type (node_type->type, window);
      skin_node = g_new (SkinNode, 1);
      skin_node->type = skin_node_type;
      skin_node->x = 0;
      skin_node->y = 0;
      g_hash_table_insert (network_skin->node_table, node_to_add, skin_node);
    }
  queue_redraw (network_skin);
}

/*
 * Xml-file reading and writing.
 * (You may assume the model comes first :)
 */
/* Add to the parent container. */
static gboolean gdam_mini_network_skin_make_xml(GdamView*    skin,
                                                xmlNode*     parent,
                                                GdamXmlInfo* info)
{
  GdamModel *model;
  GdamMiniNetwork *network;
  GdamMiniNetworkSkin *network_skin;
  GSList *list;

  model = GDAM_SKIN (skin)->model;
  network = GDAM_MINI_NETWORK (model);
  network_skin = GDAM_MINI_NETWORK_SKIN (skin);

  for (list = network->nodes; list != NULL; list = list->next)
    {
      xmlNode *node_pos;
      SkinNode *skin_node;
      GdamMiniNetworkNode *node = list->data;
      char buf[256];

      node_pos = xmlNewNode (NULL, "node_pos");
      skin_node = g_hash_table_lookup (network_skin->node_table, node);

      snprintf (buf, sizeof (buf), "%d",
		gdam_xml_info_lookup_by_object (info, node));
      xmlNewChild (node_pos, NULL, "xml_node_id", buf);

      snprintf (buf, sizeof (buf), "%d", skin_node->x);
      xmlNewChild (node_pos, NULL, "x", buf);

      snprintf (buf, sizeof (buf), "%d", skin_node->y);
      xmlNewChild (node_pos, NULL, "y", buf);

      xmlAddChild (parent, node_pos);
    }
  return (*GDAM_VIEW_CLASS (parent_class)->make_xml) (skin, parent, info);
}

static int parse_int(xmlNode* node)
{
  char *str;
  int i;
  str = xmlNodeListGetString (NULL, node->childs, TRUE);
  i = atoi (str);
  /*g_message("parse_int: %s: %d", node->name, i); */
  if (str == NULL)
    return -1;
  g_free (str);
  return i;
}

static gboolean parse_node_pos(xmlNode* children, int* x_out,
                               int* y_out, int* xml_id_out)
{
  xmlNode *at;
  *x_out = *y_out = *xml_id_out = -1;
  for (at = children; at != NULL; at = at->next)
    {
      if (strcmp (at->name, "x") == 0)
	{
	  *x_out = parse_int (at);
	  continue;
	}
      if (strcmp (at->name, "y") == 0)
	{
	  *y_out = parse_int (at);
	  continue;
	}
      if (strcmp (at->name, "xml_node_id") == 0)
	{
	  *xml_id_out = parse_int (at);
	  continue;
	}
    }
  if (*x_out == -1 || *y_out == -1 || *xml_id_out == -1)
    return FALSE;
  return TRUE;
}

/* Parse a child <GdamMiniNetworkSkin>; */
static gboolean gdam_mini_network_skin_parse_xml(GdamView*    skin,
					         xmlNode*     xml,
					         GdamXmlInfo* info)
{
  /* Now all the SkinNodes are available. */
  gdam_mini_network_skin_synchronize (skin);

  if (strcmp (xml->name, "node_pos") == 0)
    {
      int x, y, xml_node_id;
      //int num_units;                XXX: todo: save the units 
      //int* units;
      GdamMiniNetworkNode *node;
      GdamMiniNetworkSkin *network_skin;
      SkinNode *skin_node;

      if (!parse_node_pos (xml->childs, &x, &y, &xml_node_id))
	{
	  g_warning ("node pause was missing <x>, <y>, or " "<xml_node_id>");
	  return FALSE;
	}

      node = gdam_xml_info_lookup_by_id (info, xml_node_id);
      if (node == NULL)
	{
	  g_warning ("error interpreting crossref %d in "
		     "gdam_mini_network <node_pos>", xml_node_id);
	  return FALSE;
	}

      network_skin = GDAM_MINI_NETWORK_SKIN (skin);
      skin_node = g_hash_table_lookup (network_skin->node_table, node);
      g_return_val_if_fail (skin_node != NULL, FALSE);

      skin_node->x = x;
      skin_node->y = y;

      return TRUE;
    }

  return (*GDAM_VIEW_CLASS (parent_class)->parse_xml) (skin, xml, info);
}



static void gdam_mini_network_skin_init(GdamMiniNetworkSkin* skin)
{
  skin->node_table = g_hash_table_new (NULL, NULL);

}
static void gdam_mini_network_skin_class_init(GdamSkinClass* skin_class);

GtkType gdam_mini_network_skin_get_type()
{
  static GtkType mini_network_skin_type = 0;
  if (!mini_network_skin_type)
    {
      static const GtkTypeInfo mini_network_skin_info = {
	"GdamMiniNetworkSkin",
	sizeof (GdamMiniNetworkSkin),
	sizeof (GdamMiniNetworkSkinClass),
	(GtkClassInitFunc) gdam_mini_network_skin_class_init,
	(GtkObjectInitFunc) gdam_mini_network_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      parent_class = gtk_type_class (parent);
      mini_network_skin_type =
	gtk_type_unique (parent, &mini_network_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return mini_network_skin_type;
}

static GdamMiniNetworkNode*
gdam_mini_network_skin_find_node(GdamMiniNetworkSkin* skin,
                                 int                  press_x,
				 int                  press_y)
{
  GdamMiniNetwork *network;
  GSList *list;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  for (list = network->nodes; list != NULL; list = list->next)
    {
      SkinNode *skin_node;
      GdamMiniNetworkNode *node = list->data;
      skin_node = g_hash_table_lookup (skin->node_table, node);
      if (press_x >= skin_node->x
	  && press_y >= skin_node->y
	  && press_x < skin_node->x + skin_node->type->width
	  && press_y < skin_node->y + skin_node->type->height)
	{
	  return node;
	}
    }
  return NULL;
}
static int line_distance(int x, int y, int x1, int y1, int x2, int y2)
{
  int x10 = x1 - x;
  int y10 = y1 - y;
  int x21 = x2 - x1;
  int y21 = y2 - y1;
  int denom = x21 * x21 + y21 * y21;
  if (denom <= 0)
    {
      return (int) hypot (x10, y10);
    }
  else
    {
      gdouble t;
      gdouble xfac, yfac;
      t = (gdouble) (-x10 * x21 - y10 * y21) / denom;
      t = CLAMP (t, 0.0, 1.0);
      xfac = x10 + t * x21;
      yfac = y10 + t * y21;
      return (int) hypot (xfac, yfac);
    }
}
static int pipe_distance(GdamMiniNetworkSkin* skin,
                         GdamMiniNetworkPipe* pipe,
			 int                  x,
			 int                  y)
{
  SkinNode *node_source;
  SkinNode *node_dest;
  int x1, y1, y2, x2;
  node_source = g_hash_table_lookup (skin->node_table, pipe->source);
  node_dest = g_hash_table_lookup (skin->node_table, pipe->destination);
  g_return_val_if_fail (node_source != NULL, -1);
  g_return_val_if_fail (node_dest != NULL, -1);

  if (!lookup_input_offset (node_dest->type, pipe->input_name, &x1, &y1))
    {
      g_warning ("couldn't get x/y offset of input %s", pipe->input_name);
    }
  get_output_offset (node_source->type, &x2, &y2);
  x1 += node_source->x;
  y1 += node_source->y;
  x2 += node_dest->x;
  y2 += node_dest->y;
  return line_distance (x, y, x1, y1, x2, y2);
}
static GdamMiniNetworkPipe*
gdam_mini_network_skin_find_pipe(GdamMiniNetworkSkin* skin,
                                 int                  press_x,
				 int                  press_y)
{
  GdamMiniNetwork *network;
  GSList *list;
  int min_dist = 1000000;
  GdamMiniNetworkPipe *best_pipe = NULL;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  for (list = network->pipes; list != NULL; list = list->next)
    {
      int dist;
      GdamMiniNetworkPipe *pipe = list->data;
      dist = pipe_distance (skin, pipe, press_x, press_y);
      if (dist < 0)
	continue;
      if (min_dist > dist)
	{
	  min_dist = dist;
	  best_pipe = pipe;
	}
    }
  if (min_dist > 100000)
    return NULL;
  return best_pipe;
}
static const char* gdam_mini_network_skin_find_connector
					(GdamMiniNetworkSkin* skin,
					 GdamMiniNetworkNode* node,
					 int                  x,
					 int                  y)
{
  SkinNode *skin_node;
  SkinNodeType *type;
  int i;
  g_return_val_if_fail (node != NULL, NULL);
  skin_node = g_hash_table_lookup (skin->node_table, node);
  g_return_val_if_fail (skin_node != NULL, NULL);
  type = skin_node->type;
  x -= skin_node->x;
  y -= skin_node->y;
  for (i = 0; i < type->num_input_types; i++)
    {
      GdkRectangle *offset = type->input_areas + i;
      if (x >= offset->x
	  && y >= offset->y
	  && x < offset->x + offset->width && y < offset->y + offset->height)
	return type->input_names[i];
    }
  if (x >= type->output_area.x
      && y >= type->output_area.y
      && x < type->output_area.x + type->output_area.width
      && y < type->output_area.y + type->output_area.height)
    return "output";
  return NULL;
}

static void alloc_colors(GdamMiniNetworkSkin* skin)
{
  int state = GTK_WIDGET_STATE (skin->drawing_area);
  GdkGC *gc = gdk_gc_new (skin->backing_pixmap);
  gdk_gc_set_foreground (gc, util_alloc_color_cached (65535, 0, 0));
  skin->background_gc = skin->drawing_area->style->bg_gc[state];
  skin->line_gc = skin->drawing_area->style->fg_gc[state];
}

static void copy_backing_area(GdamMiniNetworkSkin* skin, const GdkRectangle* area)
{
  gdk_draw_pixmap (skin->drawing_area->window,
		   skin->drawing_area->
		   style->fg_gc[GTK_WIDGET_STATE (skin->drawing_area)],
		   skin->backing_pixmap, area->x, area->y, area->x, area->y,
		   area->width, area->height);
}
static gboolean lookup_input_offset(SkinNodeType* type, const char* name,
                                    int* x_out, int* y_out)
{
  int i;
  for (i = 0; i < type->num_input_types; i++)
    if (strcmp (name, type->input_names[i]) == 0)
      {
	int x = type->input_areas[i].x;
	int y = type->input_areas[i].y;
	int width = type->input_areas[i].width;
	int height = type->input_areas[i].height;
	*x_out = x + width / 2;
	*y_out = y + height / 2;
	return TRUE;
      }
  *x_out = *y_out = 0;
  return FALSE;
}
static void get_output_offset(SkinNodeType* type, int* x_out, int* y_out)
{
  *x_out = type->output_area.x + type->output_area.width / 2;
  *y_out = type->output_area.y + type->output_area.height / 2;
}

static void redraw_area(GdamMiniNetworkSkin* skin, 
                        GdkRectangle*        rectangle)
{
  GSList *list;
  GdkPixmap *backing_area = skin->backing_pixmap;
  GdamMiniNetwork *network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  GdkRectangle hack;
  GdkGC *gc = skin->line_gc;
  GdkSegment *segments;
  int num_segments;

  if (network->freeze_count > 0)
    return;

  if (!skin->has_io_nodes)
    {
      create_input_output_nodes (skin);
    }
  if (gc == NULL)
    alloc_colors (skin);
  if (gc == NULL)
    return;

  if (rectangle == NULL)
    {
      rectangle = &hack;
      hack.x = hack.y = 0;
      hack.width = skin->draw_width;
      hack.height = skin->draw_height;
    }

  gdk_draw_rectangle (backing_area,
		      skin->background_gc,
		      TRUE,
		      rectangle->x,
		      rectangle->y, rectangle->width, rectangle->height);


  for (list = network->nodes; list != NULL; list = list->next)
    {
      GdamMiniNetworkNode *node = list->data;
      SkinNode *skin_node;
      SkinNodeType *node_type;
      skin_node = g_hash_table_lookup (skin->node_table, node);
      g_return_if_fail (skin_node != NULL);
      node_type = skin_node->type;
      /* XXX: do clipping!!! */
      /*gdk_draw_rectangle(backing_area,
         skin->line_gc,
         FALSE,
         skin_node->x, skin_node->y,
         node_type->width, node_type->height); */
      gdk_gc_set_clip_origin (gc, skin_node->x, skin_node->y);
      gdk_gc_set_clip_mask (gc, node_type->pixmap_mask);
      gdk_draw_pixmap (backing_area,
		       gc,
		       node_type->pixmap,
		       0, 0,
		       skin_node->x, skin_node->y,
		       node_type->width, node_type->height);
      gdk_gc_set_clip_origin (gc, 0, 0);
      gdk_gc_set_clip_mask (gc, NULL);
    }
  segments = alloca (sizeof (GdkSegment) * g_slist_length (network->pipes));
  num_segments = 0;
  for (list = network->pipes; list != NULL; list = list->next)
    {
      GdamMiniNetworkPipe *pipe = list->data;
      GdamMiniNetworkNode *source = pipe->source;
      GdamMiniNetworkNode *dest = pipe->destination;
      int x_in, y_in, x_out, y_out;
      SkinNode *skin_dest;
      SkinNode *skin_source;
      skin_source = g_hash_table_lookup (skin->node_table, source);
      skin_dest = g_hash_table_lookup (skin->node_table, dest);
      g_return_if_fail (skin_source != NULL);
      g_return_if_fail (skin_dest != NULL);
      if (!lookup_input_offset (skin_dest->type, pipe->input_name,
				&x_in, &y_in))
	g_warning ("couldn't get x/y offset of input %s", pipe->input_name);
      get_output_offset (skin_source->type, &x_out, &y_out);
      x_in += skin_dest->x;
      y_in += skin_dest->y;
      x_out += skin_source->x;
      y_out += skin_source->y;
      segments[num_segments].x1 = x_in;
      segments[num_segments].y1 = y_in;
      segments[num_segments].x2 = x_out;
      segments[num_segments].y2 = y_out;
      num_segments++;
    }
  gdk_draw_segments (backing_area, gc, segments, num_segments);

  if (skin->pipe_build_node != NULL)
    {
      gdk_draw_line (backing_area,
		     skin->line_gc,
		     skin->rubber_band_x,
		     skin->rubber_band_y,
		     skin->con_start_x, skin->con_start_y);
    }

  copy_backing_area (skin, rectangle);
}
static gboolean find_connector_center(GdamMiniNetworkSkin*  skin,
                                      GdamMiniNetworkNode*  node,
				      const char*           input_name,
		                      int*                  x_out,
		                      int*                  y_out)
{
  int x, y;
  int x_offset, y_offset;
  SkinNode *skin_node;

  *x_out = *y_out = 0;

  skin_node = g_hash_table_lookup (skin->node_table, node);
  g_return_val_if_fail (skin_node != NULL, FALSE);
  x = skin_node->x;
  y = skin_node->y;

  if (strcmp (input_name, "output") != 0)
    {
      if (!lookup_input_offset (skin_node->type, input_name,
				&x_offset, &y_offset))
	return FALSE;
    }
  else
    {
      get_output_offset (skin_node->type, &x_offset, &y_offset);
    }

  *x_out = x + x_offset;
  *y_out = y + y_offset;

  return TRUE;
}

static gint
mini_network_expose(GtkDrawingArea* area,
             GdkEventExpose* expose_event,
             GdamMiniNetworkSkin*  skin)
{
  redraw_area (skin, &expose_event->area);
  return 0;
}

static void queue_redraw(GdamMiniNetworkSkin* skin)
{
  GtkWidget *widget = skin->drawing_area;
  if (widget == NULL)
    return;
  gtk_widget_queue_draw (widget);
}

static gint
mini_network_button_press(GtkDrawingArea* area,
                   GdkEventButton* button_event,
		   GdamMiniNetworkSkin* skin)
{
  int press_x = button_event->x;
  int press_y = button_event->y;
  GdamMiniNetworkNode *node;
  const char *input_name;

  if (skin->pipe_build_node != NULL)
    return 0;

  if (skin->dragging != NULL)
    {
      if (button_event->button == 3)
	{
	  gdam_mini_network_skin_remove_node (skin, skin->dragging);
	  g_return_val_if_fail (skin->dragging != NULL, 0);
	}
      if (button_event->button == 2)
	{
	  gdam_mini_network_skin_gentle_remove_node (skin, skin->dragging);
	  g_return_val_if_fail (skin->dragging != NULL, 0);
	}
      return 0;
    }

  /* When the middle button is pressed, try to divide the pipe. */

  skin->drag_start_x = press_x;
  skin->drag_start_y = press_y;

  node = gdam_mini_network_skin_find_node (skin, press_x, press_y);
  if (node == NULL)
    {
      //g_message("was not in node");
      if (button_event->button == 2 || button_event->button == 3)
	{
	  GdamMiniNetworkPipe *pipe;
	  pipe = gdam_mini_network_skin_find_pipe (skin, press_x, press_y);
	  if (pipe == NULL)
	    return 0;
	  if (button_event->button == 3)
	    {
	      gdam_mini_network_skin_remove_pipe (skin, pipe);
	      return 0;
	    }
	  gdam_mini_network_skin_divide_pipe (skin,
					      pipe,
					      press_x, press_y,
					      skin->default_type, NULL, NULL);
	  return 0;
	}
      /* Create new node. */
      node = gdam_mini_network_skin_create_node
	(skin, press_x, press_y, skin->default_type, NULL);

      return 0;
    }
  if (button_event->button == 3)
    {
      gdam_mini_network_skin_remove_node (skin, node);
      return 0;
    }
  if (button_event->button == 2)
    {
      gdam_mini_network_skin_gentle_remove_node (skin, node);
      return 0;
    }
  /*g_message("was in node %s",node->node_type->type); */
  input_name = gdam_mini_network_skin_find_connector (skin, node,
						      press_x, press_y);
  /*g_message("input name %s",input_name); */
  if (input_name == NULL)
    {
      /* Drag element */
      skin->dragging = node;
      skin->drag_start_x = press_x;
      skin->drag_start_y = press_y;
      return 0;
    }
  else
    {
      /* Beginning forming connection. */
      skin->pipe_build_node = node;
      skin->pipe_build_input = input_name;
      skin->rubber_band_x = press_x;
      skin->rubber_band_y = press_y;
      find_connector_center (skin, node, input_name,
			     &skin->con_start_x, &skin->con_start_y);
      return 0;
    }

  return 0;
}

/* Do a "build_pipe" if the node's match up input/output-wise. */
static gboolean do_symmetric_connect(GdamMiniNetworkSkin* skin,
                                     GdamMiniNetworkNode* node_1,
                                     const char*          name_1,
                                     GdamMiniNetworkNode* node_2,
                                     const char*          name_2)
{
  /* Arrange the nodes so that node_1 is the output (ie dest)
   * always. */
  if (strcmp (name_1, "output") == 0)
    {
      if (strcmp (name_2, "output") == 0)
	{
	  g_message ("You cannot connect two outputs");
	  return FALSE;
	}
    }
  else
    {
      GdamMiniNetworkNode *tmp_node;
      const char *tmp_name;
      if (strcmp (name_2, "output") != 0)
	{
	  g_message ("You cannot connect two inputs");
	  return FALSE;
	}

      tmp_node = node_1;
      tmp_name = name_1;
      node_1 = node_2;
      name_1 = name_2;
      node_2 = tmp_node;
      name_2 = tmp_name;
    }
  /*g_message("connecting %s:%s to %s:%s",
     node_1->node_type->type, name_1,
     node_2->node_type->type, name_2); */
  if (!gdam_mini_network_skin_create_pipe (skin, node_1, node_2, name_2))
    return FALSE;
  return TRUE;
}

static gint
mini_network_button_release(GtkDrawingArea* drawing_area,
                            GdkEventButton* button_event,
		            GdamMiniNetworkSkin* skin)
{
  int x = button_event->x;
  int y = button_event->y;
  int delta_x = x - skin->drag_start_x;
  int delta_y = y - skin->drag_start_y;
  GdkRectangle area;

  if (skin->dragging == NULL && skin->pipe_build_node == NULL)
    return 0;

  if (delta_x == 0 && delta_y == 0)
    {
      GdamMiniNetworkNode *node = skin->dragging;
      skin->dragging = NULL;
      if (node == NULL)
	{
	  node = skin->pipe_build_node;
	  skin->pipe_build_node = NULL;
	  skin->pipe_build_input = NULL;
	}
      if (node != NULL)
	gdam_mini_network_skin_make_dialog (skin, node);
      return 0;
    }
  if (skin->dragging)
    {
      /* Move node. */
      gdam_mini_network_skin_move_node (skin,
					skin->dragging, delta_x, delta_y);
      skin->dragging = NULL;
    }
  area.x = MIN (skin->con_start_x, skin->rubber_band_x);
  area.width = MAX (skin->con_start_x, skin->rubber_band_x) - area.x + 1;
  area.y = MIN (skin->con_start_y, skin->rubber_band_y);
  area.height = MAX (skin->con_start_y, skin->rubber_band_y) - area.y + 1;

  if (skin->pipe_build_node != NULL)
    {
      GdamMiniNetworkNode *node = skin->pipe_build_node;
      GdamMiniNetworkNode *node_2;
      const char *input_name_1 = skin->pipe_build_input;
      const char *input_name_2;
      /* Try building pipe. */
      skin->pipe_build_node = NULL;
      skin->pipe_build_input = NULL;

      /* Find new node / input. */
      node_2 = gdam_mini_network_skin_find_node (skin, x, y);
      if (node_2 == NULL)
	{
	  gdam_debug ("wasn't released on a node");
	  redraw_area (skin, &area);
	  return 0;
	}
      input_name_2 = gdam_mini_network_skin_find_connector
	(skin, node_2, x, y);
      if (input_name_2 == NULL)
	{
	  gdam_debug ("wasn't released on a node-input or output");
	  redraw_area (skin, &area);
	  return 0;
	}
      if (!do_symmetric_connect (skin,
				 node, input_name_1, node_2, input_name_2))
	{
	  g_message ("couldn't connect pipe");
	  redraw_area (skin, &area);
	  return 0;
	}
    }
  else
    {
      redraw_area (skin, &area);
      return 0;
    }
  queue_redraw (skin);
  return 0;
}

#define MIN3(x,y,z)   (((x) < (y)) ? MIN(x,z) : MIN(y,z))
#define MAX3(x,y,z)   (((x) > (y)) ? MAX(x,z) : MAX(y,z))

static gint
mini_network_button_motion(GtkDrawingArea* drawing_area,
                     GdkEventMotion* motion_event,
		     GdamMiniNetworkSkin* skin)
{
  int x, y;
  int x1, y1, x2, y2;
  guint state;
  GdkRectangle area;

  if (skin->dragging == NULL && skin->pipe_build_node == NULL)
    return 0;

  if (motion_event->is_hint)
    gdk_window_get_pointer (motion_event->window, &x, &y, &state);
  else
    {
      x = motion_event->x;
      y = motion_event->y;
      state = motion_event->state;
    }

  if (skin->dragging)
    {
      int x = motion_event->x - skin->drag_start_x;
      int y = motion_event->y - skin->drag_start_y;
      skin->drag_start_x = motion_event->x;
      skin->drag_start_y = motion_event->y;
      gdam_mini_network_skin_move_node (skin, skin->dragging, x, y);
      return 0;
    }
  x1 = skin->con_start_x;
  y1 = skin->con_start_y;
  x2 = skin->rubber_band_x;
  y2 = skin->rubber_band_y;
  area.x = MIN3 (x1, x2, motion_event->x);
  area.width = MAX3 (x1, x2, motion_event->x) - area.x + 1;
  area.y = MIN3 (y1, y2, motion_event->y);
  area.height = MAX3 (y1, y2, motion_event->y) - area.y + 1;

  skin->rubber_band_x = motion_event->x;
  skin->rubber_band_y = motion_event->y;
  redraw_area (skin, &area);
  return 0;
}

static gint
mini_network_configure(GtkWidget* widget,
                        GdkEventConfigure* configure_event,
		        GdamMiniNetworkSkin* skin)
{
  if (skin->backing_pixmap)
    gdk_pixmap_unref (skin->backing_pixmap);
  skin->backing_pixmap = gdk_pixmap_new (widget->window,
					 widget->allocation.width,
					 widget->allocation.height, -1);
  skin->draw_width = widget->allocation.width;
  skin->draw_height = widget->allocation.height;
  return TRUE;
}


static void 
compute_one_skin_node_type_pixmap(gpointer name, gpointer unused,
                                  gpointer window)
{
  (void) get_skin_node_type ((char *) name, (GdkWindow *) window);
}
                               

static void 
mini_network_realize(GtkWidget* widget)
{
  g_hash_table_foreach (gdam_mini_network_node_type_table,
			compute_one_skin_node_type_pixmap, widget->window);
}

static void connect_drawing_area(GtkDrawingArea*    drawing_area,
                                 GdamMiniNetworkSkin* skin)
{
  skin->drawing_area = GTK_WIDGET (drawing_area);
  gtk_drawing_area_size (GTK_DRAWING_AREA (drawing_area), 800, 300);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "button_press_event",
		      GTK_SIGNAL_FUNC (mini_network_button_press), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "button_release_event",
		      GTK_SIGNAL_FUNC (mini_network_button_release), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "motion_notify_event",
		      GTK_SIGNAL_FUNC (mini_network_button_motion), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "expose_event",
		      GTK_SIGNAL_FUNC (mini_network_expose), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "configure_event",
		      GTK_SIGNAL_FUNC (mini_network_configure), skin);
  gtk_signal_connect (GTK_OBJECT (drawing_area),
		      "realize",
		      GTK_SIGNAL_FUNC (mini_network_realize), skin);
  gtk_widget_show (GTK_WIDGET (drawing_area));
  gtk_widget_add_events (GTK_WIDGET (drawing_area),
			 GDK_EXPOSURE_MASK
			 | GDK_LEAVE_NOTIFY_MASK
			 | GDK_BUTTON_PRESS_MASK
			 | GDK_BUTTON_RELEASE_MASK
			 | GDK_POINTER_MOTION_MASK
			 | GDK_POINTER_MOTION_HINT_MASK);
}


/* 
 *  ____  _    _         _____                 _   _                 
 * / ___|| | _(_)_ __   |  ___|   _ _ __   ___| |_(_) ___  _ __  ___ 
 * \___ \| |/ / | '_ \  | |_ | | | | '_ \ / __| __| |/ _ \| '_ \/ __|
 *  ___) |   <| | | | | |  _|| |_| | | | | (__| |_| | (_) | | | \__ \
 * |____/|_|\_\_|_| |_| |_|   \__,_|_| |_|\___|\__|_|\___/|_| |_|___/
 */


static gboolean gdam_mini_network_drawing_area_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  if (!GTK_IS_DRAWING_AREA (object))
    {
      g_warning ("%s may only be connected to a drawing area",
		 handler_names[0]);
      return TRUE;
    }
  connect_drawing_area (GTK_DRAWING_AREA (object),
			GDAM_MINI_NETWORK_SKIN (skin));
  return TRUE;
}

typedef struct _SetType SetType;
struct _SetType {
	char* name;
	GdamMiniNetworkSkin* skin;
};

static void do_set_type(SetType* set_type)
{
  g_free (set_type->skin->default_type);
  set_type->skin->default_type = g_strdup (set_type->name);
}
static void free_set_type(gpointer ptr)
{
  SetType *type = ptr;
  g_free (type->name);
  g_free (type);
}

static gboolean gdam_mini_network_set_type_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  SetType *set_type;
  if (num_handler_names < 2)
    {
      g_warning ("%s needs 1 argument", handler_names[0]);
      return TRUE;
    }
  set_type = g_new (SetType, 1);
  set_type->skin = GDAM_MINI_NETWORK_SKIN (skin);
  set_type->name = g_strdup (handler_names[1]);
  gtk_signal_connect_full (object,
			   signal_name,
			   GTK_SIGNAL_FUNC (do_set_type),
			   NULL, set_type, free_set_type, TRUE, FALSE);
  return TRUE;
}
static gboolean gdam_mini_network_edit_container_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  GDAM_MINI_NETWORK_SKIN (skin)->edit_container = GTK_CONTAINER (object);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = { 	
	{ "gdam_mini_network_drawing_area",
	   gdam_mini_network_drawing_area_handler },
	{ "gdam_mini_network_set_type",
	   gdam_mini_network_set_type_handler },
	{ "gdam_mini_network_edit_container",
	   gdam_mini_network_edit_container_handler },
};

static void gdam_mini_network_skin_class_init(GdamSkinClass* skin_class)
{
  GdamViewClass *view_class;
  view_class = GDAM_VIEW_CLASS (skin_class);
  skin_class->model_type = GDAM_TYPE_MINI_NETWORK;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
  view_class->synchronize = gdam_mini_network_skin_synchronize;
  view_class->parse_xml = gdam_mini_network_skin_parse_xml;
  view_class->make_xml = gdam_mini_network_skin_make_xml;
}

static SkinNodeType* get_skin_node_type(const char* type,
                                        GdkWindow*  window)
{
  static GHashTable *skin_types = NULL;
  GdamMiniNetworkNodeType *node_type;
  GdamMiniNetworkInputType *input_types;
  SkinNodeType *rv;
  int num_inputs, i;
  const char *fname;
  PixmapInfo *pixmap_info;
  if (skin_types == NULL)
    skin_types = g_hash_table_new (g_str_hash, g_str_equal);
  rv = g_hash_table_lookup (skin_types, (char *) type);
  if (rv != NULL)
    {
      if (rv->pixmap != NULL)
	return rv;
    }

  node_type = gdam_mini_network_get_node_type (type);
  if (node_type == NULL)
    {
      g_warning ("couldn't get mini_network_node typed %s", type);
      return NULL;
    }

  num_inputs = node_type->num_inputs;

  fname = gdam_skin_search_for_file (node_type->pixmap);
  if (fname == NULL)
    {
      g_warning ("error: no image %s for mininetwork", node_type->pixmap);
      return NULL;
    }
  if (window != NULL)
    {
      pixmap_info = pixmap_cache_get_pixmap (window, fname);
      if (pixmap_info == NULL)
	{
	  g_warning ("couldn't find pixmap %s", node_type->pixmap);
	  return NULL;
	}
    }
  else
    {
      pixmap_info = NULL;
    }

  if (rv == NULL)
    rv = g_new0 (SkinNodeType, 1);
  rv->node_type = g_strdup (type);
  if (pixmap_info != NULL)
    {
      rv->pixmap = pixmap_info->pixmap;
      rv->pixmap_mask = pixmap_info->mask;
      //g_message("got pixmap %p [%s]", rv->pixmap, type);
      rv->width = pixmap_info->width;
      rv->height = pixmap_info->height;
      rv->num_input_types = num_inputs;
    }

  rv->input_areas = g_new (GdkRectangle, num_inputs);
  rv->input_names = g_new (char *, num_inputs);
  input_types = node_type->input_types;
  for (i = 0; i < num_inputs; i++)
    {
      rv->input_areas[i].x = input_types[i].x;
      rv->input_areas[i].y = input_types[i].y;
      rv->input_areas[i].width = input_types[i].width;
      rv->input_areas[i].height = input_types[i].height;
      rv->input_names[i] = g_strdup (input_types[i].name);
    }
  rv->output_area.x = node_type->output_area_x;
  rv->output_area.y = node_type->output_area_y;
  rv->output_area.width = node_type->output_area_width;
  rv->output_area.height = node_type->output_area_height;
  g_hash_table_insert (skin_types, rv->node_type, rv);
  return rv;
}

/*
 * All the functions which modify the model/gui and public --
 * the ui is implemented in terms of them.
 */

GdamMiniNetworkNode* gdam_mini_network_skin_create_node
                                (GdamMiniNetworkSkin* skin,
				 int                  x,
				 int                  y,
				 const char*          type,
				 const char*          options)
{
  GdamMiniNetwork *network;
  GdamMiniNetworkNode *node;
  SkinNodeType *skin_node_type;
  SkinNode *skin_node;
  GtkWidget *main_widget = GDAM_SKIN (skin)->main_widget;
  GdkRectangle redraw;
  g_return_val_if_fail (main_widget != NULL, NULL);
  g_return_val_if_fail (main_widget->window != NULL, NULL);
  if (type == NULL)
    {
      g_message ("You must select a type of node to place");
      return NULL;
    }
  g_return_val_if_fail (type != NULL, NULL);
  skin_node_type = get_skin_node_type (type, main_widget->window);
  g_return_val_if_fail (skin_node_type != NULL, NULL);
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  node = gdam_mini_network_new_node (network, type, options);
  skin_node = g_new (SkinNode, 1);
  skin_node->type = skin_node_type;
  skin_node->x = x - skin_node_type->width / 2;
  skin_node->y = y - skin_node_type->height / 2;
  g_hash_table_insert (skin->node_table, node, skin_node);
  redraw.x = skin_node->x;
  redraw.y = skin_node->y;
  redraw.width = skin_node_type->width;
  redraw.height = skin_node_type->height;
  redraw_area (skin, &redraw);
  return node;
}
GdamMiniNetworkPipe* gdam_mini_network_skin_create_pipe
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* input,
                                 GdamMiniNetworkNode* output,
                                 const char*          name)
{
  GdamMiniNetworkPipe *pipe;
  GdamMiniNetwork *network;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  pipe = gdam_mini_network_new_pipe (network, input, output, name);
  redraw_area (skin, NULL);
  return pipe;
}


void gdam_mini_network_skin_move_node
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node,
			         int                  relative_x,
			         int                  relative_y)
{
  SkinNode *skin_node;
  skin_node = g_hash_table_lookup (skin->node_table, node);
  g_return_if_fail (skin_node != NULL);
  skin_node->x += relative_x;
  skin_node->y += relative_y;
  redraw_area (skin, NULL);
}

void gdam_mini_network_skin_remove_pipe
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkPipe* pipe)
{
  GdamMiniNetwork *network;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  gdam_mini_network_remove_pipe (network, pipe);
  redraw_area (skin, NULL);
}

void gdam_mini_network_skin_remove_node
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node)
{
  gpointer skin_node;
  GdamMiniNetwork *network;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  if (network->input == node || network->output == node)
    {
      g_message ("cannot remove input or output nodes");
      return;
    }
  if (skin->controlled_node == node)
    {
      gtk_widget_destroy (skin->control_widget);
      skin->control_widget = NULL;
      skin->controlled_node = NULL;
    }
  skin_node = g_hash_table_lookup (skin->node_table, node);
  g_return_if_fail (skin_node != NULL);
  g_free (skin_node);
  g_hash_table_remove (skin->node_table, node);
  gdam_mini_network_remove_node (network, node);
  redraw_area (skin, NULL);
}

void gdam_mini_network_skin_divide_pipe
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkPipe* pipe,
				 int                  x,
				 int                  y,
				 const char*          type,
				 const char*          options,
				 const char*          input_name)
{
  GdamMiniNetworkNode *source;
  GdamMiniNetworkNode *destination;
  GdamMiniNetworkNode *new_node;
  char *input_name2;
  if (pipe == NULL)
    {
      pipe = gdam_mini_network_skin_find_pipe (skin, x, y);
    }
  if (pipe == NULL)
    {
      g_warning ("no pipe to divide");
      return;
    }
  source = pipe->source;
  destination = pipe->destination;
  input_name2 = g_strdup (pipe->input_name);

  gdam_mini_network_skin_freeze (skin);
  new_node = gdam_mini_network_skin_create_node (skin, x, y, type, options);
  if (new_node == NULL)
    {
      gdam_mini_network_skin_thaw (skin);
      g_warning ("couldn't create node of type %s", type);
      return;
    }

  if (input_name == NULL)
    {
      if (new_node->node_type->num_inputs == 0)
	{
	  g_warning ("couldn't divide pipe because `%s' has "
		     "no inputs", type);
	  gdam_mini_network_skin_thaw (skin);
	  g_free (input_name2);
	  return;
	}
      input_name = new_node->node_type->input_types[0].name;
    }
  gdam_mini_network_skin_remove_pipe (skin, pipe);
  gdam_mini_network_skin_create_pipe (skin, new_node, destination,
				      input_name);
  gdam_mini_network_skin_create_pipe (skin, source, new_node, input_name2);
  gdam_mini_network_skin_thaw (skin);
  g_free (input_name2);
}

void gdam_mini_network_skin_gentle_remove_node
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node)
{
  /* List of source nodes going into `node' */
  GSList *input_list = NULL;

  /* List of destination nodes going out of `node'. */
  GSList *output_list = NULL;
  GSList *input_name_list = NULL;

  GSList *pipes_to_remove = NULL;

  GdamMiniNetwork *network;
  GSList *list;

  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);

  gdam_mini_network_skin_freeze (skin);
  for (list = network->pipes; list != NULL; list = list->next)
    {
      GdamMiniNetworkPipe *pipe = list->data;
      gboolean remove_pipe = FALSE;
      if (pipe->source == node)
	{
	  output_list = g_slist_prepend (output_list, pipe->destination);
	  input_name_list
	    = g_slist_prepend (input_name_list, g_strdup (pipe->input_name));
	  g_message ("found inbound pipe");
	  remove_pipe = TRUE;
	}
      if (pipe->destination == node)
	{
	  input_list = g_slist_prepend (input_list, pipe->source);
	  g_message ("found outbound pipe");
	  remove_pipe = TRUE;
	}
      if (remove_pipe)
	pipes_to_remove = g_slist_prepend (pipes_to_remove, pipe);
    }
  while (pipes_to_remove != NULL)
    {
      GdamMiniNetworkPipe *pipe = pipes_to_remove->data;
      gdam_mini_network_skin_remove_pipe (skin, pipe);
      pipes_to_remove = g_slist_remove (pipes_to_remove, pipe);
    }
  gdam_mini_network_skin_remove_node (skin, node);

  /* For each input/output pair try to connect the pipe.
   * If it fails, probably the output can only handle
   * one connection -- just skip to the next output. */
  while (output_list != NULL)
    {
      GdamMiniNetworkNode *output_node = output_list->data;
      char *input_name = input_name_list->data;
      output_list = g_slist_remove (output_list, output_node);
      input_name_list = g_slist_remove (input_name_list, input_name);

      for (list = input_list; list != NULL; list = list->next)
	{
	  GdamMiniNetworkNode *input_node = list->data;
	  GdamMiniNetworkPipe *pipe;
	  pipe = gdam_mini_network_skin_create_pipe
	    (skin, input_node, output_node, input_name);
	  if (pipe == NULL)
	    break;
	}
      g_free (input_name);
    }
  gdam_mini_network_skin_thaw (skin);
  g_slist_free (input_list);
}

void gdam_mini_network_skin_freeze(GdamMiniNetworkSkin* skin)
{
  GdamMiniNetwork *network;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  gdam_mini_network_freeze (network);
}
void gdam_mini_network_skin_thaw(GdamMiniNetworkSkin* skin)
{
  GdamMiniNetwork *network;
  gboolean must_redraw;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  must_redraw = (network->freeze_count == 1);
  gdam_mini_network_thaw (network);
  if (must_redraw)
    redraw_area (skin, NULL);
}


typedef struct _ParamChangeData ParamChangeData;
struct _ParamChangeData {
	gdouble scale;
	GdamMiniNetwork* network;
	GdamMiniNetworkNode* node;
	GdamMiniNetworkParam* param;
	GtkAdjustment* adjustment;
	GtkLabel* label;

	gboolean logarithmic;
	gboolean ignore_adj_changes;
};
	
static void do_param_change(GtkAdjustment* adjustment, ParamChangeData* data)
{
  gdouble value = adjustment->value;
  GdamMiniNetworkParamValue *param_value;
  char *options;
  gdouble scale = data->scale;

  if (data->ignore_adj_changes)
    return;

  if (data->logarithmic)
    {
      value = exp (value);
    }

  param_value = &data->node->param_values[data->param->param_index];
  param_value->double_value = value * scale;

  options = gdam_mini_network_make_param_string (data->node);
  gdam_mini_network_configure_node (data->network, data->node, options);
  g_free (options);

  {
    char buf[256];
    int len;
    sprintf (buf, "%6.6g", value);
    len = strlen (buf);
    if (len < 10)
      {
	while (len < 10)
	  buf[len++] = ' ';
	buf[len] = '\0';
      }
    gtk_label_set_text (data->label, buf);
  }
}
static int find_mode_index(ParamChangeData* data, const char* text)
{
  GdamMiniNetworkParam *param = data->param;
  int i;
  for (i = 0; i < param->num_modes; i++)
    if (strcmp (param->modes[i].name, text) == 0)
      return i;
  g_warning ("couldn't find mode %s", text);
  return 0;
}
static void change_param_mode(GtkEntry* entry, ParamChangeData* data)
{
  gdouble new_value;
  GdamMiniNetworkParamValue *param_value;
  int mode_index;
  int param_index = data->param->param_index;
  mode_index = find_mode_index (data, gtk_entry_get_text (entry));
  data->scale = data->param->modes[mode_index].scale;

  data->node->modes[param_index] = mode_index;
  data->logarithmic = data->param->modes[mode_index].logarithmic;

  param_value = &data->node->param_values[param_index];
  new_value = param_value->double_value / data->scale;
  data->adjustment->value = new_value;
  data->adjustment->upper = data->param->modes[mode_index].max_value;
  data->adjustment->lower = data->param->modes[mode_index].min_value;
  if (data->logarithmic)
    {
      data->adjustment->value = log (data->adjustment->value);
      data->adjustment->upper = log (data->adjustment->upper);
      data->adjustment->lower = log (data->adjustment->lower);
    }
  gtk_adjustment_changed (data->adjustment);
}

/* Make a subwidget that uses a hscrollbar to control an adjustment
 * style value... [type in param, value in node] */
static GtkWidget* make_range_subwidget(GdamMiniNetworkSkin* skin,
                                       GdamMiniNetworkNode* node,
				       GdamMiniNetworkParam* param)
{
  GtkWidget *rv;
  GtkWidget *widget;
  GtkBox *lower_hbox;
  GtkBox *upper_hbox;
  GdamMiniNetwork *network;
  ParamChangeData *param_change_data;
  GtkAdjustment *adjustment;
  gdouble cur_value;
  gdouble scale = 1.0;
  int param_index = param->param_index;
  gboolean logarithmic = FALSE;
  GdamMiniNetworkParamMode *mode = NULL;
  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  upper_hbox = GTK_BOX (gtk_hbox_new (FALSE, 1));
  widget = gtk_label_new (param->name);
  gtk_widget_show (widget);
  gtk_box_pack_start (upper_hbox, widget, FALSE, FALSE, 0);

  if (param->num_modes > 0)
    {
      mode = param->modes + node->modes[param_index];
      logarithmic = mode->logarithmic;
      scale = mode->scale;
      lower_hbox = GTK_BOX (gtk_hbox_new (FALSE, 1));
      gtk_widget_show (GTK_WIDGET (lower_hbox));
      rv = gtk_vbox_new (FALSE, 1);
      gtk_widget_show (rv);
      gtk_box_pack_start_defaults (GTK_BOX (rv), GTK_WIDGET (upper_hbox));
      gtk_box_pack_start_defaults (GTK_BOX (rv), GTK_WIDGET (lower_hbox));
    }
  else
    {
      lower_hbox = upper_hbox;
      rv = GTK_WIDGET (lower_hbox);
    }

  cur_value = node->param_values[param_index].double_value / scale;
  if (mode)
    {
      gdouble min_val = mode->min_value;
      gdouble max_val = mode->max_value;
      gdouble val = cur_value;
      if (logarithmic)
	{
	  min_val = log (min_val);
	  max_val = log (max_val);
	  val = log (val);
	}
      adjustment = (GtkAdjustment *)
	gtk_adjustment_new (val, min_val, max_val + 0.01, 0.01, 0.1, 0.01);
    }
  else
    {
      adjustment = (GtkAdjustment *)
	gtk_adjustment_new (cur_value,
			    param->min_value,
			    param->max_value + 0.01, 0.01, 0.1, 0.01);
    }
  param_change_data = g_new0 (ParamChangeData, 1);
  param_change_data->network = network;
  param_change_data->param = param;
  param_change_data->node = node;
  param_change_data->logarithmic = logarithmic;
  //param_change_data->mode_index = node->modes[param_index];
  param_change_data->scale = scale;
  param_change_data->adjustment = adjustment;
  widget = gtk_hscrollbar_new (adjustment);
  gtk_widget_show (widget);
  gtk_box_pack_start (lower_hbox, widget, TRUE, TRUE, 0);

  {
    char buf[128];
    GtkWidget *label;
    snprintf (buf, sizeof (buf), "%6.6g", cur_value);
    label = gtk_label_new (buf);
    param_change_data->label = (GtkLabel *) label;
    gtk_widget_show (label);
    gtk_box_pack_start_defaults (upper_hbox, label);
  }

  if (param->num_modes > 0)
    {
      GtkCombo *combo;
      GList *list = NULL;
      int i;
      int mode_index = node->modes[param_index];
      for (i = 0; i < param->num_modes; i++)
	list = g_list_append (list, param->modes[i].name);
      combo = (GtkCombo *) gtk_combo_new ();
      gtk_combo_set_popdown_strings (combo, list);
      gtk_combo_set_value_in_list (combo, mode_index, FALSE);
      gtk_entry_set_text (GTK_ENTRY (combo->entry),
			  param->modes[mode_index].name);
      /* XXX: should free list? */
      gtk_signal_connect (GTK_OBJECT (combo->entry),
			  "changed",
			  GTK_SIGNAL_FUNC (change_param_mode),
			  param_change_data);
      gtk_widget_show (GTK_WIDGET (combo));
      gtk_box_pack_start (upper_hbox, GTK_WIDGET (combo), FALSE, FALSE, 0);
    }

  gtk_signal_connect_full (GTK_OBJECT (adjustment),
			   "value_changed",
			   GTK_SIGNAL_FUNC (do_param_change),
			   NULL, param_change_data, g_free, FALSE, FALSE);
  gtk_widget_show (GTK_WIDGET (upper_hbox));
  return GTK_WIDGET (rv);
}

/*
 *                 generic_range_value
 */

static void recompute_range_value(ParamChangeData* param_change_data)
{
  GdamMiniNetworkParamValue *param_value;
  int param_index = param_change_data->param->param_index;
  gfloat value;
  gfloat min_value, max_value;
  gboolean is_logarithmic = FALSE;
  gfloat normalized_value;
  param_value = &param_change_data->node->param_values[param_index];

  param_change_data->ignore_adj_changes = TRUE;
  min_value = param_value->generic_range_value.min_value;
  max_value = param_value->generic_range_value.max_value;
  value = param_value->generic_range_value.value;

  if (param_value->generic_range_value.is_logarithmic)
    {
      if (min_value > 0.0 && max_value > 0.0)
	{
	  if (value <= 0.000001)
	    value = 0.000001;
	  value = log (value);
	  min_value = log (min_value);
	  max_value = log (max_value);
	}
      else
	{
	  g_warning ("when logarithmic, min/max_value must " "be positive");
	  return;
	}
      is_logarithmic = TRUE;
    }
  if (max_value < min_value * 1.00001)
    {
      normalized_value = 0.0;
    }
  else
    {
      normalized_value = (value - min_value) / (max_value - min_value);
    }

  /*
     value = normalized_value * (max_value - min_value)
     + min_value;
     if (is_logarithmic)
     value = exp(value);
     if (param_value->generic_range_value.is_integer)
     value = rint(value); */

  gtk_adjustment_set_value (param_change_data->adjustment, normalized_value);

  param_change_data->ignore_adj_changes = FALSE;
}

static void
generic_range_min_value_changed(GtkEntry* entry,
                                ParamChangeData* param_change_data)
{
  GdamMiniNetworkParamValue *param_value;
  int param_index = param_change_data->param->param_index;
  param_value = &param_change_data->node->param_values[param_index];
  param_value->generic_range_value.min_value
    = atof (gtk_entry_get_text (entry));
  recompute_range_value (param_change_data);
}

static void
generic_range_max_value_changed(GtkEntry* entry,
                                ParamChangeData* param_change_data)
{
  GdamMiniNetworkParamValue *param_value;
  int param_index = param_change_data->param->param_index;
  param_value = &param_change_data->node->param_values[param_index];
  param_value->generic_range_value.max_value
    = atof (gtk_entry_get_text (entry));
  recompute_range_value (param_change_data);
}

static void 
generic_range_is_integer_changed(GtkToggleButton* toggle,
                                 ParamChangeData* param_change_data)
{
  GdamMiniNetworkParamValue *param_value;
  int param_index = param_change_data->param->param_index;
  param_value = &param_change_data->node->param_values[param_index];
  param_value->generic_range_value.is_integer = toggle->active;
  recompute_range_value (param_change_data);
}

static void 
generic_range_is_logarithmic_changed(GtkToggleButton* toggle,
                                     ParamChangeData* param_change_data)
{
  GdamMiniNetworkParamValue *param_value;
  int param_index = param_change_data->param->param_index;
  param_value = &param_change_data->node->param_values[param_index];
  param_value->generic_range_value.is_logarithmic = toggle->active;
  recompute_range_value (param_change_data);
}

static void 
generic_range_value_changed        (GtkAdjustment*   adjustment,
                                    ParamChangeData* data)
{
  char *options;
  GdamMiniNetworkParamValue *param_value;
  int param_index = data->param->param_index;
  gdouble min_value, max_value;
  param_value = &data->node->param_values[param_index];

  if (data->ignore_adj_changes)
    return;

  min_value = param_value->generic_range_value.min_value;
  max_value = param_value->generic_range_value.max_value;
  param_value->generic_range_value.value
    = min_value + (max_value - min_value) * adjustment->value;

  options = gdam_mini_network_make_param_string (data->node);
  gdam_mini_network_configure_node (data->network, data->node, options);
  g_free (options);
}

static GtkWidget* make_generic_range_subwidget(GdamMiniNetworkSkin* skin,
                                               GdamMiniNetworkNode* node,
				               GdamMiniNetworkParam* param)
{
  GtkWidget *rv;
  GtkWidget *widget;
  GtkBox *hbox;
  GdamMiniNetwork *network;
  ParamChangeData *param_change_data;
  GtkAdjustment *adjustment;
  gdouble cur_value;
  gdouble scale = 1.0;
  int param_index = param->param_index;
  gboolean logarithmic = FALSE;
  GdamMiniNetworkParamMode *mode = NULL;
  char tmp_buf[256];
  GdamMiniNetworkParamValue *param_value;

  network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);

  param_change_data = g_new (ParamChangeData, 1);
  param_change_data->network = network;
  param_change_data->param = param;
  param_change_data->node = node;
  param_change_data->logarithmic = logarithmic;
  param_change_data->scale = scale;
  param_change_data->ignore_adj_changes = FALSE;
  param_change_data->label = NULL;

  param_value = node->param_values + param_index;

  rv = gtk_vbox_new (FALSE, 1);
  gtk_widget_show (rv);

  /* Create the top entry in the vbox with just the param name. */
  widget = gtk_label_new (param->name);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (GTK_BOX (rv), widget);

  /* Create the entries for min/max value. */
  hbox = GTK_BOX (gtk_hbox_new (FALSE, 1));
  gtk_widget_show (GTK_WIDGET (hbox));
  gtk_box_pack_start_defaults (GTK_BOX (rv), GTK_WIDGET (hbox));
  widget = gtk_label_new ("Min value:");
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  widget = gtk_entry_new ();
  g_snprintf (tmp_buf, sizeof (tmp_buf),
	      "%.9g", param_value->generic_range_value.min_value);
  gtk_entry_set_text (GTK_ENTRY (widget), tmp_buf);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  gtk_signal_connect (GTK_OBJECT (widget),
		      "changed",
		      GTK_SIGNAL_FUNC (generic_range_min_value_changed),
		      param_change_data);

  widget = gtk_label_new ("Max value:");
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  widget = gtk_entry_new ();
  g_snprintf (tmp_buf, sizeof (tmp_buf),
	      "%.9g", param_value->generic_range_value.max_value);
  gtk_entry_set_text (GTK_ENTRY (widget), tmp_buf);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  gtk_signal_connect (GTK_OBJECT (widget),
		      "changed",
		      GTK_SIGNAL_FUNC (generic_range_max_value_changed),
		      param_change_data);

  /* Create the toggles for `is_integer' and `is_logarithmic'. */
  hbox = GTK_BOX (gtk_hbox_new (FALSE, 1));
  gtk_widget_show (GTK_WIDGET (hbox));
  gtk_box_pack_start_defaults (GTK_BOX (rv), GTK_WIDGET (hbox));
  widget = gtk_label_new ("Is integer:");
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  widget = gtk_toggle_button_new ();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				param_value->generic_range_value.is_integer);
  gtk_widget_show (widget);
  gtk_signal_connect (GTK_OBJECT (widget),
		      "toggled",
		      GTK_SIGNAL_FUNC (generic_range_is_integer_changed),
		      param_change_data);
  gtk_box_pack_start_defaults (hbox, widget);

  widget = gtk_label_new ("Is logarithmic:");
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  widget = gtk_toggle_button_new ();
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
				param_value->generic_range_value.
				is_logarithmic);
  gtk_widget_show (widget);
  gtk_signal_connect (GTK_OBJECT (widget),
		      "toggled",
		      GTK_SIGNAL_FUNC (generic_range_is_logarithmic_changed),
		      param_change_data);
  gtk_box_pack_start_defaults (hbox, widget);


  /* Create the value scrollbar (to play with and set the default value)
   */
  hbox = GTK_BOX (gtk_hbox_new (FALSE, 1));
  gtk_widget_show (GTK_WIDGET (hbox));
  gtk_box_pack_start_defaults (GTK_BOX (rv), GTK_WIDGET (hbox));
  adjustment = (GtkAdjustment *) gtk_adjustment_new
    (0.0, 0.0, 1.005, 0.1, 0.01, 0.005);
  param_change_data->adjustment = adjustment;
  recompute_range_value (param_change_data);
  widget = gtk_hscrollbar_new (adjustment);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (hbox, widget);
  gtk_signal_connect (GTK_OBJECT (adjustment),
		      "value-changed",
		      GTK_SIGNAL_FUNC (generic_range_value_changed),
		      param_change_data);

  return GTK_WIDGET (rv);
}

typedef struct _GenericChangeData GenericChangeData;
struct _GenericChangeData {
	int param_index;
	GdamMiniNetwork* network;
	GdamMiniNetworkNode* node;
};
static void do_buffer_id_change(GenericChangeData* data, int buffer_id)
{
  char *options;
  data->node->param_values[data->param_index].buffer_id_value = buffer_id;
  g_message ("got buffer id %d", buffer_id);
  options = gdam_mini_network_make_param_string (data->node);
  gdam_mini_network_configure_node (data->network, data->node, options);
  g_free (options);
}
static GtkWidget* make_buffer_id_subwidget(GdamMiniNetworkSkin* skin,
                                           GdamMiniNetworkNode* node,
				           GdamMiniNetworkParam* param)
{
  GdamBufferCombo *combo;
  GenericChangeData *data;
  GtkWidget *widget;
  GtkBox *box;
  box = (GtkBox *) gtk_hbox_new (FALSE, 1);
  widget = gtk_label_new (param->name);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (box, widget);
  combo = gdam_buffer_combo_new ();
  gtk_widget_show (GTK_WIDGET (combo));
  gtk_box_pack_start_defaults (box, GTK_WIDGET (combo));
  gtk_widget_show (GTK_WIDGET (box));

  data = g_new (GenericChangeData, 1);
  data->param_index = param->param_index;
  data->network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  data->node = node;
  gtk_signal_connect_full (GTK_OBJECT (combo),
			   "buffer-selected", GTK_SIGNAL_FUNC (do_buffer_id_change), NULL, data, g_free, TRUE,	/* object */
			   FALSE);
  return GTK_WIDGET (box);
}

static void text_changed(GtkEntry* entry, GenericChangeData* data)
{
  GdamMiniNetworkParamValue *value;
  char *options;
  value = &data->node->param_values[data->param_index];
  g_free (value->string_value);
  value->string_value = g_strdup (gtk_entry_get_text (entry));
  options = gdam_mini_network_make_param_string (data->node);
  gdam_mini_network_configure_node (data->network, data->node, options);
  g_free (options);
}

static GtkWidget* make_string_subwidget   (GdamMiniNetworkSkin* skin,
                                           GdamMiniNetworkNode* node,
				           GdamMiniNetworkParam* param)
{
  GtkWidget *widget;
  GtkBox *box;
  GenericChangeData *data;
  const char *signal_name;

  box = (GtkBox *) gtk_hbox_new (FALSE, 1);
  widget = gtk_label_new (param->name);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (box, widget);

  data = g_new (GenericChangeData, 1);
  data->param_index = param->param_index;
  data->network = GDAM_MINI_NETWORK (GDAM_SKIN (skin)->model);
  data->node = node;

  if (param->enter_not_required)
    signal_name = "changed";
  else
    signal_name = "activate";

  widget = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (widget),
		      node->param_values[param->param_index].string_value);
  gtk_widget_show (widget);
  gtk_box_pack_start_defaults (box, widget);
  gtk_signal_connect_full (GTK_OBJECT (widget), signal_name, GTK_SIGNAL_FUNC (text_changed), NULL, data, g_free, FALSE,	/* object */
			   FALSE);
  gtk_widget_show (GTK_WIDGET (box));
  return GTK_WIDGET (box);
}

static GtkWidget* make_subwidget(GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node,
				 GdamMiniNetworkParam* param)
{
  if (param->param_type == GDAM_MINI_NETWORK_RANGE)
    return make_range_subwidget (skin, node, param);
  if (param->param_type == GDAM_MINI_NETWORK_GENERIC_RANGE)
    return make_generic_range_subwidget (skin, node, param);
  if (param->param_type == GDAM_MINI_NETWORK_BUFFER_ID)
    return make_buffer_id_subwidget (skin, node, param);
  if (param->param_type == GDAM_MINI_NETWORK_STRING)
    return make_string_subwidget (skin, node, param);
  g_warning ("don't know how to make widget for param %d", param->param_type);
  return NULL;
}

static GtkWidget* make_widget   (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node)
{
  GdamMiniNetworkNodeType *node_type = node->node_type;
  GtkWidget *widget;
  int packed = 0, i;
  if (node_type->num_params == 0)
    return gtk_label_new ("Nothing to control");

  widget = gtk_vbox_new (FALSE, 1);
  {
    GtkWidget *id_label;
    char buf[255];
    sprintf (buf, "node %d", node->node_id);
    id_label = gtk_label_new (buf);
    gtk_widget_show (id_label);
    gtk_box_pack_start_defaults (GTK_BOX (widget), id_label);
  }
  for (i = 0; i < node_type->num_params; i++)
    {
      GtkWidget *subwidget = make_subwidget (skin, node,
					     node_type->params + i);
      if (subwidget == NULL)
	continue;
      gtk_box_pack_start_defaults (GTK_BOX (widget), subwidget);
      packed++;
    }

  if (packed == 0)
    {
      gtk_object_sink (GTK_OBJECT (widget));
      return gtk_label_new ("Nothing to control");
    }
  return widget;
}

void gdam_mini_network_skin_make_dialog
                                (GdamMiniNetworkSkin* skin,
                                 GdamMiniNetworkNode* node)
{
  GtkContainer *container;
  GtkWidget *control;
  if (skin->control_widget != NULL)
    {
      gtk_widget_destroy (skin->control_widget);
      skin->control_widget = NULL;
    }
  g_return_if_fail (skin->edit_container != NULL);
  skin->controlled_node = node;
  container = skin->edit_container;
  control = make_widget (skin, node);
  gtk_widget_show (control);
  gtk_container_add (container, control);
  skin->control_widget = control;
}
