/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#include "gdamrecorderskin.h"
#include "../../component/plugins/gdamrecorderfilter.h"
#include "../gdamskinutil.h"
#include <gtk/gtklabel.h>
#include <gtk/gtksignal.h>
#include <common/gdamplugin.h>


static void gdam_recorder_skin_class_init(GdamSkinClass* klass);

static void gdam_recorder_skin_init(GdamRecorderSkin* recorder_skin)
{
}

GtkType gdam_recorder_skin_get_type()
{
  static GtkType recorder_skin_type = 0;
  if (!recorder_skin_type)
    {
      static const GtkTypeInfo recorder_skin_info = {
	"GdamRecorderSkin",
	sizeof (GdamRecorderSkin),
	sizeof (GdamRecorderSkinClass),
	(GtkClassInitFunc) gdam_recorder_skin_class_init,
	(GtkObjectInitFunc) gdam_recorder_skin_init,
	/* reserved_1 */ NULL,
	/* reserved_2 */ NULL,
	(GtkClassInitFunc) NULL
      };
      GtkType parent = GDAM_TYPE_SKIN;
      recorder_skin_type = gtk_type_unique (parent, &recorder_skin_info);
      gdam_plugin_check_version (__FILE__, GDAM_VERSION);
    }
  return recorder_skin_type;
}

static void start_recording(GdamSkin* skin)
{
  GdamRecorderFilter *recorder = GDAM_RECORDER_FILTER (skin->model);
  g_message ("start_recording [%p]", skin->model);
  g_return_if_fail (recorder != NULL);
  gdam_recorder_filter_set_recording (recorder, TRUE);
}
static void stop_recording(GdamSkin* skin)
{
  GdamRecorderFilter *recorder = GDAM_RECORDER_FILTER (skin->model);
  g_return_if_fail (recorder != NULL);
  gdam_recorder_filter_set_recording (recorder, FALSE);
}
static void grab_buffer(GdamSkin* skin)
{
  GdamRecorderFilter *recorder = GDAM_RECORDER_FILTER (skin->model);
  g_return_if_fail (recorder != NULL);
  g_message ("grab buffer");
  gdam_recorder_filter_capture_buffer (recorder, TRUE);
}
static void copy_buffer(GdamSkin* skin)
{
  GdamRecorderFilter *recorder = GDAM_RECORDER_FILTER (skin->model);
  g_return_if_fail (recorder != NULL);
  g_message ("copy buffer");
  gdam_recorder_filter_capture_buffer (recorder, FALSE);
}

static void set_length_label(GtkLabel*          label,
                             int                new_length)
{
  char buf[120];
  g_snprintf (buf, sizeof (buf), "%3.3f", (gdouble) new_length / 44100.0);
  gtk_label_set_text (label, buf);
}

static gboolean gdam_recorder_start_recording_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (start_recording),
			     GTK_OBJECT (skin));
  return TRUE;
}
static gboolean gdam_recorder_stop_recording_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (stop_recording),
			     GTK_OBJECT (skin));
  return TRUE;
}
static gboolean gdam_recorder_grab_buffer_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (grab_buffer),
			     GTK_OBJECT (skin));
  return TRUE;
}
static gboolean gdam_recorder_copy_buffer_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  gtk_signal_connect_object (object,
			     signal_name,
			     GTK_SIGNAL_FUNC (copy_buffer),
			     GTK_OBJECT (skin));
  return TRUE;
}
static gboolean gdam_recorder_length_label_handler
				(GdamSkin*          skin,
				 GtkObject*         object,
				 const char*        signal_name,
				 int                num_handler_names,
				 char**             handler_names)
{
  if (!GTK_IS_LABEL (object))
    {
      g_warning ("only label may take `%s'", handler_names[0]);
      return TRUE;
    }
  gtk_signal_connect_object (GTK_OBJECT (skin->model),
			     "new_length",
			     GTK_SIGNAL_FUNC (set_length_label), object);
  return TRUE;
}

static GdamSkinGenericConnect connect_infos[] = {
	{ "gdam_recorder_start_recording",
           gdam_recorder_start_recording_handler	},
	{ "gdam_recorder_stop_recording",
           gdam_recorder_stop_recording_handler		},
	{ "gdam_recorder_grab_buffer",
           gdam_recorder_grab_buffer_handler		},
	{ "gdam_recorder_copy_buffer",
           gdam_recorder_copy_buffer_handler		},
	{ "gdam_recorder_length_label",
           gdam_recorder_length_label_handler		}	};

static void gdam_recorder_skin_class_init(GdamSkinClass* skin_class)
{
  skin_class->model_type = GDAM_TYPE_RECORDER_FILTER;
  gdam_skin_class_register_generic_handlers (skin_class,
					     G_N_ELEMENTS (connect_infos),
					     connect_infos);
}
