/*
    GDAM - Geoff & Dave's Audio Mixer
    Copyright (C) 1999    Dave Benson, Geoff Matters.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

    Contact:
        daveb@ffem.org <Dave Benson>
        geoff@ugcs.caltech.edu <Geoff Matters>
*/
#ifndef __GDAM_SEQUENCER_SKIN_H_
#define __GDAM_SEQUENCER_SKIN_H_


typedef struct _GdamSequencerSkin GdamSequencerSkin;
typedef struct _GdamSequencerSkinClass GdamSequencerSkinClass;

#include "../gdamskin.h"
#include <client/gdamchannel.h>
#include <component/gdamsequencer.h>
#include <common/gdamsample.h>
#include <gtk/gtkentry.h>
#include <gtk/gtklabel.h>
#include <gdk/gdk.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

GtkType gdam_sequencer_skin_get_type();
#define GDAM_TYPE_SEQUENCER_SKIN			(gdam_sequencer_skin_get_type ())
#define GDAM_SEQUENCER_SKIN(obj)              (GTK_CHECK_CAST ((obj), GDAM_TYPE_SEQUENCER_SKIN, GdamSequencerSkin))
#define GDAM_SEQUENCER_SKIN_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GDAM_TYPE_SEQUENCER_SKIN, GdamSequencerSkinClass))
#define GDAM_IS_SEQUENCER_SKIN(obj)           (GTK_CHECK_TYPE ((obj), GDAM_TYPE_SEQUENCER_SKIN))
#define GDAM_IS_SEQUENCER_SKIN_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GDAM_TYPE_SEQUENCER_SKIN))

#define SEQUENCER_NUM_LOCAL_BUFFERS 8

struct _GdamSequencerSkinClass {
	GdamSkinClass		skin_class;
};
struct _GdamSequencerSkin {
	GdamSkin			skin;

	GList*		buffers;

	guint		mixer_id;

	guint		sequence_id;

	/* A selection area. */
	gboolean	has_selection;
	GdkPoint	selection[2];
	GHashTable*     selected_elements;
	int		selection_mode;

	int		display_mode;

	/* The drawing area for the samples... */
	GtkWidget*	drawing_area;
	int		drawing_area_width;
	int		drawing_area_height;

	/* Font for the floating subsamples. */
	GdkFont*	font;

	/* Entry-box for a filename... */
	GtkEntry*	filename_entry;

	/* Rendering information. */
	int		circle_radius;
	GdkGC*		text_gc;
	GdkGC*		circle_gc;
	GdkGC*		circle_boundary_gc;
	GdkGC*		grid_gc;
	GdkGC*		background;
	GdkGC*		selection_gc;

	/* Grid line locations. */
	int		num_beat_grid;
	gdouble*	beat_grid;
	int		num_resample_grid;
	gdouble*	log_resample_grid;
	int		num_gain_grid;
	gdouble*	gain_grid;

	/* the gain/resample of new sources. */
	gdouble  	default_resample, default_gain;

	/* doublebuffering */
	GdkPixmap*	backing_pixmap;

	/* If the user is dragging a subsample around,
	 * that is indicated here. */
	int		num_dragging;
	GdamSequencerElement*	dragging;
	int		drag_start_x, drag_start_y;
	int		drag_current_x, drag_current_y;
	GHashTable*	in_transit;	/* elements which are not drawn,
	                                 * but are played, and will be removed
					 * when the current drag is over. */
	/* Are the dragged elements from a selection buffer? */
	gboolean	was_dragging_selected;

	/* The boundaries of the viewable area. */
	gdouble		min_beat;
	gdouble		max_beat;
	gdouble		min_log_hertz;
	gdouble		max_log_hertz;
	gdouble		min_gain;
	gdouble		max_gain;

	/* These adjustments are for the zooming and panning.
	 * Sigh... we should use a general method for this.... */
	GtkAdjustment*	time_pan_adj;
	GtkAdjustment*	time_zoom_adj;
	GtkAdjustment*	vertical_pan_adj;
	GtkAdjustment*	vertical_zoom_adj;

	/* Buffer id's that are not supposed to be drawn. */
	GHashTable* 	hidden_buffers;

	/* A map from buffer_ids => widget and back
	 * for the box of buttons that allows
	 * select-all-of-type, delete-all-of-type, etc. */
	GtkContainer*	buffer_id_holder;
	GHashTable*	buffer_id_to_info;

	GtkAdjustment* tempo_adj;

	/* Offset from the center of the rendering where the user clicked. */
	int		drag_click_offset_x, drag_click_offset_y;

	/* Height and width of the drawing area. */
	int		draw_width, draw_height;

	/* We keep a set of buffer ids around.  we don't manage the buffers. */
	int		local_buffers[SEQUENCER_NUM_LOCAL_BUFFERS];

	GHashTable*	buffer_info_hash;

	guint		redraw_thread;

	gdouble		asynch_position;
	gdouble		asynch_step_size;
	GtkLabel*	asynch_position_label;
	gboolean	asynch_autostep;
	GtkToggleButton* autostep_toggle_button;
	/* This should be removed if we're destroyed... */
	guint 		input_hook_id;
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif
